#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc ButtonsDef {} {
    ButtonsFileDef
    ButtonsEditDef
    ButtonsViewDef
    ButtonsFrameDef
    ButtonsBinDef
    ButtonsZoomDef
    ButtonsScaleDef
    ButtonsColorDef
    ButtonsRegionDef
    ButtonsWCSDef
    ButtonsHelpDef
}

proc CreateButtons {} {
    global ds9
    global buttons

    set ds9(buttons) [frame $ds9(main).buttons -relief ridge -bd 1]
    set buttons(majorPrev) $ds9(buttons).file
    set buttons(majorCurrent) $ds9(buttons).file

    frame $ds9(buttons).major
    RadioButton $ds9(buttons).major.file \
	[string tolower [msgcat::mc {File}]] \
	buttons(majorCurrent) $ds9(buttons).file MajorButton
    RadioButton $ds9(buttons).major.edit \
	[string tolower [msgcat::mc {Edit}]] \
	buttons(majorCurrent) $ds9(buttons).edit MajorButton
    RadioButton $ds9(buttons).major.view \
	[string tolower [msgcat::mc {View}]] \
	buttons(majorCurrent) $ds9(buttons).view MajorButton
    RadioButton $ds9(buttons).major.frame \
	[string tolower [msgcat::mc {Frame}]] \
	buttons(majorCurrent) $ds9(buttons).frame MajorButton
    RadioButton $ds9(buttons).major.bin \
	[string tolower [msgcat::mc {Bin}]] \
	buttons(majorCurrent) $ds9(buttons).bin MajorButton
    RadioButton $ds9(buttons).major.zoom \
	[string tolower [msgcat::mc {Zoom}]] \
	buttons(majorCurrent) $ds9(buttons).zoom MajorButton
    RadioButton $ds9(buttons).major.scale \
	[string tolower [msgcat::mc {Scale}]] \
	buttons(majorCurrent) $ds9(buttons).scale MajorButton
    RadioButton $ds9(buttons).major.color \
	[string tolower [msgcat::mc {Color}]] \
	buttons(majorCurrent) $ds9(buttons).color MajorButton
    RadioButton $ds9(buttons).major.region \
	[string tolower [msgcat::mc {Region}]] \
	buttons(majorCurrent) $ds9(buttons).region MajorButton
    RadioButton $ds9(buttons).major.wcs \
	[string tolower [msgcat::mc {WCS}]] \
	buttons(majorCurrent) $ds9(buttons).wcs MajorButton
    RadioButton $ds9(buttons).major.help \
	[string tolower [msgcat::mc {Help}]] \
	buttons(majorCurrent) $ds9(buttons).help MajorButton

    global pbuttons
    array set pbuttons {
	major,file 1
	major,edit 1
	major,view 1
	major,frame 1
	major,bin 1
	major,zoom 1
	major,scale 1
	major,color 1
	major,region 1
	major,wcs 1
	major,help 1
    }

    set buttons(major) "
        $ds9(buttons).major.file pbuttons(major,file)
        $ds9(buttons).major.edit pbuttons(major,edit)
        $ds9(buttons).major.view pbuttons(major,view)
        $ds9(buttons).major.frame pbuttons(major,frame)
        $ds9(buttons).major.bin pbuttons(major,bin)
        $ds9(buttons).major.zoom pbuttons(major,zoom)
        $ds9(buttons).major.scale pbuttons(major,scale)
        $ds9(buttons).major.color pbuttons(major,color)
        $ds9(buttons).major.region pbuttons(major,region)
        $ds9(buttons).major.wcs pbuttons(major,wcs)
        $ds9(buttons).major.help pbuttons(major,help)
    "
    CreateButtonsFile
    CreateButtonsEdit
    CreateButtonsView
    CreateButtonsFrame
    CreateButtonsBin
    CreateButtonsZoom
    CreateButtonsScale
    CreateButtonsColor
    CreateButtonsRegion
    CreateButtonsWCS
    CreateButtonsHelp

    LayoutButtons
}

proc ButtonButton {button text cmd} {
    global ds9

    switch $ds9(wm) {
	x11 -
	win32 {
	    button $button \
		-text $text \
		-command $cmd \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 2 
	}
	aqua {
	    button $button \
		-text $text \
		-command $cmd \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 2
	}
    }
}

proc RadioButton {button text varname value cmd} {
    global ds9

    switch $ds9(wm) {
	x11 -
	win32 {
	    radiobutton $button \
		-text $text \
		-variable $varname \
		-value $value \
		-command $cmd \
		-indicatoron false \
		-selectcolor {} \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 3
	}
	aqua {
	    button $button \
		-text $text \
		-command \
		"RadioButtonAqua $button $varname \{$value\} \{$cmd\}" \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 2
	    
	    uplevel #0 trace variable $varname w \
		[list "RadioButtonAquaCB $button \{$value\}"]

	    upvar #0 $varname var
	    if {[$button cget -state] != {disabled}} {
		if {$var == $value} {
		    $button configure -state active
		} else {
		    $button configure -state normal
		}
	    }
	}
    }

    return $button
}

proc RadioButtonAqua {button varname value cmd} {
    uplevel #0 [list set $varname $value]
    eval $cmd
}

proc RadioButtonAquaCB {button value varname id op} {
    upvar #0 $varname var
    global $varname

    if {[$button cget -state] != {disabled}} {
	if [array exists $varname] {
	    if {$var($id) == $value} {
		$button configure -state active
	    } else {
		$button configure -state normal
	    }
	} else {
	    if {$var == $value} {
		$button configure -state active
	    } else {
		$button configure -state normal
	    }
	}
    }
}

proc CheckButton {button text varname cmd} {
    global ds9

    switch $ds9(wm) {
	x11 -
	win32 {
	    checkbutton $button \
		-text $text \
		-variable $varname \
		-command $cmd \
		-indicatoron false \
		-selectcolor {} \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 3
	}
	aqua {
	    button $button \
		-text $text \
		-command "CheckButtonAqua $button $varname \{$cmd\}" \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 2

	    uplevel #0 trace variable $varname w \
		[list "CheckButtonAquaCB $button"]

	    upvar #0 $varname var
	    if {[$button cget -state] != {disabled}} {
		if {$var} {
		    $button configure -state active
		} else {
		    $button configure -state normal
		}
	    }
	}
    }
}

proc CheckButtonAqua {button varname cmd} {
    upvar #0 $varname var

    uplevel #0 [list set $varname [expr !$var]]
    eval $cmd
}

proc CheckButtonAquaCB {button varname id op} {
    upvar #0 $varname var
    global $varname

    if {[$button cget -state] != {disabled}} {
	if [array exists $varname] {
	    if {$var($id)} {
		$button configure -state active
	    } else {
		$button configure -state normal
	    }
	} else {
	    if {$var} {
		$button configure -state active
	    } else {
		$button configure -state normal
	    }
	}
    }
}

proc LayoutButtons {} {
    global ds9
    global buttons
    global view

    pack forget $ds9(buttons).major
    switch $view(layout) {
	horizontal {
	    $ds9(buttons) configure -width 0
	    pack propagate $ds9(buttons) on
	    pack $ds9(buttons).major -side top -fill x -expand true
	}
	vertical {
	    $ds9(buttons) configure -width 125
	    pack propagate $ds9(buttons) off
	    pack $ds9(buttons).major -side top -fill x -expand true -anchor n
	}
    }

    UpdateButtons buttons(major)
    UpdateButtons buttons(file)
    UpdateButtons buttons(edit)
    UpdateButtons buttons(view)
    UpdateButtons buttons(frame)
    UpdateButtons buttons(bin)
    UpdateButtons buttons(zoom)
    UpdateButtons buttons(scale)
    UpdateButtons buttons(color)
    UpdateButtons buttons(region)
    UpdateButtons buttons(wcs)
    UpdateButtons buttons(help)

    MajorButton
}

proc MajorButton {} {
    global buttons
    global view

    pack forget $buttons(majorPrev)
    switch $view(layout) {
	horizontal {
	    pack $buttons(majorCurrent) -side top -fill x -expand true
	}
	vertical {
	    pack $buttons(majorCurrent) -side bottom -fill x -expand true -anchor s
	}
    }
    set buttons(majorPrev) $buttons(majorCurrent)
}

proc UpdateButtons {varname} {
    upvar #0 $varname var

    foreach {which what} $var {
	pack forget $which
    }
    foreach {which what} $var {
	ShowButton $which $what
    }
}

proc ShowButton {which varname} {
    upvar #0 $varname var
    global view

    if {$var} {
 	switch $view(layout) {
 	    horizontal {pack $which -side left -fill both -expand true}
 	    vertical {pack $which -side top -fill both -expand true}
 	}
    }
}

