#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CrosshairDef {} {
    global crosshair
    set crosshair(lock) none
}

proc ButtonCrosshair {which x y} {
    global ds9
    global crosshair

    $which crosshair begin motion canvas $x $y

    if {$crosshair(lock) != "none"} {
	set coord [$which get crosshair $crosshair(lock) scientific]
	foreach f $ds9(frames) {
	    if {$f != $which && [$f has system $crosshair(lock)]} {
		switch -- $ds9(display,mode) {
		    single {$f crosshair $crosshair(lock) $coord}
		    tile {$f crosshair begin motion $crosshair(lock) $coord}
		    blink {}
		}
	    }
	}
    }
}

proc MotionCrosshair {which x y} {
    global ds9
    global crosshair

    $which crosshair motion canvas $x $y

    if {$crosshair(lock) != "none"} {
	set coord [$which get crosshair $crosshair(lock) scientific]
	foreach f $ds9(frames) {
	    if {$f != $which && [$f has system $crosshair(lock)]} {
		switch -- $ds9(display,mode) {
		    single {$f crosshair $crosshair(lock) $coord}
		    tile {$f crosshair motion $crosshair(lock) $coord}
		    blink {}
		}
	    }
	}
    }
}

proc ReleaseCrosshair {which x y} {
    global ds9
    global crosshair

    $which crosshair canvas $x $y

    if {$crosshair(lock) != "none"} {
	set coord [$which get crosshair $crosshair(lock) scientific]
	foreach f $ds9(frames) {
	    if {$f != $which && [$f has system $crosshair(lock)]} {
		$f crosshair $crosshair(lock) $coord
	    }
	}
    }
}

proc ArrowKeyCrosshair {which x y} {
    global ds9
    global crosshair

    $which crosshair warp $x $y

    if {$crosshair(lock) != "none"} {
	set coord [$which get crosshair $crosshair(lock) scientific]
	foreach f $ds9(frames) {
	    if {$f != $which && [$f has system $crosshair(lock)]} {
		$f crosshair $crosshair(lock) $coord 
	    }
	}
    }

    set coord [$which get crosshair canvas]
    set X [lindex $coord 0]
    set Y [lindex $coord 1]

    UpdateColormapLevelMosaic $which $X $Y canvas
    UpdateInfoBox $which $X $Y canvas
    UpdatePixelTableDialog $which $X $Y canvas
    UpdateGraph $which $X $Y canvas
}

proc ProcessCrosshairCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current
    global ds9
    global crosshair

    # we need our coordinate matrices set up
    RealizeDS9

    set x [lindex $var [expr $i+0]]
    set y [lindex $var [expr $i+1]]
    set sys [lindex $var [expr $i+2]]
    set sky [lindex $var [expr $i+3]]
    set format {}

    incr i 1
    incr i [FixSpec sys sky format physical fk5 degrees]

    set ds9(mode) crosshair
    ChangeMode

    if {$current(frame) != {}} {
	$current(frame) crosshair $sys $sky $x $y
	set coord [$current(frame) get crosshair canvas]
	UpdateColormapLevelMosaic $current(frame) \
	    [lindex $coord 0] [lindex $coord 1] canvas
	UpdateInfoBox $current(frame) \
	    [lindex $coord 0] [lindex $coord 1] canvas

	if {$crosshair(lock) != "none"} {
	    set coord \
		[$current(frame) get crosshair $crosshair(lock) scientific]
	    foreach f $ds9(frames) {
		if {$f != $current(frame) && [$f has system $crosshair(lock)]} {
		    $f crosshair $crosshair(lock) $coord
		}
	    }
	}
    }
}

proc ProcessSendCrosshairCmd {proc id param} {
    global current

    set sys [lindex $param 0]
    set sky [lindex $param 1]
    set format [lindex $param 2]
    FixSpec sys sky format physical fk5 degrees

    if {$current(frame) != {}} {
	$proc $id "[$current(frame) get crosshair $sys $sky $format]\n"
    }
}

