#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

# who are we?
set ds9(app) [file rootname [file tail $argv0]]
# if we have a problem at this point, dump simple message and exit
if {[catch {tk windowingsystem} ds9(wm)]} {
    puts stderr "Unable to initialize window system."
    exit
}

# options
switch $ds9(wm) {
    x11 -
    win32 {}
    aqua {
	option add {*Frame*Background} lightgrey
	option add {*Label*Background} lightgrey
	option add {*Entry*HighlightBackground} lightgrey
	option add {*Button*HighlightBackground} lightgrey
	option add {*Menubutton*Background} lightgrey
	option add {*Checkbutton*Background} lightgrey
	option add {*Text*HighlightBackground} lightgrey
	option add {*Text*BorderWidth} 2
	option add {*Text*Relief} sunken
    }
}
option add {*Entry*BorderWidth} 1
option add {*Scale*BorderWidth} 1
option add {*Radiobutton*BorderWidth} 1
option add {*Checkbutton*BorderWidth} 1
option add {*Scrollbar*BorderWidth} 1
option add {*Label*BorderWidth} 2

# initialize
set ds9(root) "/usr/share/ds9"

package require msgcat
package require http

package require base64
package require log
package require ftp
package require textutil::repeat
package require textutil::tabify

package require tkcon
source $ds9(root)/xmlrpc0.3/xmlrpc.tcl

package require BLT
package require Tktable
package require tclxpa
package require img::base
package require img::jpeg
package require img::tiff
package require img::png
package require img::gif
package require img::ppm
package require img::bmp
package require img::xbm
package require img::window

source $ds9(root)/src/source.tcl

# fix ::tk and msgcat
rename ::tk::msgcat::mc {}
rename ::tk::msgcat::mcmax {}

namespace import ::msgcat::mc
namespace import ::msgcat::mcmax
::msgcat::mcload [file join $::tk_library msgs]

# Define Variables
DS9Def
2MASSDef
3DDef
AnalysisDef
APDef
BinDef
BlinkDef
ButtonsDef
CanvasDef
CATDef
CATSymDef
CATCDSSrchDef
CentroidDef
ColorbarDef
ContourDef
CoordDef
CrosshairDef
CubeDef
CurrentDef
CursorDef
DebugDef 
ESODef
ExamineDef
FIRSTDef
GraphDef
GridDef
GroupDef
HelpDef
HTTPDef
HVDef
IISDef
MagnifierDef
MarkerDef
MaskDef
MinMaxDef
NRESDef
NVSSDef
PannerDef
PanZoomDef
PixelDef
PrefsDef
PSDef
RGBDef
SAODef
SaveImageDef
ScaleDef
SkyViewDef
SmoothDef
STSCIDef
TemplateDef
TileDef
ViewDef
VODef
WCSDef
ZScaleDef

# let's start
set ds9(init) 1

# set up signal trap
# not supported under windows
switch $tcl_platform(platform) {
    unix {signal add SIGINT QuitDS9}
    windows {}
}

# set the appname
tk appname $ds9(app)

# environment vars
# we don't want to see any error messages if xpa is not available
if { [info exists env(XPA_VERBOSITY)] == 0 } {
  set env(XPA_VERBOSITY) 0
}
# set filter ptype to contained (default is process)
set env(FILTER_PTYPE) c
# set filter error proc so it will not kill ds9
set env(GERROR) 0

# Virtual Events
switch $ds9(wm) {
    x11 {}
    win32 {}
    aqua {event add <<Redo>> <Command-Key-Z>}
}
event add <<Open>> <${ds9(ctrl)}o>
event add <<PageSetup>> <${ds9(ctrl)}P>
event add <<Print>> <${ds9(ctrl)}p>
event add <<SelectAll>> <${ds9(ctrl)}a>
event add <<Find>> <${ds9(ctrl)}f>
event add <<FindNext>> <${ds9(ctrl)}g>

# Init Temporary Dir before prefs
InitTempDir

# Init the filter compiler
InitFilterCompiler

# Load any preferences here, before we do any real work
LoadPrefs

# adjust default canvas size based on layout
switch $view(layout) {
    horizontal {
	set canvas(width) 680
	set canvas(height) 450
    }
    vertical {
	set canvas(width) 540
	set canvas(height) 646
    }
}

# set default font
if {$ds9(font) != {default}} {
    option add *font "$ds9(font) $ds9(font,size) $ds9(font,style)"
}

switch $ds9(wm) {
    x11 -
    win32 {}
    aqua {
	::tk::unsupported::MacWindowStyle style $ds9(top) document "closeBox fullZoom collapseBox resizable"
	# we need to map the top window so we can get the proper truecolor masks
	update idletasks
    }
}

# We want to withdraw the window til everything is ready to go
wm withdraw $ds9(top)

set title "SAOImage $ds9(title)"
wm title $ds9(top) "$title"
wm iconname $ds9(top) "$title"
wm protocol $ds9(top) WM_DELETE_WINDOW QuitDS9
unset title

$ds9(top) configure -menu $ds9(mb)

# we need to set certain variables before anything else
# such as color, title, language
ProcessCommandLineFirst

# initialize language
switch $ds9(language) {
    locale {
	switch $ds9(wm) {
	    aqua {
		foreach ll [MacOSXGetLocale] {
		    if {[SetLanguage $ll]} {
			break;
		    }
		}
	    }
	    default {}
	}
    }
    default {SetLanguage $ds9(language)}
}

# set the visual
set ds9(visual) [winfo visual .]
set ds9(depth) [winfo depth .]

switch $ds9(wm) {
    x11 {
	if {$ds9(depth)==15} {
	    set ds9(depth) 16
	}
	if {$ds9(depth)==32} {
	    set ds9(depth) 24
	}
    } 
    win32 {
	if {$ds9(depth)==32} {
	    set ds9(depth) 24
	}
    }
    aqua {
	if {$ds9(depth)==15} {
	    set ds9(depth) 16
	}
    }
}

switch -- $ds9(visual)$ds9(depth) {
    pseudocolor8 {}
    truecolor8 {}
    truecolor16 {}
    truecolor24 {}
    default {BadVisualError}
}

# create our main frame
set ds9(main) [frame ${ds9(top)}ds9]
pack $ds9(main) -fill both -expand true

# Create image canvas
CreateCanvas

# Create Colorbar-- Create this first, so in case of a private colormap,
# gui colors will be allocated in the new colormap, not the default colormap
CreateColorbar

# Create other parts of the display
CreateMenuBar
CreateInfoPanel
CreateButtons
CreateGraphs

# Make sure that the wm knows when to swap in the colormap (if needed)
wm colormapwindows . "$ds9(main) $ds9(canvas)"

# Initialize the display
InitColorbar
InitPanner
InitDialogBox

# Set our current state of things
ChangeMode

# force a update, then layout
update
ConfigureView

# our first frame
CreateFrame

# do this last so we don't get an ConfigureView event
InitCanvas

# ok, ready to show the window
wm deiconify $ds9(top)
update

# Init external File Formats
# we want this before processing the command line
InitExternalFile

# Init analysis file formats
InitAnalysisFile

# Configure HTTP
ConfigHTTP

# SAMP
InitSAMP

# and process any command line items
# we want to see something before any fits files are loaded
ProcessCommandLine

# Initialize IIS
# after command line options to set port/fifo/unix...
catch {IISInit}

# XPA
InitXPA

# and finally, process any command line options that are dependant on xpa
ProcessCommandLineXPA

# any os events received?
switch $ds9(wm) {
    x11 {}
    win32 {}
    aqua {
	MacOSXOpenDocEvent 0
	MacOSXPrintDocEvent 1
    }
}

# Load any initalization tcl code
SourceInitFile {.ini}

# do we have the correct prefs file?
CheckPrefs

# ok, we're done
set ds9(init) 0
