#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc ExamineDef {} {
    global examine

    set examine(mode) new
    set examine(zoom) 4
}

proc ButtonExamine {which x y} {
    if {![$which has fits]} {
	return
    }

    switch -- [$which get type] {
	base {ButtonExamineBase $which $x $y}
	rgb {ButtonExamineRGB $which $x $y}
	3d {ButtonExamine3D $which $x $y}
    }
}

proc ButtonExamineBase {which x y} {
    global current
    global ds9
    global examine

    # current slice
    set slice [$which get fits slice]

    # current coord
    if {[$which has fits bin]} {
	set coord [$which get coordinates canvas $x $y physical]
    } else {
	set coord [$which get coordinates canvas $x $y image]
    }

    # find filename
    set fn [$which get fits file name full canvas $x $y]

    set ds9(next) $which
    GotoFrame

    # create frame if needed
    switch -- $examine(mode) {
	new {CreateFrame}
	one {
	    if {[info exists examine(one)]} {
		if {$which == $examine(one)} {
		    # do nothing, we clicked in the examine frame
		    return
		}
		DeleteSingleFrame $examine(one)
		CreateFrame
		set examine(one) $current(frame)
	    } else {
		CreateFrame
		set examine(one) $current(frame)
	    }
	}
    }

    # go to tile mode in case
    set ds9(display,user) tile
    DisplayMode

    # load data
    LoadFits $fn

    # update
    RealizeDS9

    # zoom to about
    if {[$current(frame) has fits bin]} {
	set bf "[$current(frame) get bin factor]"
	set bx [expr [lindex $bf 0]/$examine(zoom)]
	set by [expr [lindex $bf 1]/$examine(zoom)]
	$current(frame) bin factor to $bx $by about \
	    [lindex $coord 0] [lindex $coord 1]
    } else {
	$current(frame) zoom $examine(zoom) $examine(zoom) \
	    about image [lindex $coord 0] [lindex $coord 1]
    }

    # set slice
    $current(frame) update fits slice $slice

    # back to original frame
    set ds9(next) $which
    GotoFrame
    $current(frame) update fits slice $slice

    # update any dialogs
    UpdateDS9
}

proc ButtonExamineRGB {which x y} {
    global current
    global ds9
    global examine

    # curent channel
    set channel [$which get rgb channel]

    # current slice
    set slice [$which get fits slice]

    # current coord
    $which rgb channel red
    if {[$which has fits bin]} {
	set coord [$which get coordinates canvas $x $y physical]
    } else {
	set coord [$which get coordinates canvas $x $y image]
    }

    # find filename
    foreach cc {red green blue} {
	$which rgb channel $cc
	set fn($cc) [$which get fits file name full canvas $x $y]
    }

    set ds9(next) $which
    GotoFrame

    # create frame if needed
    switch -- $examine(mode) {
	new {CreateRGBFrame}
	one {
	    if {[info exists examine(one)]} {
		if {$which == $examine(one)} {
		    # do nothing, we clicked in the examine frame
		    return
		}
		DeleteSingleFrame $examine(one)
		CreateRGBFrame
		set examine(one) $current(frame)
	    } else {
		CreateRGBFrame
		set examine(one) $current(frame)
	    }
	}
    }

    # go to tile mode in case
    set ds9(display,user) tile
    DisplayMode

    # cycle thru channels
    foreach cc {red green blue} {
	$which rgb channel $cc
	$current(frame) rgb channel $cc

	# load data
	if {$fn($cc) != {}} {
	    LoadFits $fn($cc)
	}
    }

    # update
    RealizeDS9

    # zoom to about
    if {[$current(frame) has fits bin]} {
	foreach cc {red green blue} {
	    $which rgb channel $cc
	    $current(frame) rgb channel $cc

	    set bf "[$current(frame) get bin factor]"
	    set bx [expr [lindex $bf 0]/$examine(zoom)]
	    set by [expr [lindex $bf 1]/$examine(zoom)]
	    $current(frame) bin factor to $bx $by about \
		[lindex $coord 0] [lindex $coord 1]
	}
    } else {
	$current(frame) rgb channel red
	$current(frame) zoom $examine(zoom) $examine(zoom) \
	    about image [lindex $coord 0] [lindex $coord 1]
    }

    # set slice
    $current(frame) update fits slice $slice

    # set channel
    $current(frame) rgb channel $channel

    # back to original frame
    set ds9(next) $which
    GotoFrame
    $current(frame) rgb channel $channel
    $current(frame) update fits slice $slice

    # update any dialogs
    UpdateDS9
}

proc ButtonExamine3D {which x y} {
}

# Cmds

proc ProcessSendImexamCmd {proc id param} {
    global ds9
    global cursor

    set ds9(imexam,frame) {}
    set ds9(imexam,x) {}
    set ds9(imexam,y) {}
    set ds9(imexam,event) {}
    set ds9(imexam,mode) $ds9(mode)

    set ds9(mode) imexam

    set ds9(imexam) 0
    set ds9(imexam,key) 0
    set ds9(imexam,any) 0

    # turn on blinking cursor
    set cursor(timer) 1
    CursorTimer

    switch -- [string tolower [lindex $param 0]] {
	key {
	    set ds9(imexam,key) 1
	    set varname {ds9(imexam,key)}
	    set param [lreplace $param 0 0]
	}
	any {
	    set ds9(imexam,any) 1
	    set varname {ds9(imexam,any)}
	    set param [lreplace $param 0 0]
	}
	default {
	    set ds9(imexam) 1
	    set varname {ds9(imexam)}
	}
    }

    switch -- [string tolower [lindex $param 0]] {
	value -
	data {
	    vwait $varname
	    set w [lindex $param 1]
	    set h [lindex $param 2]
	    if {$w == {}} {
		set w 1
	    }
	    if {$h == {}} {
		set h 1
	    }
	    $proc $id "$ds9(imexam,event) [$ds9(imexam,frame) get data canvas $ds9(imexam,x) $ds9(imexam,y) $w $h]\n"
	}
	coordinate {
	    set sys [lindex $param 1]
	    set sky [lindex $param 2]
	    set skyformat [lindex $param 3]
	    switch -- $skyformat {
		{} {set skyformat degrees}
	    }
	    switch -- $sky {
		{} {set sky fk5}
	    }
	    switch -- $sys {
		{} {set sys physical}
		fk4 -
		fk5 -
		icrs -
		galactic -
		ecliptic {set sky $sys; set sys wcs}
	    }

	    vwait $varname
	    $proc $id "$ds9(imexam,event) [$ds9(imexam,frame) get coordinates canvas $ds9(imexam,x) $ds9(imexam,y) $sys $sky $skyformat]\n"
	}
    }

    # turn off blinking cursor
    set cursor(timer) 0

    set ds9(mode) $ds9(imexam,mode)

    set ds9(imexam) 0
    set ds9(imexam,key) 0

    set ds9(imexam,frame) {}
    set ds9(imexam,x) {}
    set ds9(imexam,y) {}
    set ds9(imexam,event) {}
    set ds9(imexam,mode) {}
}

