#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

# Open

proc OpenFits {{layer {}}} {
    global current

    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    # just in case (could be invoked via a menu keyshortcut)
    if {$current(frame) == {}} {
	CreateFrame
    }

    if {$layer == {mask}} {
	if {![MaskLoad]} {
	    return
	}
    }

    StartLoad
    LoadFits $fn $layer
    FinishLoad
}

proc OpenDataCubeFits {{layer {}}} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    if {$layer == {mask}} {
	if {![MaskLoad]} {
	    return
	}
    }

    StartLoad
    LoadDataCubeFits $fn $layer
    FinishLoad
}

proc OpenMultiFrameFits {} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    StartLoad
    LoadMultiFrameFits $fn
    FinishLoad
}

proc OpenMosaicImageIRAFFits {{layer {}}} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    if {$layer == {mask}} {
	if {![MaskLoad]} {
	    return
	}
    }

    StartLoad
    LoadMosaicImageIRAFFits $fn $layer
    FinishLoad
}

proc OpenMosaicIRAFFits {{layer {}}} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    if {$layer == {mask}} {
	if {![MaskLoad]} {
	    return
	}
    }

    StartLoad
    LoadMosaicIRAFFits $fn $layer
    FinishLoad
}

proc OpenMosaicImageWCSFits {{layer {}}} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    set sys {}
    if [MosaicWCSDialog sys] {
	if {$layer == {mask}} {
	    if {![MaskLoad]} {
		return
	    }
	}

	StartLoad
	LoadMosaicImageWCSFits $sys $fn $layer
	FinishLoad
    }
}

proc OpenMosaicWCSFits {{layer {}}} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    set sys {}
    if [MosaicWCSDialog sys] {
	if {$layer == {mask}} {
	    if {![MaskLoad]} {
		return
	    }
	}

	StartLoad
	LoadMosaicWCSFits $sys $fn $layer
	FinishLoad
    }
}

proc OpenMosaicImageWFPC2Fits {} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    StartLoad
    LoadMosaicImageWFPC2Fits $fn
    FinishLoad
}

proc OpenRGBImageFits {} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    StartLoad
    LoadRGBImageFits $fn
    FinishLoad
}

proc OpenRGBCubeFits {} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    StartLoad
    LoadRGBCubeFits $fn
    FinishLoad
}

proc OpenArray {{layer {}}} {
    set fn [OpenFileDialog arrayfbox]
    if {[string length $fn] == 0} {
	return
    }

    # do we have an array spec tag'd on
    if {![regexp -nocase {(.*)(\[.*\])} $fn foo base ext]} {
	set ext {}
	if [ArrayDialog 1 ext] {
	    append fn "$ext"
	} else {
	    return
	}
    }

    if {$layer == {mask}} {
	if {![MaskLoad]} {
	    return
	}
    }

    StartLoad
    LoadArray $fn $layer
    FinishLoad
}

proc OpenRGBArray {} {
    set fn [OpenFileDialog arrayfbox]
    if {[string length $fn] == 0} {
	return
    }

    # do we have an array spec tag'd on
    if {![regexp -nocase {(.*)(\[.*\])} $fn foo base ext]} {
	set ext {}
	if [ArrayDialog 3 ext] {
	    append fn "$ext"
	} else {
	    return
	}
    }

    StartLoad
    LoadRGBArray $fn
    FinishLoad
}

proc OpenURL {} {
    global http

    set fn {}
    if {[EntryDialog [msgcat::mc {URL}] [msgcat::mc {Enter World Wide Web Location (URL)}] 80 fn]} {
	StartLoad
	LoadURL $fn
	FinishLoad
    }
}

proc OpenVar {} {
    set fn [OpenFileDialog fitsvarfbox]
    if {[string length $fn] == 0} {
	return
    }

    if {![catch {set ch [open "$fn"]}]} {
	fconfigure $ch -translation binary -encoding binary
	global vardata
	set vardata [read -nonewline $ch]
	close $ch

	StartLoad
	LoadVar vardata $fn {}
	FinishLoad
    }
}

# Load

proc LoadFits {fn {layer {}}} {
    global current
    global ds9
    global loadParam
    global marker

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) {}
    set loadParam(file,name) $fn
    ConvertFile
    # save load type, since ProcessLoad will clear loadParam
    if {$loadParam(load,type) == "mmapincr"} {
	set mmap 1
    } else {
	set mmap 0
    }
    ProcessLoad

    # now autoload markers
    if {$ds9(automarker) && $mmap} {
	# now, load fits[REGION] if present
	set id [string first "\[" $fn]
	if {$id > 0} {
	    set base [string range $fn 0 [expr $id-1]]
	} else {
	    set base $fn
	}

	set reg "${base}\[REGION\]"
	if {[$current(frame) fitsy has ext "\"$reg\""]} {
	    RealizeDS9
	    catch {
		$current(frame) marker load fits "\"$reg\"" $marker(color) $marker(dashlist) $marker(width) "\{$marker(font) $marker(font,size) $marker(font,style)\}"
	    }
	}
    }
}

proc LoadSFits {hdr fn {layer {}}} {
    global current
    global ds9
    global loadParam
    global marker

    set loadParam(load,type) smmap
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) {}
    set loadParam(file,name) $fn
    set loadParam(file,header) $hdr
    ProcessLoad
}

proc LoadDataCubeFits {fn {layer {}}} {
    global loadParam

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) {data cube}
    set loadParam(file,name) $fn
    ConvertFile
    ProcessLoad
}

proc LoadMultiFrameFits {fn} {
    global loadParam
    global current
    global ds9
    global errorInfo

    set ext 0
    set cnt 0
    set did 0
    set need 0

    # start with new frame?
    switch -- [$current(frame) get type] {
	base {
	    if [$current(frame) has fits] {
		CreateFrame
		set did 1
	    }
	}
	rgb -
	3d {
	    CreateFrame
	    set did 1
	}
    }

    while {1} {

	# create a new frame
	if {$need} {
	    CreateFrame
	    set did 1
	}

	set loadParam(load,type) mmapincr
	set loadParam(load,layer) {}
	set loadParam(file,type) fits
	set loadParam(file,mode) {}
	set loadParam(file,name) "$fn\[$ext\]"
	ConvertFile 

	set rr [ProcessLoad 0]
	if  {$rr != {}} {
	    if {$ext} {
		# reset xpa error
		set errorInfo {}
		if {$did} {
		    DeleteCurrentFrame
		    incr ds9(next,num) -1
		}
		if {!$cnt} {
		    Error "$rr"
		}
		break;
	    }
	} else {
	    # ignore any bin tables
	    if {![$current(frame) has fits bin]}  {
		incr cnt
		set need 1
	    } else {
		set need 0
	    }
	}

	incr ext
    }

    # go into tile mode if more than one
    if {$cnt && $ds9(display,user) != "tile"} {
	set ds9(display,user) tile
	DisplayMode
    }
}

proc LoadMosaicImageIRAFFits {fn {layer {}}} {
    global loadParam

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) {mosaic image iraf}
    set loadParam(file,name) $fn
    ConvertFile
    ProcessLoad
}

proc LoadMosaicIRAFFits {fn {layer {}}} {
    global loadParam

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) {mosaic iraf}
    set loadParam(file,name) $fn
    ConvertFile
    ProcessLoad
}

proc LoadMosaicIRAFSFits {hdr fn {layer {}}} {
    global loadParam

    set loadParam(load,type) smmap
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) {mosaic iraf}
    set loadParam(file,name) $fn
    set loadParam(file,header) $hdr
    ProcessLoad
}

proc LoadMosaicImageWCSFits {sys fn {layer {}}} {
    global loadParam

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) [list mosaic image $sys]
    set loadParam(file,name) $fn
    ConvertFile
    ProcessLoad
}

proc LoadMosaicWCSFits {sys fn {layer {}}} {
    global loadParam

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) [list mosaic $sys]
    set loadParam(file,name) $fn
    ConvertFile
    ProcessLoad
}

proc LoadMosaicWCSSFits {sys hdr fn {layer {}}} {
    global loadParam

    set loadParam(load,type) smmap
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) [list mosaic $sys]
    set loadParam(file,name) $fn
    set loadParam(file,header) $hdr
    ProcessLoad
}

proc LoadMosaicImageWFPC2Fits {fn} {
    global loadParam

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) {}
    set loadParam(file,type) fits
    set loadParam(file,mode) {mosaic image wfpc2}
    set loadParam(file,name) $fn
    ConvertFile
    ProcessLoad
}

proc LoadRGBImageFits {fn} {
    global loadParam
    global current

    switch -- [$current(frame) get type] {
	base -
	3d {Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]}
	rgb {
	    set loadParam(load,type) mmapincr
	    set loadParam(load,layer) {}
	    set loadParam(file,type) fits
	    set loadParam(file,mode) {rgb image}
	    set loadParam(file,name) $fn
	    ConvertFile
	    ProcessLoad
	}
    }
}

proc LoadRGBCubeFits {fn} {
    global loadParam
    global current

    switch -- [$current(frame) get type] {
	base -
	3d {Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]}
	rgb {
	    set loadParam(load,type) mmapincr
	    set loadParam(load,layer) {}
	    set loadParam(file,type) fits
	    set loadParam(file,mode) {rgb cube}
	    set loadParam(file,name) $fn
	    ConvertFile
	    ProcessLoad
	}
    }
}

proc LoadRGBCubeSFits {hdr fn} {
    global loadParam
    global current

    switch -- [$current(frame) get type] {
	base -
	3d {Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]}
	rgb {
	    set loadParam(load,type) smmap
	    set loadParam(load,layer) {}
	    set loadParam(file,type) fits
	    set loadParam(file,mode) {rgb cube}
	    set loadParam(file,name) $fn
	    set loadParam(file,header) $hdr
	    ProcessLoad
	}
    }
}

proc LoadRGBArray {fn} {
    global loadParam
    global current

    # if no zdim is present, insert one
    set exp {.*\[.*zdim[ ]*=[ ]*[0-9]+}
    if {![regexp $exp $fn]} {
	set i [string last "\]" $fn]
        set fn "[string range $fn 0 [expr $i-1]],zdim=3\]"
    }

    switch -- [$current(frame) get type] {
	base -
	3d {Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]}
	rgb {
	    set loadParam(load,type) mmapincr
	    set loadParam(load,layer) {}
	    set loadParam(file,type) array
	    set loadParam(file,mode) {rgb cube}
	    set loadParam(file,name) $fn
	    ConvertFile
	    ProcessLoad
	}	
    }
}

proc LoadVar {varname fn mode {layer {}}} {
    global loadParam

    set loadParam(load,type) var
    set loadParam(load,layer) $layer
    set loadParam(var,name) $varname
    set loadParam(file,type) fits
    set loadParam(file,name) "$fn"
    set loadParam(file,mode) $mode
    ProcessLoad
}

proc LoadArray {fn {layer {}}} {
    global loadParam

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) $layer
    set loadParam(file,type) array
    set loadParam(file,mode) {}
    set loadParam(file,name) $fn
    ConvertFile
    ProcessLoad
}

proc LoadURL {url} {
    if {[string length $url] == 0} {
	return
    }

    ParseURL $url r
    switch -- $r(scheme) {
	ftp {LoadURLFTP $r(authority) $r(path)}
	file {LoadURLFile $r(path)}
	http -
	default {LoadURLHTTP $url}
    }
}

proc LoadURLFTP {host path} {
    global loadParam
    global ds9
    global debug

    set ftp [ftp::Open $host "ftp" "-ds9@" -mode passive]
    if {$ftp > -1} {
	set fn "$ds9(tmpdir)/[file tail $path]"
	set ftp::VERBOSE $debug(tcl,ftp)
	set "ftp::ftp${ftp}(Output)" FTPLog
	ftp::Type $ftp binary
	if [ftp::Get $ftp $path $fn] {
	    # alloc it because we are going to delete it after load
	    set loadParam(load,type) allocgz
	    set loadParam(load,layer) {}
	    set loadParam(file,type) fits
	    set loadParam(file,mode) {}
	    set loadParam(file,name) $fn
	    set loadParam(file,fn) $fn
	    ConvertFile
	    ProcessLoad
	}

	ftp::Close $ftp

	if [file exists $fn] {
	    catch {file delete -force $fn}
	}
    }
}

proc LoadURLFile {path} {
    global loadParam
    global ds9

    # alloc it because we can't assume it will last
    set loadParam(load,type) allocgz
    set loadParam(load,layer) {}
    set loadParam(file,type) fits
    set loadParam(file,mode) {}
    set loadParam(file,name) $path
    set loadParam(file,fn) $path
    ConvertFile
    ProcessLoad
}

proc LoadURLHTTP {url} {
    global ds9
    global http

    ParseURL $url r
    set fn "$ds9(tmpdir)/[file tail $r(path)]"

    set code 200
    set meta {}
    set mime "application/fits"
    set encoding {}

    set ch [open $fn w]
    set token [http::geturl $url -timeout $http(timeout) -channel $ch -binary 1 -headers "[ProxyHTTP]"]
    catch {close $ch}

    upvar #0 $token t

    # Code
    set code [http::ncode $token]

    # Meta
    set meta $t(meta)

    # Mime-type
    # we want to strip and extra info after ';'
    regexp -nocase {([^;])*} $t(type) mime

    # Content-Encoding
    foreach {name value} $meta {
	if {[regexp -nocase ^content-encoding $name]} {
	    switch -- [string tolower $value] {
		gzip -
		x-gzip {set encoding gzip}
		compress -
		bzip2 {set encoding bzip2}
		Z {set encoding compress}
		pack -
		z {set encoding pack}
		default {}
	    }
	}
    }

    HTTPLog $token
    # Result?
    switch -- $code {
	200 {}
	default {
	    Error "HTTP [msgcat::mc {Error}] $code"
	    return
	}
    }

    http::cleanup $token

    global debug
    if {$debug(tcl,hv)} {
	puts stderr "Load HTTP: fn $fn : code $code : meta $meta : mime $mime : encoding $encoding"
    }

    # NOTE: error notices may come as text/html
    switch -- [string tolower $mime] {
	"image/fits" -
	"application/fits" {}

	"application/fits-image" -
	"application/fits-table" -
	"application/fits-group" {}

	"image/x-fits" -
	"binary/x-fits" -
	"application/x-fits" {}

	"image/x-gfits" -
	"binary/x-gfits" -
	"image/gz-fits" -
	"display/gz-fits" {set encoding gzip}

	"image/bz2-fits" -
	"display/bz2-fits" {set encoding bzip2}

	"image/x-cfits" -
	"binary/x-cfits" {set encoding compress}

	"image/x-zfits" -
	"binary/x-zfits" {set encoding pack}

	default {
	    Error "[msgcat::mc {File not Found or Unable to load FITS data MIME type}] $mime"
	    return
	}
    }

    # alloc it because we are going to delete it after load
    StartLoad
    global loadParam
    set loadParam(load,type) allocgz
    set loadParam(load,layer) {}
    set loadParam(file,type) fits
    set loadParam(file,mode) {}
    set loadParam(file,name) $fn
    set loadParam(file,fn) $fn

    # may have to convert the file, based on content-encoding
    switch -- "$encoding" {
	bzip2 {
	    catch {set ch [open "| bunzip2 < $fn " r]}
	    set loadParam(load,type) channel
	    set loadParam(channel,name) $ch
	}
	compress {
	    catch {set ch [open "| uncompress < $fn " r]}
	    set loadParam(load,type) channel
	    set loadParam(channel,name) $ch
	}
	pack {
	    catch {set ch [open "| pcat $fn " r]}
	    set loadParam(load,type) channel
	    set loadParam(channel,name) $ch
	}
    }

    ProcessLoad
    FinishLoad

    if {[file exists $fn]} {
	catch {file delete -force $fn}
    }
}

# Support

proc MultiLoad {{layer {}}} {
    global ds9
    global current

    global debug
    if {$debug(tcl,layout)} {
	puts stderr "MultiLoad"
    }

    if {$layer != {}} {
	return
    }

    set cnt [llength $ds9(frames)]
    switch $cnt {
	0 {
	    CreateFrame
	    return
	}
	1 {
	    if {![$current(frame) has fits]} {
		return
	    }
	}
    }

    switch -- [$current(frame) get type] {
	base {CreateFrame}
	rgb {return}
	3d {Create3DFrame}
    }

    # go into tile mode if more than one
    if {$cnt >= 1 && $ds9(display,user) != "tile"} {
	set ds9(display,user) tile
	DisplayMode
    }
}

proc ProcessLoad {{err 1}} {
    global current
    global loadParam

    if [catch {
	switch -- $loadParam(load,type) {
	    alloc -
	    allocgz {$current(frame) load $loadParam(file,type) \
			 $loadParam(file,mode) \
			 \{$loadParam(file,name)\} \
			 $loadParam(load,type) \
			 \{$loadParam(file,fn)\} \
			 $loadParam(load,layer)}
	    channel {
		fconfigure $loadParam(channel,name) -translation binary \
		    -encoding binary
		$current(frame) load $loadParam(file,type) \
		    $loadParam(file,mode) \
		    \{$loadParam(file,name)\} \
		    $loadParam(load,type) \
		    $loadParam(channel,name) \
		    $loadParam(load,layer)
	    }
	    mmap -
	    mmapincr {$current(frame) load $loadParam(file,type) \
			  $loadParam(file,mode) \
			  \{$loadParam(file,name)\} \
			  $loadParam(load,type) \
			  $loadParam(load,layer)}
	    smmap {$current(frame) load $loadParam(file,type) \
		       $loadParam(file,mode) \
		       \{$loadParam(file,header)\} \
		       \{$loadParam(file,name)\} \
		       $loadParam(load,type) \
		       $loadParam(load,layer)}
	    shared {$current(frame) load $loadParam(file,type) \
			$loadParam(file,mode) \
			\{$loadParam(file,name)\} \
			$loadParam(load,type) \
			$loadParam(shared,idtype) \
			$loadParam(shared,id) \
			$loadParam(load,layer)}
	    sshared {$current(frame) load $loadParam(file,type) \
			 $loadParam(file,mode) \
			 \{$loadParam(file,name)\} \
			 $loadParam(load,type) \
			 $loadParam(shared,idtype) \
			 $loadParam(shared,hdr) \
			 $loadParam(shared,id) \
			 $loadParam(load,layer)}
	    socket -
	    socketgz {$current(frame) load $loadParam(file,type) \
			  $loadParam(file,mode) \
			  \{$loadParam(file,name)\} \
			  $loadParam(load,type) \
			  $loadParam(socket,id) \
			  $loadParam(load,layer)}
	    var {$current(frame) load $loadParam(file,type) \
		     $loadParam(file,mode) \
		     \{$loadParam(file,name)\} \
		     $loadParam(load,type) \
		     $loadParam(var,name) \
		     $loadParam(load,layer)}
	}
    } rr] {
	if {$err} {
	    Error "[msgcat::mc {Unable to load}] $loadParam(file,type) $loadParam(file,mode) $loadParam(file,name)"
	}
    }

    unset loadParam
    return $rr
}

proc StartLoad {} {
    SetWatchCursor
}

proc FinishLoad {} {
    global loadParam
    global current
    global threed

    GridUpdate
    ContourUpdate

    # do after updategrid and updatecontour
    UpdateDS9
    UnsetWatchCursor

    # just in case, frame may have been deleted before FinishLoad during startup
    if {$current(frame) == {}} {
	return
    }

    # if header(s) were open, remove them
    DestroyHeader $current(frame)

    # do we have a data cube?
    switch -- [$current(frame) get type] {
	base -
	rgb {
	    if [$current(frame) has fits cube] {
		CubeDialog
	    }
	}
	3d {
	    if [$current(frame) has fits cube] {
		CubeDialog
	    }
	    if [$current(frame) has fits] {
		set mm [$current(frame) get minmax]
		set threed(threshold) \
		    [expr ([lindex $mm 1]-[lindex $mm 0])/2.+[lindex $mm 0]]
		3DThreshold 0
		Update3DDialog
	    }
	}
    }
}

proc ConvertFile {} {
    global ds9

    foreach t $ds9(fileProcs) {
	if [$t] {
	    return
	}
    }
}

# File Types

proc Stdin {} {
    global loadParam

    if {[string index $loadParam(file,name) 0] == "-"} {
	set loadParam(load,type) allocgz
	set loadParam(file,name) "stdin[string range $loadParam(file,name) 1 end]"
	set loadParam(file,fn) $loadParam(file,name)
	return 1
    } elseif {[string range $loadParam(file,name) 0 4] == "stdin" || 
	      [string range $loadParam(file,name) 0 4] == "STDIN"} {
	set loadParam(load,type) allocgz
	set loadParam(file,name) "stdin[string range $loadParam(file,name) 5 end]"
	set loadParam(file,fn) $loadParam(file,name)
	return 1
    }

    return 0
}

proc BZip2Fits {} {
    global loadParam

    if { [regexp {(.*)\.bz2($|\[)} $loadParam(file,name) matched root] } {
	if [catch {set ch [open "| bunzip2 < $root.bz2 " r]}] {
	    return 0
	}
	set loadParam(load,type) channel
	set loadParam(channel,name) $ch
	return 1
    }

    return 0
}

proc CompressFits {} {
    global loadParam

    if { [regexp {(.*)\.Z($|\[)} $loadParam(file,name) matched root] } {
	if [catch {set ch [open "| uncompress < $root.Z " r]}] {
	    return 0
	}
	set loadParam(load,type) channel
	set loadParam(channel,name) $ch
	return 1
    }

    return 0
}

proc PackFits {} {
    global loadParam

    if { [regexp {(.*)\.z($|\[)} $loadParam(file,name) matched root] } {
	if [catch {set ch [open "| pcat $root.z " r]}] {
	    return 0
	}
	set loadParam(load,type) channel
	set loadParam(channel,name) $ch
	return 1
    }

    return 0
}

proc GzipFits {} {
    global loadParam

    if { [regexp {(.*)\.gz($|\[)} $loadParam(file,name) matched root] } {
	set loadParam(load,type) allocgz
	set loadParam(file,fn) $loadParam(file,name)
	return 1
    }

    return 0
}

proc FTZFits {} {
    global loadParam

    if { [regexp -nocase {(.*)\.(ftz)($|\[)} $loadParam(file,name) matched root ext] } {
	set loadParam(load,type) allocgz
	set loadParam(file,fn) $loadParam(file,name)
	return 1
    }

    return 0
}

proc ExternalFits {} {
    global loadParam
    global extFits

    foreach id [array names extFits] {
	if {[string match $id "$loadParam(file,name)"]} {
	    regsub -all {\$loadParam(file,name)} $extFits($id) "$loadParam(file,name)" result
	    set cmd "| $result"
	    if [catch {set ch [open "$cmd" r]}] {
		return 0
	    }
	    set loadParam(load,type) channel
	    set loadParam(channel,name) $ch
	    return 1
	}
    }
    return 0
}

proc ArrayDialog {depth varname} {
    upvar $varname var
    global env
    global ed
    global ds9

    set w {.arr}
    set length 300

    set ed(ok) 0
    set ed(x) $ds9(array,x)
    set ed(y) $ds9(array,y)
    set ed(z) $depth
    set ed(bitpix) $ds9(array,bitpix)
    set ed(skip) $ds9(array,skip)
    set ed(arch) $ds9(array,arch)

    if {[info exists env(DS9_ARRAY)]} {
	if {[regexp {.*(dims.?=)([0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(x) $item
	    set ed(y) $item
	}
	if {[regexp {.*(dim.?=)([0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(x) $item
	    set ed(y) $item
	}
	if {[regexp {.*(xdim.?=)([0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(x) $item
	}
	if {[regexp {.*(ydim.?=)([0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(y) $item
	}
	if {[regexp {.*(zdim.?=)([0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(z) $item
	}
	if {[regexp {.*(bitpix.?=)(-?[0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(bitpix) $item
	}
	if {[regexp {.*(skip.?=)(-?[0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(skip) $item
	}
	if {[regexp {.*arch.?=bigendian} $env(DS9_ARRAY) foo item]} {
	    set ed(arch) $item
	}
	if {[regexp {.*arch.?=littleendian} $env(DS9_ARRAY) foo item]} {
	    set ed(arch) $item
	}
    }

    DialogCreate $w [msgcat::mc {Load Array}]

    frame $w.arr
    frame $w.arr.dim -relief ridge -bd 1
    frame $w.arr.bitpix -relief ridge -bd 1
    frame $w.arr.skip -relief ridge -bd 1
    frame $w.arr.arch -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.arr.dim $w.arr.bitpix $w.arr.skip $w.arr.arch -side top \
	-ipadx 4 -ipady 4 -fill x -expand true
    pack $w.arr -fill x -expand true
    pack $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    label $w.arr.dim.title -text [msgcat::mc {Dimension}]
    label $w.arr.dim.dummy -width 4
    entry $w.arr.dim.x -textvariable ed(x) -width 6
    entry $w.arr.dim.y -textvariable ed(y) -width 6
    entry $w.arr.dim.z -textvariable ed(z) -width 6
    grid $w.arr.dim.title $w.arr.dim.x $w.arr.dim.y $w.arr.dim.z \
	$w.arr.dim.dummy -padx 4 -sticky w

    label $w.arr.bitpix.title -text [msgcat::mc {Pixel Size}]
    label $w.arr.bitpix.dummy -width 9
    radiobutton $w.arr.bitpix.char -text {Char} \
	-variable ed(bitpix) -value 8
    radiobutton $w.arr.bitpix.short -text {Short} \
	-variable ed(bitpix) -value 16
    radiobutton $w.arr.bitpix.ushort -text {UShort} \
	-variable ed(bitpix) -value -16
    radiobutton $w.arr.bitpix.long -text {Long} \
	-variable ed(bitpix) -value 32
    radiobutton $w.arr.bitpix.float -text {Float} \
	-variable ed(bitpix) -value -32
    radiobutton $w.arr.bitpix.double -text {Double} \
	-variable ed(bitpix) -value -64
    grid $w.arr.bitpix.title $w.arr.bitpix.char \
	$w.arr.bitpix.dummy -padx 4 -sticky w
    grid x $w.arr.bitpix.short -padx 4 -sticky w
    grid x $w.arr.bitpix.ushort -padx 4 -sticky w
    grid x $w.arr.bitpix.long -padx 4 -sticky w
    grid x $w.arr.bitpix.float -padx 4 -sticky w
    grid x $w.arr.bitpix.double -padx 4 -sticky w

    label $w.arr.skip.t1 -text [msgcat::mc {Skip First}]
    label $w.arr.skip.t2 -text [msgcat::mc {Bytes}]
    label $w.arr.skip.dummy -width 4
    entry $w.arr.skip.skip -textvariable ed(skip) -width 6
    grid $w.arr.skip.t1 $w.arr.skip.skip $w.arr.skip.t2 \
	$w.arr.skip.dummy -padx 4 -sticky w

    label $w.arr.arch.title -text [msgcat::mc {Architecture}]
    label $w.arr.arch.dummy -width 0
    radiobutton $w.arr.arch.big -text {Big-Endian} \
	-variable ed(arch) -value bigendian
    radiobutton $w.arr.arch.little -text {Little-Endian} \
	-variable ed(arch) -value littleendian
    grid $w.arr.arch.title $w.arr.arch.big $w.arr.arch.dummy -padx 4 -sticky w
    grid x $w.arr.arch.little -padx 4 -sticky w

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok)
    DialogDismiss $w

    if {$ed(ok)} {
	set ds9(array,x) $ed(x)
	set ds9(array,y) $ed(y)
	set ds9(array,bitpix) $ed(bitpix)
	set ds9(array,skip) $ed(skip)
	set ds9(array,arch) $ed(arch)

	set var "\[xdim=$ed(x),ydim=$ed(y),zdim=$ed(z),bitpix=$ed(bitpix),skip=$ed(skip),arch=$ed(arch)\]"
    }

    set rr $ed(ok)
    unset ed
    return $rr
}

proc MosaicWCSDialog {varname} {
    upvar $varname var
    global ed

    set w {.wcs}
    set length 300

    set ed(ok) 0
    set ed(sys) wcs
    set ed(label) WCS

    DialogCreate $w [msgcat::mc {Load Mosaic}]

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    label $w.param.title -text [msgcat::mc {Select Coordinate System }]
    menubutton $w.param.sys -textvariable ed(label) \
	-menu $w.param.sys.m -relief raised -width 10
    grid $w.param.title $w.param.sys

    menu $w.param.sys.m
    $w.param.sys.m add radiobutton -label [msgcat::mc {WCS}] \
	-variable ed(sys) -value "wcs" -command [list set ed(label) WCS]
    $w.param.sys.m add separator
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$w.param.sys.m add radiobutton -variable ed(sys) \
	    -label "[msgcat::mc {WCS}] $l" \
	    -value "wcs$l" \
	    -command [list set ed(label) "[msgcat::mc {WCS}] $l"]
    }

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok)
    DialogDismiss $w

    if {$ed(ok)} {
	set var $ed(sys)
    }

    set rr $ed(ok)
    unset ed
    return $rr
}

# Header

proc DisplayHeaderMenu {} {
    global current

    set last {}
    set prim {}
    set cnt [$current(frame) get fits count]

    if {$cnt > 0} {
	set slb(count) 0

	for {set ii 1} {$ii <= $cnt} {incr ii} {
	    set fn [$current(frame) get fits file name $ii]
	    set simple [$current(frame) get fits header $ii keyword {SIMPLE}]
	    set xten [$current(frame) get fits header $ii keyword {XTENSION}]

	    if {$xten != {}} {
		set b [string first {[} $fn]
		set pn [string range $fn 0 [expr $b-1]]
		if {$b > 0 && $pn != $prim} {
		    incr slb(count)
		    set slb($slb(count),item) $pn
		    set slb($slb(count),value) "-$ii"
		    set prim $pn
		}
	    }
	    
	    if {$fn != $last && $fn != $prim} {
		incr slb(count)
		set slb($slb(count),item) $fn
		set slb($slb(count),value) $ii
		set last $fn
	    }
	}

	if {$slb(count) == 1} {
	    DisplayHeader $current(frame) 1 $fn
	} else {
	    if [SLBDialog slb {Select Header} 40] {
		DisplayHeader $current(frame) $slb(value) $slb(item)
	    }
	}
    }
}

proc DisplayHeader {frame which title} {
    global current

    set varname "hd[string range $frame end end]-$which"

    upvar #0 $varname var
    global $varname

    SimpleTextDialog $varname $title 80 40 insert top [$current(frame) get fits header $which]

    # create a special text tag for keywords
    $var(text) tag configure keyword -font {courier 12} -foreground blue
    
    # color tag keywords
    set stop [$var(text) index end]
    for {set ii 1.0} {$ii<$stop} {set ii [expr $ii+1]} {
	$var(text) tag add keyword $ii "$ii +8 chars"
    }
}

proc DestroyHeader {frame} {
    global st

    set ttt "hd[string range $frame end end]"

    foreach x [array names st] {
	set f [split $x ,]
	if {[lindex $f 1] == "top"} {
	    set varname [lindex $f 0]

	    upvar #0 $varname var
	    global $varname

	    set fff [split $tt :]
	    if {[lindex $fff 0] == $ttt} {
		if {[info exists $varname] && [winfo exists ${varname}(top)]} {
		    SimpleTextDestroy $varname
		}
	    }
	}
    }
}

# Cmds

# Array

proc ProcessArrayCmd {varname iname sock fn} {
    upvar $varname var
    upvar $iname i

    global loadParam
    global current

    set loadParam(file,type) array
    set loadParam(load,layer) {}

    # new/mask
    switch -- [string tolower [lindex $var $i]] {
	new {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		rgb {
		    # leave rgb in place for next section
		    CreateRGBFrame
		}
		default {
		    CreateFrame
		}
	    }
	}
	mask {
	    incr i
	    set loadParam(load,layer) mask
	}
    }

    # mode/name
    set loadParam(file,mode) {}
    set loadParam(file,name) {}

    switch -- [string tolower [lindex $var $i]] {
	rgb {
	    incr i
	    switch -- [$current(frame) get type] {
		base -
		3d {
		    Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]
		    return
		}
		rgb {
		    set loadParam(file,mode) {rgb cube}
		    set loadParam(file,name) [lindex $var $i]
		    incr i
		}
	    }
	}
	default {
	    set loadParam(file,name) [lindex $var $i]
	    incr i
	}
    }
    if {$loadParam(file,name) == {}} {
	set loadParam(file,name) stdin
    }

    StartLoad
    if {$sock != {}} {
	set loadParam(load,type) socketgz
	set loadParam(socket,id) $sock
	ProcessLoad
    } else {
	set ch [open $fn r]
	set loadParam(load,type) channel
	set loadParam(channel,name) $ch
	ProcessLoad
	catch {close $ch}
    }
    FinishLoad
}

proc ProcessSendArrayCmd {proc id param sock fn} {
    global current

    if {$current(frame) == {}} {
	return
    }

    if {$sock != {}} {
	$current(frame) save array socket $sock \
	    [string tolower [lindex $param 0]]
    } elseif {$fn != {}} {
	append fn {.arr}
	$current(frame) save array file $fn \
	    [string tolower [lindex $param 0]]
	$proc $id {} $fn
    }
}

# File

proc ProcessFileCmd {varname iname sock fn} {
    upvar $varname var
    upvar $iname i

    global current

    set layer {}

    switch -- [string tolower [lindex $var $i]] {
	new {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		rgbimage -
		rgbcube -
		rgbarray {
		    incr i
		    CreateRGBFrame
		}
		default {
		    CreateFrame
		}
	    }
	}
	mask {
	    incr i
	    set layer mask
	}
    }

    switch -- [string tolower [lindex $var $i]] {
	save {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		gz {
		    incr i
		    $current(frame) save fits image file [lindex $var $i] gz
		}
		resample {
		    incr i
		    switch -- [string tolower [lindex $var $i]] {
			gz {
			    incr i
			    $current(frame) save fits resample file [lindex $var $i] gz
			}
			default {
			    $current(frame) save fits resample file [lindex $var $i]
			}
		    }
		}
		default {
		    $current(frame) save fits image file [lindex $var $i]
		}
	    }
	}

	default {
	    StartLoad
	    switch -- [string tolower [lindex $var $i]] {
		array {
		    incr i
		    LoadArray [lindex $var $i] $layer
		}

		fits {
		    incr i
		    LoadFits [lindex $var $i] $layer
		}
		sfits {
		    incr i
		    LoadSFits [lindex $var $i] [lindex $var [expr $i+1]] $layer
		    incr i
		}

		datacube -
		medatacube {
		    incr i
		    LoadDataCubeFits [lindex $var $i] $layer
		}

		multiframe {
		    incr i
		    LoadMultiFrameFits [lindex $var $i]
		}

		mosaicimage {
		    incr i
		    switch -- [lindex $var $i] {
			iraf {
			    incr i
			    LoadMosaicImageIRAFFits [lindex $var $i] $layer
			}
			wfpc2 {
			    incr i
			    LoadMosaicImageWFPC2Fits [lindex $var $i]
			}
			default {
			    if {[string range [lindex $var $i] 0 2] == "wcs"} {
				LoadMosaicImageWCSFits [lindex $var $i] [lindex $var [expr $i+1]] $layer
			    incr i
			    } else {
				LoadMosaicImageWCSFits wcs [lindex $var $i] $layer
			    }
			}
		    }
		}
		mosaic {
		    incr i
		    switch -- [lindex $var $i] {
			iraf {
			    incr i
			    LoadMosaicIRAFFits [lindex $var $i] $layer
			}
			default {
			    if {[string range [lindex $var $i] 0 2] == "wcs"} {
				LoadMosaicWCSFits [lindex $var $i] [lindex $var [expr $i+1]] $layer
				incr i
			    } else {
				LoadMosaicWCSFits wcs [lindex $var $i] $layer
			    }
			}
		    }
		}
		smosaic {
		    incr i
		    switch -- [lindex $var $i] {
			iraf {
			    incr i
			    LoadMosaicIRAFSFits [lindex $var $i] [lindex $var [expr $i+1]] $layer
			    incr i
			}
			default {
			    if {[string range [lindex $var $i] 0 2] == "wcs"} {
				LoadMosaicWCSSFits [lindex $var $i] [lindex $var [expr $i+1]] [lindex $var [expr $i+2]] $layer
			    incr i 2
			    } else {
				LoadMosaicWCSSFits wcs [lindex $var $i] [lindex $var [expr $i+1]] $layer
			    }
			    incr i
			}
		    }
		}

		mosaicimageiraf {
		    # backward compatibility
		    incr i
		    LoadMosaicImageIRAFFits [lindex $var $i] $layer
		}
		mosaiciraf {
		    # backward compatibility
		    incr i
		    LoadMosaicIRAFFits [lindex $var $i] $layer
		}
		mosaicimagewcs {
		    # backward compatibility
		    incr i
		    LoadMosaicImageWCSFits wcs [lindex $var $i] $layer
		}
		mosaicwcs {
		    # backward compatibility
		    incr i
		    LoadMosaicWCSFits wcs [lindex $var $i] $layer
		}
		mosaicwfpc2 -
		mosaicimagewfpc2 {
		    # backward compatibility
		    incr i
		    LoadMosaicImageWFPC2Fits [lindex $var $i]
		}

		rgbcube {
		    incr i
		    LoadRGBCubeFits [lindex $var $i]
		}
		srgbcube {
		    incr i
		    LoadRGBCubeSFits [lindex $var $i] [lindex $var [expr $i+1]]
		    incr i
		}
		rgbimage {
		    incr i
		    LoadRGBImageFits [lindex $var $i]
		}
		rgbarray {
		    incr i
		    LoadRGBArray [lindex $var $i]
		}

		url {
		    incr i
		    LoadURL [lindex $var $i]
		}
		default {
		    LoadFits [lindex $var $i] $layer
		}
	    }
	    FinishLoad
	}
    }
}

proc ProcessSendFileCmd {proc id param} {
    global current

    if {$current(frame) != {}} {
	$proc $id "[$current(frame) get fits file name full]\n"
    }
}

# Fits

proc ProcessFitsCmd {varname iname sock fn} {
    upvar $varname var
    upvar $iname i

    global loadParam
    global current

    set loadParam(file,type) fits
    set loadParam(load,layer) {}

    switch -- [string tolower [lindex $var $i]] {
	new {
	    incr i
	    CreateFrame
	}
	mask {
	    incr i
	    set loadParam(load,layer) mask
	}
    }

    # mode/name
    set loadParam(file,mode) {}
    set loadParam(file,name) {}

    switch -- [string tolower [lindex $var $i]] {
	datacube -
	medatacube {
	    incr i
	    set loadParam(file,mode) {data cube}
	}

	mosaicimage {
	    incr i
	    switch -- [lindex $var $i] {
		iraf -
		wfpc2 {
		    set loadParam(file,mode) [list mosaic image [lindex $var $i]]
		    incr i
		    set loadParam(file,name) [lindex $var $i]
		}
		default {
		    if {[string range [lindex $var $i] 0 2] == "wcs"} {
			set loadParam(file,mode) [list mosaic image [lindex $var $i]]
			incr i
			set loadParam(file,name) [lindex $var $i]
		    } else {
			set loadParam(file,mode) [list mosaic image wcs]
			set loadParam(file,name) [lindex $var $i]
		    }
		}
	    }
	}
	mosaic {
	    incr i
	    switch -- [lindex $var $i] {
		iraf {
		    set loadParam(file,mode) [list mosaic [lindex $var $i]]
		    incr i
		    set loadParam(file,name) [lindex $var $i]
		}
		default {
		    if {[string range [lindex $var $i] 0 2] == "wcs"} {
			set loadParam(file,mode) [list mosaic [lindex $var $i]]
			incr i
			set loadParam(file,name) [lindex $var $i]
		    } else {
			set loadParam(file,mode) [list mosaic wcs]
			set loadParam(file,name) [lindex $var $i]
		    }
		}
	    }
	}

	mosaicimageiraf {
	    # backward compatibility
	    incr i
	    set loadParam(file,mode) {mosaic image iraf}
	    set loadParam(file,name) [lindex $var $i]
	}
	mosaiciraf {
	    # backward compatibility
	    incr i
	    set loadParam(file,mode) {mosaic iraf}
	    set loadParam(file,name) [lindex $var $i]
	}
	mosaicimagewcs {
	    # backward compatibility
	    incr i
	    set loadParam(file,mode) {mosaic image wcs}
	    set loadParam(file,name) [lindex $var $i]
	}
	mosaicwcs {
	    # backward compatibility
	    incr i
	    set loadParam(file,mode) {mosaic wcs}
	    set loadParam(file,name) [lindex $var $i]
	}
	mosaicwfpc2 -
	mosaicimagewfpc2 {
	    # backward compatibility
	    incr i
	    set loadParam(file,mode) {mosaic image wfpc2}
	    set loadParam(file,name) [lindex $var $i]
	}

	rgbimage {
	    incr i
	    switch -- [$current(frame) get type] {
		base -
		3d {
		    Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]
		    return
		}
		rgb {
		    incr i
		    set loadParam(file,mode) {rgb image}
		    set loadParam(file,name) [lindex $var $i]
		}
	    }
	}
	rgbcube {
	    incr i
	    switch -- [$current(frame) get type] {
		base -
		3d {
		    Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]
		    return
		}
		rgb {
		    incr i
		    set loadParam(file,mode) {rgb cube}
		    set loadParam(file,name) [lindex $var $i]
		}
	    }
	}
	default {
	    set loadParam(file,name) [lindex $var $i]
	}
    }
    if {$loadParam(file,name) == {}} {
	set loadParam(file,name) stdin
    }

    StartLoad
    if {$sock != {}} {
	set loadParam(load,type) socketgz
	set loadParam(socket,id) $sock
    } else {
	set loadParam(load,type) allocgz
	set loadParam(file,fn) $fn
    }
    ProcessLoad
    FinishLoad
}

proc ProcessSendFitsCmd {proc id param sock fn} {
    global current

    if {$current(frame) == {}} {
	return
    }

    switch -- [string tolower [lindex $param 0]] {
	size {
	    set sys [lindex $param 1] 
	    set sky [lindex $param 2] 
	    set format [lindex $param 3]
	    if {$sys == {} && $sky == {} && $format == {}} {
		$proc $id "[$current(frame) get fits size]\n"
	    } else {
		FixSpec sys sky format image fk5 degrees
		$proc $id "[$current(frame) get fits size $sys $sky $format]\n"
	    }
	}
	width {$proc $id "[$current(frame) get fits width]\n"}
	height {$proc $id "[$current(frame) get fits height]\n"}
	bitpix {$proc $id "[$current(frame) get fits bitpix]\n"}
	depth {$proc $id "[$current(frame) get fits depth]\n"}
	header {
	    switch [llength $param] {
		1 {ProcessSend $proc $id {} $fn {.txt} \
			"[$current(frame) get fits header 1]\n"}
		2 {ProcessSend $proc $id {} $fn {.txt} \
			"[$current(frame) get fits header [lindex $param 1]]\n"}
		3 {$proc $id "[$current(frame) get fits header 1 keyword [lindex $param 2]]\n"}
		4 {$proc $id "[$current(frame) get fits header [lindex $param 1] keyword [lindex $param 3]]\n"}
	    }
	}
	type {
	    if {[$current(frame) has fits bin]} {
		$proc $id "table\n"
	    } else {
		$proc $id "image\n"
	    }
	}
	table {
	    if {$sock != {}} {
		$current(frame) save fits table socket $sock [lindex $param 1]
	    } elseif {$fn != {}} {
		if {[lindex $param 1] != {}} {
		    append fn ".fits.[lindex $param 1]"
		} else {
		    append fn {.fits}
		}
		$current(frame) save fits table file $fn [lindex $param 1]
		$proc $id {} $fn
	    }
	}
	image {
	    if {$sock != {}} {
		$current(frame) save fits image socket $sock [lindex $param 1]
	    } elseif {$fn != {}} {
		if {[lindex $param 1] != {}} {
		    append fn ".fits.[lindex $param 1]"
		} else {
		    append fn {.fits}
		}
		$current(frame) save fits image file $fn [lindex $param 1]
		$proc $id {} $fn
	    }
	}
	resample {
	    if {$sock != {}} {
		$current(frame) save fits resample socket $sock [lindex $param 1]
	    } elseif {$fn != {}} {
		if {[lindex $param 1] != {}} {
		    append fn ".fits.[lindex $param 1]"
		} else {
		    append fn {.fits}
		}
		$current(frame) save fits resample file $fn [lindex $param 1]
		$proc $id {} $fn
	    }
	}
	gz {
	    if {$sock != {}} {
		$current(frame) save fits image socket $sock gz
	    } elseif {$fn != {}} {
		append fn {.fits.gz}
		$current(frame) save fits image file $fn gz
		$proc $id {} $fn
	    }
	}
	default {
	    if {$sock != {}} {
		$current(frame) save fits image socket $sock
	    } elseif {$fn != {}} {
		append fn {.fits}
		$current(frame) save fits image file $fn
		$proc $id {} $fn
	    }
	}
    }
}

# Header

proc ProcessHeaderCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current

    if {$current(frame) != {}} {
	set item [lindex $var $i]
	if {$item != {} && [string is integer [lindex $var $i]]} {
	    set ii [lindex $var $i]
	} else {
	    set ii 1
	    incr i -1
	}

	catch {DisplayHeader $current(frame) $ii [$current(frame) get fits file name $ii]}
    }
}

# Shm

proc ProcessShmCmd {varname iname ml} {
    upvar $varname var
    upvar $iname i

    global loadParam
    global current
    global ds9

    StartLoad
    set done 0
    while {!$done} {

	# defaults
	set loadParam(load,type) shared
	set loadParam(load,layer) {}
	set loadParam(file,type) fits
	set loadParam(file,mode) {}

	set nn [lindex $var [expr $i+4]]
	if {$nn == {} || [string range $nn 0 0] == "-"} {
	    set def 1
	} else {
	    set def 0
	}

	switch -- [lindex $var $i] {
	    key -
	    shmid {
		if {$ml} {
		    MultiLoad
		}
		set loadParam(shared,idtype) [lindex $var $i]
		set loadParam(shared,id) [lindex $var [expr $i+1]]
		set loadParam(file,name) [lindex $var [expr $i+2]]
		incr i 2
	    }

	    fits {
		if {$ml} {
		    MultiLoad
		}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    sfits {
		if {$ml} {
		    MultiLoad
		}
		set loadParam(load,type) sshared
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,hdr) [lindex $var [expr $i+2]]
		set loadParam(shared,id) [lindex $var [expr $i+3]]
		set loadParam(file,name) [lindex $var [expr $i+4]]
		incr i 4
	    }

	    mosaicimage {
		if {$ml} {
		    MultiLoad
		}
		if {$def} {
		    set loadParam(file,mode) {mosaic image iraf}
		    set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		    set loadParam(shared,id) [lindex $var [expr $i+2]]
		    set loadParam(file,name) [lindex $var [expr $i+3]]
		    incr i 3
		} else {
		    set loadParam(file,mode) \
			[list mosaic image [lindex $var [expr $i+1]]]
		    set loadParam(shared,idtype) [lindex $var [expr $i+2]]
		    set loadParam(shared,id) [lindex $var [expr $i+3]]
		    set loadParam(file,name) [lindex $var [expr $i+4]]
		    incr i 4
		}
	    }
	    mosaic {
		if {$def} {
		    set loadParam(file,mode) {mosaic iraf}
		    set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		    set loadParam(shared,id) [lindex $var [expr $i+2]]
		    set loadParam(file,name) [lindex $var [expr $i+3]]
		    incr i 3
		} else {
		    set loadParam(file,mode) \
			[list mosaic [lindex $var [expr $i+1]]]
		    set loadParam(shared,idtype) [lindex $var [expr $i+2]]
		    set loadParam(shared,id) [lindex $var [expr $i+3]]
		    set loadParam(file,name) [lindex $var [expr $i+4]]
		    incr i 4
		}
	    }
	    smosaic {
		set loadParam(load,type) sshared
		set loadParam(file,mode) \
		    [list mosaic [lindex $var [expr $i+1]]]
		set loadParam(shared,idtype) [lindex $var [expr $i+2]]
		set loadParam(shared,hdr) [lindex $var [expr $i+3]]
		set loadParam(shared,id) [lindex $var [expr $i+4]]
		set loadParam(file,name) [lindex $var [expr $i+5]]
		incr i 5
	    }

	    mosaicimageiraf {
		# backward compatibility
		if {$ml} {
		    MultiLoad
		}
		set loadParam(file,mode) {mosaic image iraf}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    mosaiciraf {
		# backward compatibility
		set loadParam(file,mode) {mosaic iraf}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    mosaicimagewcs {
		# backward compatibility
		if {$ml} {
		    MultiLoad
		}
		set loadParam(file,mode) {mosaic image wcs}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    mosaicwcs {
		# backward compatibility
		set loadParam(file,mode) {mosaic wcs}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    mosaicimagewfpc2 {
		# backward compatibility
		if {$ml} {
		    MultiLoad
		}
		set loadParam(file,mode) {mosaic image wfpc2}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }

	    rgbcube {
		switch -- [$current(frame) get type] {
		    base -
		    3d {Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]}
		    rgb {set loadParam(file,mode) {rgb cube}}
		}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    srgbcube {
		set loadParam(load,type) sshared
		switch -- [$current(frame) get type] {
		    base -
		    3d {Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]}
		    rgb {set loadParam(file,mode) {rgb cube}}
		}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,hdr) [lindex $var [expr $i+2]]
		set loadParam(shared,id) [lindex $var [expr $i+3]]
		set loadParam(file,name) [lindex $var [expr $i+4]]
		incr i 4
	    }
	    rgbimage {
		switch -- [$current(frame) get type] {
		    base -
		    3d {Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]}
		    rgb {set loadParam(file,mode) {rgb image}}
		}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    rgbarray {
		switch -- [$current(frame) get type] {
		    base -
		    3d {Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]}
		    rgb {
			set loadParam(file,type) array
			set loadParam(file,mode) {rgb cube}
		    }
		}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    array {
		if {$ml} {
		    MultiLoad
		}
		set loadParam(file,type) array
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }

	    default {
		if {$ml} {
		    MultiLoad
		}
		set loadParam(shared,idtype) key
		set loadParam(shared,id) [lindex $var $i]
		set loadParam(file,name) [lindex $var [expr $i+1]]
		incr i 1
	    }
	}

	ProcessLoad

	# more to come?
	incr i
	if {([lindex $var $i] == "-shm") || 
	    ([lindex $var $i] == "shm")} {
	    set done 0
	    incr i
	} else {
	    set done 1
	    incr i -1
	}
    }
    FinishLoad
}

proc ProcessSendShmCmd {proc id param} {
    global current

    if {$current(frame) != {}} {
	$proc $id "[$current(frame) get fits file name full]\n"
    }
}

# Preserve

proc ProcessPreserveCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global ds9
    global scale
    global panzoom
    global marker

    switch -- [string tolower [lindex $var $i]] {
	scale {
	    incr i
	    set scale(preserve) [FromYesNo [lindex $var $i]]
	    PreserveScale
	}
	pan {
	    incr i
	    set panzoom(preserve) [FromYesNo [lindex $var $i]]
	    PreservePan
	}
	marker -
	regions {
	    incr i
	    set marker(preserve) [FromYesNo [lindex $var $i]]
	    MarkerPreserve
	}
    }
}

proc ProcessSendPreserveCmd {proc id param} {
    global scale
    global panzoom
    global marker

    switch -- [string tolower $param] {
	scale {$proc $id [ToYesNo $scale(preserve)]}
	pan {$proc $id [ToYesNo $panzoom(preserve)]}
	regions {$proc $id [ToYesNo $marker(preserve)]}
    }
}

# Update

proc ProcessUpdateCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current
    global ds9

    if {$current(frame) == {}} {
	return
    }

    if {[string range [lindex $var $i] 0 0] != "-"} {
	switch -- [string tolower [lindex $var $i]] {
	    on -
	    yes {set ds9(idletasks) 1}
	    no -
	    off {set ds9(idletasks) 0}

	    now {
		if {[string is integer [lindex $var [expr $i+1]]]} {
		    $current(frame) update now \
			[lindex $var [expr $i+1]] \
			[lindex $var [expr $i+2]] [lindex $var [expr $i+3]] \
			[lindex $var [expr $i+4]] [lindex $var [expr $i+5]]
		    
		    incr i 5
		} else {
		    $current(frame) update now
		}
	    }
	    {} {
		$current(frame) update
		incr i -1
	    }

	    default {
		$current(frame) update \
		    [lindex $var $i] \
		    [lindex $var [expr $i+1]] [lindex $var [expr $i+2]] \
		    [lindex $var [expr $i+3]] [lindex $var [expr $i+4]]
		incr i 4
	    }
	}
    } else {
	$current(frame) update
	incr i -1
    }
}

