#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc GridDef {} {
    global griddlg
    global grid

    set griddlg(top) .grid
    set griddlg(mb) .gridmb

    set grid(view) 0
    GridDefault
}

proc GridDefault {} {
    global grid

    set grid(type) analysis
    set grid(system) wcs
    set grid(sky) fk5
    set grid(skyformat) sexagesimal

    set grid(grid) 1
    set grid(grid,color) blue
    set grid(grid,width) 1
    set grid(grid,style) 0
    set grid(grid,gap1) {}
    set grid(grid,gap2) {}
    set grid(grid,gapunit1) {}
    set grid(grid,gapunit2) {}

    set grid(axes) 1
    set grid(axes,color) red
    set grid(axes,width) 1
    set grid(axes,style) 0
    set grid(axes,type) interior

    set grid(tick) 1
    set grid(tick,color) white
    set grid(tick,width) 1
    set grid(tick,style) 0

    set grid(border) 1
    set grid(border,color) blue
    set grid(border,width) 1
    set grid(border,style) 0

    set grid(numlab) 1
    set grid(numlab,font) helvetica
    set grid(numlab,style) bold
    set grid(numlab,size) 10
    set grid(numlab,color) green
    set grid(numlab,gap1) {}
    set grid(numlab,gap2) {}
    set grid(numlab,type) interior
    set grid(numlab,vertical) 0

    set grid(title) 1
    set grid(title,text) {}
    set grid(title,def) 1
    set grid(title,gap) {}
    set grid(title,font) helvetica
    set grid(title,style) bold
    set grid(title,size) 12
    set grid(title,color) black

    set grid(textlab) 1
    set grid(textlab,text1) {}
    set grid(textlab,def1) 1
    set grid(textlab,gap1) {}
    set grid(textlab,text2) {}
    set grid(textlab,def2) 1
    set grid(textlab,gap2) {}
    set grid(textlab,font) helvetica
    set grid(textlab,style) bold
    set grid(textlab,size) 10
    set grid(textlab,color) black

    set grid(edge,1) bottom
    set grid(edge,2) left
}

proc GridUpdate {} {
    global grid
    global current

    if {$current(frame) != {}} {
	GridAdjustOptions
	if {$grid(view) && [$current(frame) has fits]} {
	    $current(frame) grid create $grid(system) $grid(sky) \
		$grid(skyformat) $grid(type) [GridOptions]
	} else {
	    $current(frame) grid delete
	}
    }
}	    

proc GridUpdateZoom {} {
    global grid

    if {$grid(type) == "publication"} {
	GridUpdate
    }
}

proc GridAdjustOptions {} {
    global grid
    global current

    if {$current(frame) != {}} {
	# change values if needed
	AdjustCoord $current(frame) grid(system)

	# adjust units
	set grid(grid,gapunit1) pixels
	set grid(grid,gapunit2) pixels
	switch -- $grid(system) {
	    image -
	    physical -
	    amplifier -
	    detector {}
	    default {
		if {$current(frame) != {}} {
		    if {[$current(frame) has wcs equatorial $grid(system)]} {
			set grid(grid,gapunit1) degrees
			set grid(grid,gapunit2) degrees
		    }
		}
	    }
	}
    } else {
	set grid(grid,gapunit1) pixels
	set grid(grid,gapunit2) pixels
    }
}

proc GridOptions {} {
    global grid
    global current

    set opt "\""

    # Grid
    append opt " Grid=$grid(grid),"
    append opt " Colour(grid)=[GridColor2Ast $grid(grid,color)],"
    append opt " Width(grid)=$grid(grid,width),"
    append opt " Style(grid)=$grid(grid,style),"

    # Axes
    append opt " DrawAxes=$grid(axes),"
    append opt " Colour(axes)=[GridColor2Ast $grid(axes,color)],"
    append opt " Width(axes)=$grid(axes,width),"
    append opt " Style(axes)=$grid(axes,style),"

    # Format
    switch $grid(system) {
	image -
	physical -
	detector -
	amplifier {}
	default {
	    if {[$current(frame) has wcs equatorial $grid(system)]} {
		switch $grid(sky) {
		    "fk4" -
		    "fk5" -
		    "icrs" {
			switch $grid(skyformat) {
			    degrees {
				append opt " Format(1)=d.3,"
				append opt " Format(2)=d.3,"
			    }
			    sexagesimal {
				append opt " Format(1)=hms.1,"
				append opt " Format(2)=dms.1,"
			    }
			    hms {
				append opt " Format(1)=lhms.1,"
				append opt " Format(2)=ldms.1,"
			    }
			}
		    }
		    "galactic" -
		    "ecliptic" {
			switch $grid(skyformat) {
			    degrees {
				append opt " Format(1)=d.3,"
				append opt " Format(2)=d.3,"
			    }
			    sexagesimal {
				append opt " Format(1)=dms.1,"
				append opt " Format(2)=dms.1,"
			    }
			    hms {
				append opt " Format(1)=ldms.1,"
				append opt " Format(2)=ldms.1,"
			    }
			}
		    }
		}
	    }
	}
    }

    # Ticks
    if {!$grid(tick)} {
	append opt " MajTickLen=0,"
	append opt " MinTick(1)=0,"
	append opt " MinTick(2)=0,"
    }
    append opt " Colour(ticks)=[GridColor2Ast $grid(tick,color)],"
    append opt " Width(ticks)=$grid(tick,width),"
    append opt " Style(ticks)=$grid(tick,style),"

    # Border
    append opt " Border=$grid(border),"
    append opt " Colour(border)=[GridColor2Ast $grid(border,color)],"
    append opt " Width(border)=$grid(border,width),"
    append opt " Style(border)=$grid(border,style),"

    # Labels
    append opt " Labelling=$grid(axes,type),"
    append opt " LabelUp=$grid(numlab,vertical),"

    append opt " Edge(1)=$grid(edge,1),"
    append opt " Edge(2)=$grid(edge,2),"

    # NumLab
    append opt " NumLab=$grid(numlab),"
    set opt \
"$opt Font(numlab)=[GridFont2Ast $grid(numlab,font) $grid(numlab,style)],"
    append opt " Size(numlab)=$grid(numlab,size),"
    append opt " Colour(numlab)=[GridColor2Ast $grid(numlab,color)],"

    # TextLab
    switch -- $grid(type) {
	analysis {append opt " TextLab(1)=0,"}
	publication {append opt " TextLab(1)=$grid(textlab),"}
    }
    if {!$grid(textlab,def1)} {
	append opt " Label(1)=[GridStripComma $grid(textlab,text1)] ,"
    }

    switch -- $grid(type) {
	analysis {append opt " TextLab(2)=0,"}
	publication {append opt " TextLab(2)=$grid(textlab),"}
    }
    if {!$grid(textlab,def2)} {
	append opt " Label(2)=[GridStripComma $grid(textlab,text2)] ,"
    }

    set opt \
"$opt Font(textlab)=[GridFont2Ast $grid(textlab,font) $grid(textlab,style)],"
    append opt " Size(textlab)=$grid(textlab,size),"
    append opt " Colour(textlab)=[GridColor2Ast $grid(textlab,color)],"

    # Title
    switch -- $grid(type) {
	analysis {append opt " DrawTitle=0,"}
	publication {append opt " DrawTitle=$grid(title),"}
    }

    if {$grid(title,def)} {
	set t [GridStripComma "[$current(frame) get fits object name]"]
	if {$t != {}} {
	    append opt " Title=$t ,"
	}
    } else {
	set t [GridStripComma "$grid(title,text)"]
	if {$t != {}} {
	    append opt " Title=$t ,"
	}
    }

    set opt \
"$opt Font(title)=[GridFont2Ast $grid(title,font) $grid(title,style)],"
    append opt " Size(title)=$grid(title,size),"
    append opt " Colour(title)=[GridColor2Ast $grid(title,color)],"

    # Grid Spacing
    if {$grid(grid,gap1) != {}} {
	if {$grid(grid,gapunit1) == "degrees"} {
	    append opt " Gap(1)=[expr 3.14159/180.*$grid(grid,gap1)],"
	} else {
	    append opt " Gap(1)=$grid(grid,gap1),"
	}
    }

    if {$grid(grid,gap2) != {}} {
	if {$grid(grid,gapunit2) == "degrees"} {
	    append opt " Gap(2)=[expr 3.14159/180.*$grid(grid,gap2)],"
	} else {
	    append opt " Gap(2)=$grid(grid,gap2),"
	}
    }

    # axes numerics
    set flip 0
    switch -- $grid(type) {
	analysis {
	    set numx -.03
	    set numy -.03
	}
	publication {
	    set numx -.02
	    set numy -.01
	    switch -- $grid(axes,type) {
		interior {}
		exterior {
		    switch -- $grid(numlab,type) {
			interior {}
			exterior {set flip 1}
		    }
		}
	    }
	}
    }

    # override
    if {$grid(numlab,gap1) != {}} {
	set numx [expr -$grid(numlab,gap1)/100.]
    }
    if {$grid(numlab,gap2) != {}} {
	set numy [expr -$grid(numlab,gap2)/100.]
    }
    if {$flip} {
	set numx [expr -$numx]
	set numy [expr -$numy]
    }
    append opt " NumLabGap(1)=[expr $numx/[lindex $current(zoom) 0]],"
    append opt " NumLabGap(2)=[expr $numy/[lindex $current(zoom) 1]],"

    # Label gaps
    switch -- $grid(type) {
	analysis {
	    set axisx 0
	    set axisy 0
	    set title 0
	}
	publication {
	    if {$grid(textlab,gap1) != {}} {
		set axisx [expr $grid(textlab,gap1)/100.]
	    } else {
		set axisx .04
	    }
	    if {$grid(textlab,gap2) != {}} {
		set axisy [expr $grid(textlab,gap2)/100.]
	    } else {
		set axisy .04
	    }

	    if {$grid(title,gap) != {}} {
		set title [expr $grid(title,gap)/100.]
	    } else {
		set title .04
	    }
	}
    }

    append opt " TextLabGap(1)=[expr $axisx/[lindex $current(zoom) 0]],"
    append opt " TextLabGap(2)=[expr $axisy/[lindex $current(zoom) 1]],"
    append opt " TitleGap=[expr -1-($title/[lindex $current(zoom) 0])],"

    # Orientation
    append opt " Edge(1)=top,"
    append opt " Edge(2)=left,"

    # The End
    append opt " \""

    global debug

    if {$debug(tcl,grid)} {
	puts stderr "GridOptions"
	puts stderr "$opt"
    }

    return $opt
}

proc GridParseOptions {opt} {
    global grid
    global current
    global debug

    if {$debug(tcl,grid)} {
	puts stderr "GridParseOptions"
	puts stderr "$opt"
    }

    # Grid
    strtok " Grid=%d," opt grid(grid)
    set cc {}
    strtok " Colour(grid)=%d," opt cc
    set grid(grid,color) [GridAst2Color $cc]
    strtok " Width(grid)=%d," opt grid(grid,width)
    strtok " Style(grid)=%d," opt grid(grid,style)

    # Axes
    strtok " DrawAxes=%d," opt grid(axes)
    set cc {}
    strtok " Colour(axes)=%d," opt cc
    set grid(axes,color) [GridAst2Color $cc]
    strtok " Width(axes)=%d," opt grid(axes,width)
    strtok " Style(axes)=%d," opt grid(axes,style)

    # Format
    set format1 {}
    set format2 {}
    strtok { Format(1)=%[^,]} opt format1
    strtok { Format(2)=%[^,]} opt format2

    # Ticks
    set grid(tick) 1
    strtok " MajTickLen=%d," opt grid(tick)
    strtok " MinTick(1)=%d," opt grid(tick)
    strtok " MinTick(2)=%d," opt grid(tick)

    set cc {}
    strtok " Colour(ticks)=%d," opt cc
    set grid(tick,color) [GridAst2Color $cc]
    strtok " Width(ticks)=%d," opt  grid(tick,width)
    strtok " Style(ticks)=%d," opt grid(tick,style)

    # Border
    strtok " Border=%d," opt grid(border)
    set cc {}
    strtok " Colour(border)=%d," opt cc
    set grid(border,color) [GridAst2Color $cc]
    strtok " Width(border)=%d," opt grid(border,width)
    strtok " Style(border)=%d," opt grid(border,style)

    # Labels
    strtok { Labelling=%[^,],} opt grid(axes,type)
    strtok " LabelUp=%d," opt grid(numlab,vertical)

    strtok { Edge(1)=%[^,],} opt grid(edge,1)
    strtok { Edge(2)=%[^,],} opt grid(edge,2)

    # Numlab
    strtok " NumLab=%d," opt grid(numlab)

    set numlabFont 0
    strtok " Font(numlab)=%d," opt numlabFont
    GridAst2Font $numlabFont grid(numlab,font) grid(numlab,style)
    
    strtok " Size(numlab)=%d," opt grid(numlab,size)
    set cc {}
    strtok " Colour(numlab)=%d," opt cc
    set grid(numlab,color) [GridAst2Color $cc]

    # Textlab
    switch -- $grid(type) {
	analysis {strtok " TextLab(1)=%d," opt dummy}
	publication {strtok " TextLab(1)=%d," opt grid(textlab)}
    }
    set grid(textlab,def1) [expr ![strtok { Label(1)=%[^,],} opt grid(textlab,text1)]]

    switch -- $grid(type) {
	analysis {strtok " TextLab(2)=%d," opt dummy}
	publication {strtok " TextLab(2)=%d," opt grid(textlab)}
    }
    set grid(textlab,def2) [expr ![strtok { Label(2)=%[^,],} opt grid(textlab,text2)]]

    set textlabFont 0
    strtok " Font(textlab)=%d," opt textlabFont
    GridAst2Font $textlabFont grid(textlab,font) grid(textlab,style)

    strtok " Size(textlab)=%d," opt grid(textlab,size)
    set cc {}
    strtok " Colour(textlab)=%d," opt cc
    set grid(textlab,color) [GridAst2Color $cc]

    # Title
    switch -- $grid(type) {
	analysis {strtok " DrawTitle=%d," opt dummy}
	publication {strtok " DrawTitle=%d," opt grid(title)}
    }
    strtok { Title=%[^,],} opt grid(title,text)

    set titleFont 0
    strtok " Font(title)=%d," opt titleFont
    GridAst2Font $titleFont grid(title,font) grid(title,style)

    strtok " Size(title)=%d," opt grid(title,size)
    set cc {}
    strtok " Colour(title)=%d" opt cc
    set grid(title,color) [GridAst2Color $cc]

    # Spacing
    set gap1 {}
    strtok { Gap(1)=%[^,]} opt gap1
    if {($gap1 != {}) && ($grid(grid,gapunit1) == "degrees")} {
	set grid(grid,gap1) [expr 180.*$gap1/3.14159]
    } else {
	set grid(grid,gap1) $gap1
    }

    set gap2 {}
    strtok { Gap(2)=%[^,]} opt gap2
    if {($gap2 != {}) && ($grid(grid,gapunit2) == "degrees")} {
	set grid(grid,gap2) [expr 180.*$gap2/3.14159]
    } else {
	set grid(grid,gap2) $gap2
    }

    # Gaps
    set dummy {}
    strtok " NumLabGap(1)=%f," opt dummy
    strtok " NumLabGap(2)=%f," opt dummy
    strtok " TextLabGap(1)=%f," opt dummy
    strtok " TextLabGap(2)=%f," opt dummy
    strtok " TitleGap=%f," opt dummy

    # Orientation
    set dummy {}
    strtok { Edge(1)=%[^,],} opt dummy
    strtok { Edge(2)=%[^,],} opt dummy
}

proc strtok {tmp strvar varvar} {
    upvar $strvar str
    upvar $varvar var
    global debug

    set r [scan $str $tmp var]
    if {$r} {
	set i [string first "," $str]
	set str [string range $str [expr $i+1] end]

	if {$debug(tcl,grid)} {
	    puts stderr "Grid strtok: Success parse $tmp"
	}
    } else {
	if {$debug(tcl,grid)} {
	    puts stderr "Grid strtok: Failed to parse $tmp"
	}
    }

    return $r
}

proc GridAst2Color {ast} {
    switch -- $ast {
	0 {return {black}}
	1 {return {white}}
	2 {return {red}}
	3 {return {green}}
	4 {return {blue}}
	5 {return {cyan}}
	6 {return {magenta}}
	7 {return {yellow}}
    }
}

proc GridColor2Ast {which} {
    switch -- $which {
	black {return 0}
	white {return 1}
	red {return 2}
	green {return 3}
	blue {return 4}
	cyan {return 5}
	magenta {return 6}
	yellow {return 7}
	default {return 0}
    }
}

proc GridAst2Font {ast fnvar fsvar} {
    upvar $fnvar fn
    upvar $fsvar fs

    switch -- $ast {
	0 {set fn "system"; set fs "normal"}
	1 {set fn "times"; set fs "normal"}
	2 {set fn "helvetica"; set fs "normal"}
	3 {set fn "symbol"; set fs "normal"}
	4 {set fn "courier"; set fs "normal"}
	10 {set fn "system"; set fs "bold"}
	11 {set fn "times"; set fs "bold"}
	12 {set fn "helvetica"; set fs "bold"}
	13 {set fn "symbol"; set fs "bold"}
	14 {set fn "courier"; set fs "bold"}
	20 {set fn "system"; set fs "italic"}
	21 {set fn "times"; set fs "italic"}
	22 {set fn "helvetica"; set fs "italic"}
	23 {set fn "symbol"; set fs "italic"}
	24 {set fn "courier"; set fs "italic"}
	default {set fn "system"; set fs "normal"}
    }
}

proc GridFont2Ast {fn fs} {
    if {$fn == "system" && $fs == "normal"} {
	return 0;
    } elseif {$fn == "times" && $fs == "normal"} {
	return 1;
    } elseif {$fn == "helvetica" && $fs == "normal"} {
	return 2;
    } elseif {$fn == "symbol" && $fs == "normal"} {
	return 3;
    } elseif {$fn == "courier" && $fs == "normal"} {
	return 4;
    } elseif {$fn == "system" && $fs == "bold"} {
	return 10;
    } elseif {$fn == "times" && $fs == "bold"} {
	return 11;
    } elseif {$fn == "helvetica" && $fs == "bold"} {
	return 12;
    } elseif {$fn == "symbol" && $fs == "bold"} {
	return 13;
    } elseif {$fn == "courier" && $fs == "bold"} {
	return 14;
    } elseif {$fn == "system" && $fs == "italic"} {
	return 20;
    } elseif {$fn == "times" && $fs == "italic"} {
	return 21;
    } elseif {$fn == "helvetica" && $fs == "italic"} {
	return 22;
    } elseif {$fn == "symbol" && $fs == "italic"} {
	return 23;
    } elseif {$fn == "courier" && $fs == "italic"} {
	return 24;
    } else {
	return 0;
    }
}

proc GridDialog {} {
    global griddlg
    global grid
    global current
    global gr
    global ds9

    # see if we already have a window visible

    if [winfo exists $griddlg(top)] {
	raise $griddlg(top)
	return
    }

    set w $griddlg(top)
    set title [msgcat::mc {Coordinate Grid Parameters}]

    # create the window

    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox collapseBox"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW GridDestroyDialog
    wm minsize $w 400 200

    $w configure -menu $griddlg(mb)

    menu $griddlg(mb)

    AppleMenu griddlg(mb)

    GridFileMenu
    GridEditMenu
    GridTypeMenu
    GridCoordMenu
    GridViewMenu
    GridColorMenu
    GridLineMenu
    GridFontMenu

    frame $w.buttons -relief ridge -bd 1
    frame $w.label -relief ridge -bd 1
    frame $w.param -relief ridge -bd 1
    pack $w.label $w.param $w.buttons -fill x -expand true -ipadx 4 -ipady 4

    # Labels

    label $w.label.label -text [msgcat::mc {Title}]
    entry $w.label.title -textvariable grid(title,text) \
	-width 35
    checkbutton $w.label.default -text [msgcat::mc {Default}] -variable grid(title,def)
    label $w.label.label1 -text "[msgcat::mc {Axis}] 1"
    entry $w.label.title1 -textvariable grid(textlab,text1) \
	-width 35 
    checkbutton $w.label.default1 -text [msgcat::mc {Default}] -variable grid(textlab,def1)
    label $w.label.label2 -text "[msgcat::mc {Axis}] 2"
    entry $w.label.title2 -textvariable grid(textlab,text2) \
	-width 35 
    checkbutton $w.label.default2 -text [msgcat::mc {Default}] -variable grid(textlab,def2)

    grid $w.label.label $w.label.title $w.label.default \
	-padx 4 -pady 1 -sticky w
    grid $w.label.label1 $w.label.title1 $w.label.default1 \
	-padx 4 -pady 1 -sticky w
    grid $w.label.label2 $w.label.title2 $w.label.default2 \
	-padx 4 -pady 1 -sticky w

    # Params

    label $w.param.lspacet -text [msgcat::mc {Spacing}]
    label $w.param.lspace -text "[msgcat::mc {Label}] %"
    label $w.param.ngap -text "[msgcat::mc {Numerics}] %"
    label $w.param.lgap -text [msgcat::mc {Grid Gap}]

    label $w.param.titlet -text [msgcat::mc {Title}]
    entry $w.param.spacet -textvariable grid(title,gap) \
	-width 8 

    label $w.param.title1 -text "[msgcat::mc {Axis}] 1"
    entry $w.param.tspace1 -textvariable grid(textlab,gap1) -width 8 
    entry $w.param.nspace1 -textvariable grid(numlab,gap1) -width 8 
    entry $w.param.gap1 -textvariable grid(grid,gap1) -width 8 
    label $w.param.gapunit1 -textvariable grid(grid,gapunit1) -padx 4

    label $w.param.title2 -text "[msgcat::mc {Axis}] 2"
    entry $w.param.tspace2 -textvariable grid(textlab,gap2) -width 8 
    entry $w.param.nspace2 -textvariable grid(numlab,gap2) -width 8 
    entry $w.param.gap2 -textvariable grid(grid,gap2) -width 8 
    label $w.param.gapunit2 -textvariable grid(grid,gapunit2) -padx 4

    grid $w.param.lspacet $w.param.lspace $w.param.ngap $w.param.lgap \
	x -padx 4 -pady 1 -sticky news
    grid $w.param.titlet $w.param.spacet -padx 4 -pady 1 -sticky w
    grid $w.param.title1 $w.param.tspace1 $w.param.nspace1 \
	$w.param.gap1 $w.param.gapunit1 -padx 4 -pady 1 -sticky w
    grid $w.param.title2 $w.param.tspace2 $w.param.nspace2 \
	$w.param.gap2 $w.param.gapunit2 -padx 4 -pady 1 -sticky w

    button $w.buttons.apply -text [msgcat::mc {Apply}] \
	-command GridApplyDialog
    button $w.buttons.reset -text [msgcat::mc {Reset}] \
	-command GridResetDialog
    button $w.buttons.clear -text [msgcat::mc {Clear}] \
	-command GridClearDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command GridDestroyDialog
    pack $w.buttons.apply $w.buttons.reset $w.buttons.clear $w.buttons.close \
	-side left -padx 10 -expand true

    bind $w <Return> "GridApplyDialog"

    # some window managers need a hint
    raise $w

    UpdateGridDialog
}

proc GridApplyDialog {} {
    global grid

    set grid(view) 1
    GridUpdate
}

proc GridResetDialog {} {
    GridDefault
    GridUpdate
}

proc GridClearDialog {} {
    global grid

    set grid(view) 0
    GridUpdate
}

proc GridDestroyDialog {} {
    global griddlg

    if {[winfo exists $griddlg(top)]} {
	destroy $griddlg(top)
	destroy $griddlg(mb)
    }
}

proc GridCutDialog {} {
    global griddlg

    tk_entryCut [focus -displayof $griddlg(top)]
}

proc GridCopyDialog {} {
    global griddlg

    tk_entryCopy [focus -displayof $griddlg(top)]
}

proc GridPasteDialog {} {
    global griddlg

    tk_entryPaste [focus -displayof $griddlg(top)]
}

proc UpdateGridMenu {} {
    global grid
    global current

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateGridMenu"
    }

    # set menu

    if {($current(frame) != {}) && [$current(frame) has fits]} {
	set grid(view) [$current(frame) has grid]
    }

    # reassign system and format

    if {($current(frame) != {}) && [$current(frame) has fits]} {
	if [$current(frame) has grid] {
	    set l  [$current(frame) get grid]
	    set grid(system) [lindex $l 0]
	    set grid(sky) [lindex $l 1]
	    set grid(skyformat) [lindex $l 2]
	    set grid(type) [lindex $l 3]

	    GridParseOptions [$current(frame) get grid option]
	}
    }
}

proc UpdateGridDialog {} {
    global current
    global griddlg
    global grid

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateGridDialog"
    }

    GridAdjustOptions

    if {[winfo exists $griddlg(top)]} {
	# enable/disable menu items
	UpdateCoordMenu "$current(frame)" $griddlg(mb).coord
	if {$current(frame) != {}} {
	    if {[$current(frame) has wcs equatorial $grid(system)]} {
		$griddlg(mb).coord entryconfig {HMS/DMS} -state normal
	    } else {
		$griddlg(mb).coord entryconfig {HMS/DMS} -state disabled
	    }
	} else {
	    $griddlg(mb).coord entryconfig {HMS/DMS} -state normal
	}
    }
}

proc GridFileMenu {} {
    global griddlg
    global grid

    $griddlg(mb) add cascade -label [msgcat::mc {File}] -menu $griddlg(mb).file

    menu $griddlg(mb).file
    $griddlg(mb).file add command -label [msgcat::mc {Apply}] \
	-command GridApplyDialog
    $griddlg(mb).file add command -label [msgcat::mc {Reset}] \
	-command GridResetDialog
    $griddlg(mb).file add command -label [msgcat::mc {Clear}] \
	-command GridClearDialog
    $griddlg(mb).file add separator
    $griddlg(mb).file add command -label "[msgcat::mc {Load Configuration}]..." \
	-command GridLoadDialog
    $griddlg(mb).file add command -label "[msgcat::mc {Save Configuration}]..." \
	-command GridSaveDialog
    $griddlg(mb).file add separator
    $griddlg(mb).file add command -label [msgcat::mc {Close}] \
	-command GridDestroyDialog
}

proc GridEditMenu {} {
    global griddlg
    global ds9

    $griddlg(mb) add cascade -label [msgcat::mc {Edit}] -menu $griddlg(mb).edit

    menu $griddlg(mb).edit
    $griddlg(mb).edit add command -label [msgcat::mc {Cut}] \
	-command GridCutDialog -accelerator "${ds9(ctrl)}X"
    $griddlg(mb).edit add command -label [msgcat::mc {Copy}] \
	-command GridCopyDialog -accelerator "${ds9(ctrl)}C"
    $griddlg(mb).edit add command -label [msgcat::mc {Paste}] \
	-command GridPasteDialog -accelerator "${ds9(ctrl)}V"
}

proc GridCoordMenu {} {
    global griddlg
    global grid

    $griddlg(mb) add cascade -label [msgcat::mc {Coordinate}] \
	-menu $griddlg(mb).coord

    DefaultCoordMenu $griddlg(mb).coord \
	grid(system) grid(sky) grid(skyformat) UpdateGridDialog
    $griddlg(mb).coord add radiobutton -label {HMS/DMS} \
	-variable grid(skyformat) -value hms -command UpdateGridDialog
}

proc GridTypeMenu {} {
    global griddlg
    global grid

    $griddlg(mb) add cascade -label [msgcat::mc {Type}] -menu $griddlg(mb).type

    menu $griddlg(mb).type
    $griddlg(mb).type add radiobutton -label [msgcat::mc {Analysis}] \
	-variable grid(type) -value analysis
    $griddlg(mb).type add radiobutton -label [msgcat::mc {Publication}] \
	-variable grid(type) -value publication
    $griddlg(mb).type add separator
    $griddlg(mb).type add radiobutton -label [msgcat::mc {Interior Axes}] \
	-variable grid(axes,type) -value interior
    $griddlg(mb).type add radiobutton -label [msgcat::mc {Exterior Axes}] \
	-variable grid(axes,type) -value exterior
    $griddlg(mb).type add separator
    $griddlg(mb).type add radiobutton -label [msgcat::mc {Interior Numerics}] \
	-variable grid(numlab,type) -value interior
    $griddlg(mb).type add radiobutton -label [msgcat::mc {Exterior Numerics}] \
	-variable grid(numlab,type) -value exterior
    $griddlg(mb).type add separator
    $griddlg(mb).type add checkbutton -label [msgcat::mc {Vertical Text}] \
	-variable grid(numlab,vertical)
}

proc GridViewMenu {} {
    global griddlg
    global grid

    $griddlg(mb) add cascade -label [msgcat::mc {View}] -menu $griddlg(mb).view

    menu $griddlg(mb).view
    $griddlg(mb).view add checkbutton -label [msgcat::mc {Grid}] \
	-variable grid(grid)
    $griddlg(mb).view add checkbutton -label [msgcat::mc {Axes}] \
	-variable grid(axes)
    $griddlg(mb).view add checkbutton -label [msgcat::mc {Axes Numbers}] \
	-variable grid(numlab)
    $griddlg(mb).view add checkbutton -label [msgcat::mc {Axes Tickmarks}] \
	-variable grid(tick)
    $griddlg(mb).view add checkbutton -label [msgcat::mc {Axes Label}] \
	-variable grid(textlab)
    $griddlg(mb).view add checkbutton -label [msgcat::mc {Title}] \
	-variable grid(title)
    $griddlg(mb).view add checkbutton -label [msgcat::mc {Border}] \
	-variable grid(border)
}

proc GridColorMenu {} {
    global griddlg
    global grid

    $griddlg(mb) add cascade -label [msgcat::mc {Color}] -menu $griddlg(mb).color

    menu $griddlg(mb).color
    $griddlg(mb).color add cascade -label [msgcat::mc {Grid}] \
	-menu $griddlg(mb).color.grid
    $griddlg(mb).color add cascade -label [msgcat::mc {Axes}] \
	-menu $griddlg(mb).color.axes
    $griddlg(mb).color add cascade -label [msgcat::mc {Axes Numbers}] \
	-menu $griddlg(mb).color.numlab
    $griddlg(mb).color add cascade -label [msgcat::mc {Axes Label}] \
	-menu $griddlg(mb).color.textlab
    $griddlg(mb).color add cascade -label [msgcat::mc {Title}] \
	-menu $griddlg(mb).color.title
    $griddlg(mb).color add cascade -label [msgcat::mc {Tickmarks}] \
	-menu $griddlg(mb).color.tick
    $griddlg(mb).color add cascade -label [msgcat::mc {Border}] \
	-menu $griddlg(mb).color.border

    DefaultColorMenu $griddlg(mb).color.grid grid(grid,color) {}
    DefaultColorMenu $griddlg(mb).color.axes grid(axes,color) {}
    DefaultColorMenu $griddlg(mb).color.title grid(title,color) {}
    DefaultColorMenu $griddlg(mb).color.textlab grid(textlab,color) {}
    DefaultColorMenu $griddlg(mb).color.numlab grid(numlab,color) {}
    DefaultColorMenu $griddlg(mb).color.tick grid(tick,color) {}
    DefaultColorMenu $griddlg(mb).color.border grid(border,color) {}
}

proc GridLineMenu {} {
    global griddlg
    global grid

    $griddlg(mb) add cascade -label [msgcat::mc {Line}] -menu $griddlg(mb).line

    menu $griddlg(mb).line
    $griddlg(mb).line add cascade -label [msgcat::mc {Grid}] \
	-menu $griddlg(mb).line.grid
    $griddlg(mb).line add cascade -label [msgcat::mc {Axes}] \
	-menu $griddlg(mb).line.axes
    $griddlg(mb).line add cascade -label [msgcat::mc {Tickmarks}] \
	-menu $griddlg(mb).line.tick
    $griddlg(mb).line add cascade -label [msgcat::mc {Border}] \
	-menu $griddlg(mb).line.border

    GridCreateLineMenu $griddlg(mb).line.grid grid(grid,width) grid(grid,style)
    GridCreateLineMenu $griddlg(mb).line.axes grid(axes,width) grid(axes,style)
    GridCreateLineMenu $griddlg(mb).line.tick grid(tick,width) grid(tick,style)
    GridCreateLineMenu $griddlg(mb).line.border \
	grid(border,width) grid(border,style)
}

proc GridCreateLineMenu {which var1 var2} {
    global griddlg
    global grid

    DefaultWidthDashMenu $which $var1 {} $var2 {}
}

proc GridFontMenu {} {
    global griddlg
    global grid

    $griddlg(mb) add cascade -label Font -menu $griddlg(mb).font

    menu $griddlg(mb).font
    $griddlg(mb).font add cascade -label [msgcat::mc {Axes Numbers}] \
	-menu $griddlg(mb).font.numlab
    $griddlg(mb).font add cascade -label [msgcat::mc {Axes Label}] \
	-menu $griddlg(mb).font.textlab
    $griddlg(mb).font add cascade -label [msgcat::mc {Title}] \
	-menu $griddlg(mb).font.title

    DefaultFontMenu $griddlg(mb).font.title \
	grid(title,font) grid(title,size) grid(title,style) {}
    DefaultFontMenu $griddlg(mb).font.textlab \
	grid(textlab,font) grid(textlab,size) grid(textlab,style) {}
    DefaultFontMenu $griddlg(mb).font.numlab \
	grid(numlab,font) grid(numlab,size) grid(numlab,style) {}
}

proc GridLoadDialog {} {
    GridLoad [OpenFileDialog gridfbox]
}

proc GridLoad {filename} {
    global grid

    if {$filename != {}} {
	source $filename
    }

    set grid(view) 1
    GridUpdate
}

proc GridSaveDialog {} {
    GridSave [SaveFileDialog gridfbox]
}

proc GridSave {filename} {
    global grid

    if {$filename != {}} {
	set file [open $filename w]
	puts $file "global grid"
	puts $file "array set grid \{ [array get grid] \}"
	close $file
    }
}

proc GridStripComma {str} {
    # strip ','
    set t {}
    regsub -all "," "$str" " " t
    return $t
}

# Process Cmds

proc ProcessGridCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global grid
    switch -- [string tolower [lindex $var $i]] {
	yes -
	true -
	on -
	1 -
	no -
	false -
	off -
	0 {
	    set grid(view) [FromYesNo [lindex $var $i]]
	    GridUpdate
	}

	type {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		axes {
		    # backward compatible
		    incr i; set grid(axes,type) [lindex $var $i]
		}
		numerics {
		    # backward compatible
		    incr i; set grid(numlab,type) [lindex $var $i]
		}
		default {set grid(type) [lindex $var $i]}
	    }
	    GridUpdate
	}

	system {incr i; set grid(system) [lindex $var $i]; GridUpdate}
	sky {incr i; set grid(sky) [lindex $var $i]; GridUpdate}
	skyformat {incr i; set grid(skyformat) [lindex $var $i]; GridUpdate}

	grid {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		color {incr i; set grid(grid,color) [lindex $var $i]}
		width {incr i; set grid(grid,width) [lindex $var $i]}
		style {incr i; set grid(grid,style) [lindex $var $i]}
		gap1 {incr i; set grid(grid,gap1) [lindex $var $i]}
		gap2 {incr i; set grid(grid,gap2) [lindex $var $i]}
		default {set grid(grid) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	axes {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		color {incr i; set grid(axes,color) [lindex $var $i]}
		width {incr i; set grid(axes,width) [lindex $var $i]}
		style {incr i; set grid(axes,style) [lindex $var $i]}
		type {incr i; set grid(axes,type) [lindex $var $i]}
		default {set grid(axes) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	tick {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		color {incr i; set grid(tick,color) [lindex $var $i]}
		width {incr i; set grid(tick,width) [lindex $var $i]}
		style {incr i; set grid(tick,style) [lindex $var $i]}
		default {set grid(tick) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	border {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		color {incr i; set grid(border,color) [lindex $var $i]}
		width {incr i; set grid(border,width) [lindex $var $i]}
		style {incr i; set grid(border,style) [lindex $var $i]}
		default {set grid(border) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	numlab {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		font {incr i; set grid(numlab,font) [lindex $var $i]}
		fontstyle {incr i; set grid(numlab,style) [lindex $var $i]}
		fontsize {incr i; set grid(numlab,size) [lindex $var $i]}
		color {incr i; set grid(numlab,color) [lindex $var $i]}
		gap1 {incr i; set grid(numlab,gap1) [lindex $var $i]}
		gap2 {incr i; set grid(numlab,gap1) [lindex $var $i]}
		type {incr i; set grid(numlab,type) [lindex $var $i]}
		vertical {incr i; set grid(numlab,vertical) [FromYesNo [lindex $var $i]]}
		default {set grid(numlab) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	title {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		text {incr i; set grid(title,text) [lindex $var $i]}
		def {incr i; set grid(title,def) [FromYesNo [lindex $var $i]]}
		gap {incr i; set grid(title,gap) [lindex $var $i]}
		font {incr i; set grid(title,font) [lindex $var $i]}
		fontstyle {incr i; set grid(title,style) [lindex $var $i]}
		fontsize {incr i; set grid(title,size) [lindex $var $i]}
		color {incr i; set grid(title,color) [lindex $var $i]}
		default {set grid(title) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	textlab {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		text1 {incr i; set grid(textlab,text1) [lindex $var $i]}
		def1 {incr i; set grid(textlab,def1) [FromYesNo [lindex $var $i]]}
		gap1 {incr i; set grid(textlab,gap1) [lindex $var $i]}
		text2 {incr i; set grid(textlab,text2) [lindex $var $i]}
		def2 {incr i; set grid(textlab,def2) [FromYesNo [lindex $var $i]]}
		gap2 {incr i; set grid(textlab,gap2) [lindex $var $i]}
		font {incr i; set grid(textlab,font) [lindex $var $i]}
		fontstyle {incr i; set grid(textlab,style) [lindex $var $i]}
		fontsize {incr i; set grid(textlab,size) [lindex $var $i]}
		color {incr i; set grid(textlab,color) [lindex $var $i]}
		default {set grid(textlab) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	view {
	    # backward compatable
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		grid {incr i; set grid(grid) [FromYesNo [lindex $var $i]]}
		axes {
		    incr i
		    switch -- [string tolower [lindex $var $i]] {
			numbers {incr i; set grid(numlab) \
				     [FromYesNo [lindex $var $i]]}
			tickmarks {incr i; set grid(tick) \
				       [FromYesNo [lindex $var $i]]}
			label {incr i; set grid(textlab) \
				   [FromYesNo [lindex $var $i]]}
			default {set grid(axes) [FromYesNo [lindex $var $i]]}
		    }
		}
		title {incr i; set grid(title) [FromYesNo [lindex $var $i]]}
		border {incr i; set grid(border) [FromYesNo [lindex $var $i]]}
		vertical {incr i; set grid(numlab,vertical) [FromYesNo [lindex $var $i]]}
	    }
	    GridUpdate
	}

	reset {GridResetDialog}
	load {
	    incr i
	    set fn [lindex $var $i]
	    FileLast gridfbox $fn

	    GridLoad $fn
	}
	save {
	    incr i
	    set fn [lindex $var $i]
	    FileLast gridfbox $fn

	    GridSave $fn
	}
	default {
	    set grid(view) 1
	    GridUpdate
	    incr i -1
	}
    }
}

proc ProcessSendGridCmd {proc id param} {
    global grid

    switch -- [lindex $param 0] {
	type {
	    switch -- [lindex $param 1] {
		axes {
		    # backward compatible
		    $proc $id "$grid(axes,type)\n"
		}
		numerics {
		    # backward compatible
		    $proc $id "$grid(numlab,type)\n"
		}
		default {$proc $id "$grid(type)\n"}
	    }
	}

	system {$proc $id "$grid(system)\n"}
	sky {$proc $id "$grid(sky)\n"}
	skyformat {$proc $id "$grid(skyformat)\n"}

	grid {
	    switch -- [lindex $param 1] {
		color {$proc $id "$grid(grid,color)\n"}
		width {$proc $id "$grid(grid,width)\n"}
		style {$proc $id "$grid(grid,style)\n"}
		gap1 {$proc $id "$grid(grid,gap1)\n"}
		gap2 {$proc $id "$grid(grid,gap2)\n"}
		default {$proc $id [ToYesNo $grid(grid)]}
	    }
	}

	axes {
	    switch -- [lindex $param 1] {
		color {$proc $id "$grid(axes,color)\n"}
		width {$proc $id "$grid(axes,width)\n"}
		style {$proc $id "$grid(axes,style)\n"}
		type {$proc $id "$grid(axes,type)\n"}
		default {$proc $id [ToYesNo $grid(axes)]}
	    }
	}

	tick {
	    switch -- [lindex $param 1] {
		color {$proc $id "$grid(tick,color)\n"}
		width {$proc $id "$grid(tick,width)\n"}
		style {$proc $id "$grid(tick,style)\n"}
		default {$proc $id [ToYesNo $grid(tick)]}
	    }
	}

	border {
	    switch -- [lindex $param 1] {
		color {$proc $id "$grid(border,color)\n"}
		width {$proc $id "$grid(border,width)\n"}
		style {$proc $id "$grid(border,style)\n"}
		default {$proc $id [ToYesNo $grid(border)]}
	    }
	}

	numlab {
	    switch -- [lindex $param 1] {
		font {$proc $id "$grid(numlab,font)\n"}
		fontstyle {$proc $id "$grid(numlab,style)\n"}
		fontsize {$proc $id "$grid(numlab,size)\n"}
		color {$proc $id "$grid(numlab,color)\n"}
		gap1 {$proc $id "$grid(numlab,gap1)\n"}
		gap2 {$proc $id "$grid(numlab,gap2)\n"}
		type {$proc $id "$grid(numlab,type)\n"}
		vertical {$proc $id "$grid(numlab,vertical)\n"}
		default {$proc $id [ToYesNo $grid(numlab)]}
	    }
	}

	title {
	    switch -- [lindex $param 1] {
		text {$proc $id "$grid(title,text)\n"}
		def {$proc $id [ToYesNo $grid(title,def)]}
		gap {$proc $id "$grid(title,gap)\n"}
		font {$proc $id "$grid(title,font)\n"}
		fontstyle {$proc $id "$grid(title,style)\n"}
		fontsize {$proc $id "$grid(title,size)\n"}
		color {$proc $id "$grid(title,color)\n"}
		default {$proc $id [ToYesNo $grid(title)]}
	    }
	}

	textlab {
	    switch -- [lindex $param 1] {
		text1 {$proc $id "$grid(textlab,text1)\n"}
		def1 {$proc $id [ToYesNo $grid(textlab,def1)]}
		gap1 {$proc $id "$grid(textlab,gap1)\n"}
		text2 {$proc $id "$grid(textlab,text2)\n"}
		def2 {$proc $id [ToYesNo $grid(textlab,def2)]}
		gap2 {$proc $id "$grid(textlab,gap2)\n"}
		font {$proc $id "$grid(textlab,font)\n"}
		fontstyle {$proc $id "$grid(textlab,style)\n"}
		fontsize {$proc $id "$grid(textlab,size)\n"}
		color {$proc $id "$grid(textlab,color)\n"}
		default {$proc $id [ToYesNo $grid(textlab)]}
	    }
	}

	view {
	    # backward compatible
	    switch -- [lindex $param 1] {
		grid {$proc $id [ToYesNo $grid(grid)]}
		axes {
		    switch -- [lindex $param 2] {
			numbers {$proc $id [ToYesNo $grid(numlab)]}
			tickmarks {$proc $id [ToYesNo $grid(tick)]}
			label {$proc $id [ToYesNo $grid(textlab)]}
			default {$proc $id [ToYesNo $grid(axes)]}
		    }
		}
		title {$proc $id [ToYesNo $grid(title)]}
		border {$proc $id [ToYesNo $grid(border)]}
		vertical {$proc $id [ToYesNo $grid(numlab,vertical)]}
	    }
	}
	default {$proc $id [ToYesNo $grid(view)]}
    }
}
