#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CanvasDef {} {
    global canvas
    global ds9

    set canvas(font) "*helvetica-medium-r-*-100-*"
    set canvas(width) 680
    set canvas(height) 450
    set canvas(gap) 4

    switch $ds9(wm) {
	x11 {
	    # this is not fool proof. it does not take into account redirecting
	    # the DISPLAY. There must be a better way.
	    global tcl_platform
	    switch -- $tcl_platform(os) {
		Darwin {set canvas(gap,bottom) 14}
		default {set canvas(gap,bottom) 0}
	    }
	}
	win32 {set canvas(gap,bottom) 0}
	aqua {set canvas(gap,bottom) 14}
    }
}

proc BlinkDef {} {
    global blink
    global pblink

    set blink(interval) 500
    set blink(index) -1
    set blink(id) 0

    set pblink(interval) $blink(interval)
}

proc TileDef {} {
    global tile
    global ptile

    set tile(top) .tile
    set tile(mb) .tilemb

    set tile(mode) grid
    set tile(grid,row) 10
    set tile(grid,col) 10
    set tile(grid,mode) automatic
    set tile(grid,gap) 4

    set ptile(mode) $tile(mode)
}

proc ViewDef {} {
    global view
    global pview

    set view(layout) horizontal
    set view(info) 1
    set view(panner) 1
    set view(magnifier) 1
    set view(buttons) 1
    set view(colorbar) 1
    set view(graph,horz) 0
    set view(graph,vert) 0

    set view(info,filename) 1
    set view(info,object) 1
    set view(info,minmax) 0
    set view(info,lowhigh) 0
    set view(info,wcs) 1
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	set "view(info,wcs$l)" 0
    }
    set view(info,detector) 0
    set view(info,amplifier) 0
    set view(info,physical) 1
    set view(info,image) 1
    set view(info,frame) 1

    array set pview [array get view]
}

# canvas

proc CreateCanvas {} {
    global ds9
    global canvas

    LayoutViewAdjust diff
    set ww [expr $canvas(width)+$diff(x)]
    set hh [expr $canvas(height)+$diff(y)]

    global debug
    if {$debug(tcl,layout)} {
	puts stderr "CreateCanvas $canvas(width) $canvas(height) ${ww}x${hh}"
    }

    set ds9(image) [frame $ds9(main).f -relief ridge -bd 1]
    set ds9(canvas) [canvas $ds9(image).c -width $ww -height $hh \
			 -highlightthickness 0 -insertofftime 0 -bg $ds9(bg)]
#    pack $ds9(canvas) -fill both -expand true
    grid rowconfigure $ds9(image) 0 -weight 1
    grid columnconfigure $ds9(image) 0 -weight 1
    grid $ds9(canvas) -row 0 -column 0 -sticky news

    # extra space for window tab
    if {$canvas(gap,bottom)>0} {
	set f [frame $ds9(image).b -width 1 -height $canvas(gap,bottom) \
		   -bg $ds9(bg)]
	grid $f -row 1 -column 0 -sticky ew
    }
	
    # needed to realize window so Layout routines will work
    grid $ds9(image)
}

proc InitCanvas {} {
    global ds9

    # Bindings
    bind $ds9(canvas) <Tab> [list NextFrame]
    bind $ds9(canvas) <Shift-Tab> [list PrevFrame]
    switch $ds9(wm) {
	x11 {bind $ds9(canvas) <ISO_Left_Tab> [list PrevFrame]} 
	aqua {}
	win32 {}
    }

    bind $ds9(canvas) <Configure> [list ConfigureView]
    switch $ds9(wm) {
	x11 -
	aqua {
	    bind $ds9(canvas) <Enter> [list focus $ds9(canvas)]
	    bind $ds9(canvas) <Leave> [list focus {}]
	}
	win32 {
	    bind $ds9(canvas) <KeyPress> [list focus $ds9(canvas)]
	    bind $ds9(canvas) <Button> [list focus $ds9(canvas)]
	    bind $ds9(canvas) <Leave> [list focus {}]
	}
    }

    # backward compatible bindings
    switch $ds9(wm) {
	x11 -
	win32 {
	    bind $ds9(canvas) <Button-3> {Button3Canvas %x %y}
	    bind $ds9(canvas) <B3-Motion> {Motion3Canvas %x %y}
	    bind $ds9(canvas) <ButtonRelease-3> {Release3Canvas %x %y}
	} 
	aqua {
	    # swap button-2 and button-3 on the mighty mouse
	    bind $ds9(canvas) <Button-2> {Button3Canvas %x %y}
	    bind $ds9(canvas) <B2-Motion> {Motion3Canvas %x %y}
	    bind $ds9(canvas) <ButtonRelease-2> {Release3Canvas %x %y}

	    # x11 command key emulation
	    bind $ds9(canvas) <Command-Button-1> {Button3Canvas %x %y}
	    bind $ds9(canvas) <Command-B1-Motion> {Motion3Canvas %x %y}
	    bind $ds9(canvas) <Command-ButtonRelease-1> {Release3Canvas %x %y}
	}
    }

    # freeze
    bind $ds9(canvas) <f> {ToggleBindEvents}
}

proc Button3Canvas {x y} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts stderr "Button3Canvas"
    }

    set ds9(b3) 1
    ButtonColormap $x $y
}

proc Motion3Canvas {x y} {
    global debug
    if {$debug(tcl,events)} {
	puts stderr "Motion3Canvas"
    }

    MotionColormap $x $y
}

proc Release3Canvas {x y} {
    global ds9
    global debug
    if {$debug(tcl,events)} {
	puts stderr "Release3Canvas"
    }

    set ds9(b3) 0
    ReleaseColormap $x $y
}

proc UnBindEventsCanvas {} {
    global ds9

    foreach f $ds9(active) {
	UnBindEventsFrame $f
    }
}

proc BindEventsCanvas {} {
    global ds9
    global current

    switch -- $ds9(display,mode) {
	single -
	blink {BindEventsFrame $current(frame)}
	tile {
	    foreach f $ds9(active) {
		BindEventsFrame $f
	    }
	}
    }
}

# view

proc ConfigureView {} {
    global ds9
    global canvas

    global debug
    if {$debug(tcl,layout)} {
	puts stderr "ConfigureView old $canvas(width) $canvas(height)"
    }

    # calculate ds9(canvas) size
    LayoutViewAdjust diff

    # adjust window size
    set canvas(width) [expr [winfo width $ds9(canvas)]-$diff(x)]
    set canvas(height) [expr [winfo height $ds9(canvas)]-$diff(y)]

    if {$debug(tcl,layout)} {
	puts stderr "ConfigureView new $canvas(width) $canvas(height)"
    }

    LayoutView
}

proc UpdateView {} {
    global ds9
    global canvas

    # note: assume canvas(width) and canvas(height) have been set to desired
    # values.

    global debug
    if {$debug(tcl,layout)} {
	puts stderr "UpdateView to $canvas(width) x $canvas(height)"
    }

    # save current size
    set wo [winfo width $ds9(top)]
    set ho [winfo height $ds9(top)]

    # calculate ds9(canvas) size
    LayoutViewAdjust diff

    # adjust window size
    set ww [expr $canvas(width)+$diff(x)]
    set hh [expr $canvas(height)+$diff(y)]

    # determine how much to change
    set wc [winfo width $ds9(canvas)]
    set hc [winfo height $ds9(canvas)]

    set wt [winfo width $ds9(top)]
    set ht [winfo height $ds9(top)]
    
    if {$debug(tcl,layout)} {
	puts stderr "UpdateView before ds9(top) $wt x $ht"
    }

    set w [expr $ww - $wc + $wt]
    set h [expr $hh - $hc + $ht]

    if {$debug(tcl,layout)} {
	puts stderr "UpdateView after ds9(top) $w x $h"
    }

    # change window size
    wm geometry $ds9(top) ${w}x${h}

    LayoutView
}

proc LayoutView {} {
    global view

    global debug
    if {$debug(tcl,layout)} {
	puts stderr "LayoutView"
    }

    switch $view(layout) {
	horizontal {LayoutViewHorz}
	vertical {LayoutViewVert}
    }

    LayoutInfoPanel
    LayoutButtons
    LayoutFrames
    LayoutColorbar
    LayoutGraphs
}

proc LayoutViewAdjust {varname} {
    upvar $varname var

    global view
    global colorbar
    global graph
    global canvas

    # these numbers are total arbitrary
    set colorbar(horizontal,gap) [expr .56 +$colorbar(font,size)/30.]
    set colorbar(vertical,gap) [expr .2 +$colorbar(font,size)/50.]
    if {$colorbar(numerics)} {
	set colorbar(horizontal,height) \
	    [expr int($colorbar(thick) + $colorbar(horizontal,gap)*20. + \
			  $colorbar(font,size)*.3) +4]
	set colorbar(vertical,width) \
	    [expr int($colorbar(thick) + $colorbar(font,size)*4) +4]
    } else {
	set colorbar(horizontal,height) [expr $colorbar(thick) +2]
	set colorbar(vertical,width) [expr $colorbar(thick) +2]
    }

    set var(x) 0
    set var(y) 0

    set cbh [expr $view(colorbar) && \
	     ![string compare $colorbar(orientation) {horizontal}]]
    set cbv [expr $view(colorbar) && \
	     ![string compare $colorbar(orientation) {vertical}]]
    set grh $view(graph,horz)
    set grv $view(graph,vert)

    # basics
    if {$cbh} {
	incr var(y) $colorbar(horizontal,height)
    }

    if {$cbv} {
	incr var(x) $colorbar(vertical,width)
    }

    if {$grh} {
	incr var(y)  $graph(size)
    }

    if {$grv} {
	incr var(x)  $graph(size)
    }

    # canvas gap
    if {$cbh || $grh} {
	incr var(y) $canvas(gap)
    }

    if {$cbv || $grv} {
	incr var(x) $canvas(gap)
    }

    # graph gap
    if {$grv && !$cbh} {
	incr var(y) $graph(gap,y)
    }

    if {$grh && !$cbv} {
	incr var(x) $graph(gap,x)
    }

    global debug
    if {$debug(tcl,layout)} {
	puts stderr "LayoutViewAdjust $var(x) $var(y)"
    }
}

proc LayoutViewReset {} {
    global ds9

    global debug
    if {$debug(tcl,layout)} {
	puts stderr "LayoutViewReset"
    }

    # horizontal
    grid rowconfigure $ds9(main) 2 -weight 0
    grid columnconfigure $ds9(main) 0 -weight 0

    # vertical
    grid rowconfigure $ds9(main) 0 -weight 0
    grid columnconfigure $ds9(main) 2 -weight 0

    grid forget $ds9(panel)
    grid forget $ds9(buttons)
    grid forget $ds9(image)
    pack forget $ds9(info)
    pack forget $ds9(panner)
    pack forget $ds9(magnifier)
}

proc LayoutViewHorz {} {
    global ds9
    global current
    global view

    # canvas
    grid rowconfigure $ds9(main) 2 -weight 1
    grid columnconfigure $ds9(main) 0 -weight 1
    grid $ds9(image) -row 2 -column 0 -sticky news

    # info panel
    if {$view(info) || $view(magnifier) || $view(panner)} {
	grid $ds9(panel) -row 0 -column 0 -sticky ew -columnspan 3
    } else {
	grid forget $ds9(panel)
    }

    if {$view(info)} {
	pack $ds9(info) -side left -anchor nw -padx $ds9(internalpad) \
	    -pady $ds9(internalpad) -fill x -expand true
    } else {
	pack forget $ds9(info)
    }

    if {$view(panner)} {
	pack $ds9(panner) -side right -padx $ds9(internalpad) \
	    -pady $ds9(internalpad)
    } else {
	pack forget $ds9(panner)
    }

    if {$view(magnifier)} {
	pack $ds9(magnifier) -side right -padx $ds9(internalpad) \
	    -pady $ds9(internalpad)
	if {$view(panner)} {
	    pack $ds9(magnifier) -before $ds9(panner)
	}
    } else {
	pack forget $ds9(magnifier)
    }

    # buttons
    if {$view(buttons)} {
	grid $ds9(buttons) -row 1 -sticky ew -columnspan 3
    } else {
	grid forget $ds9(buttons)
    }
}

proc LayoutViewVert {} {
    global ds9
    global current
    global view

    # canvas
    grid rowconfigure $ds9(main) 0 -weight 1
    grid columnconfigure $ds9(main) 2 -weight 1
    grid $ds9(image) -row 0 -column 2 -sticky news

    # info panel
    if {$view(info) || $view(magnifier) || $view(panner)} {
	grid $ds9(panel) -row 0 -column 0 -sticky ns
    } else {
	grid forget $ds9(panel)
    }

    if {$view(magnifier)} {
	pack $ds9(magnifier) -side top -padx $ds9(internalpad) \
	    -pady $ds9(internalpad)
    } else {
	pack forget $ds9(magnifier)
    }

    if {$view(info)} {
	pack $ds9(info) -side top -padx $ds9(internalpad) \
	    -pady $ds9(internalpad) -fill y -expand true
	if {$view(magnifier)} {
	    pack $ds9(info) -after $ds9(magnifier)
	}
    } else {
	pack forget $ds9(info)
    }

    if {$view(panner)} {
	pack $ds9(panner) -side bottom -padx $ds9(internalpad) \
	    -pady $ds9(internalpad)
    } else {
	pack forget $ds9(panner)
    }

    # buttons
    if {$view(buttons)} {
	grid $ds9(buttons) -row 0 -column 1 -sticky ns
    } else {
	grid forget $ds9(buttons)
    }
}

proc LayoutFrames {} {
    global ds9
    global current
    global tile
    global view
    global colorbar
    
    # turn everything off
    foreach f $ds9(frames) {
	$f hide
	$f highlite off
	$f panner off
	$f magnifier off
	UnBindEventsFrame $f
    }

    if {$ds9(active,num) > 0} {
	switch -- $ds9(display,mode) {
	    single {TileOne}
	    tile {
		switch -- $tile(mode) {
		    row {TileRect 1 $ds9(active,num) $tile(grid,gap)}
		    column {TileRect $ds9(active,num) 1 $tile(grid,gap)}
		    grid {
			switch -- $tile(grid,mode) {
			    automatic {
				TileRect [expr int(sqrt($ds9(active,num)-1))+1] \
				    [expr int(sqrt($ds9(active,num))+.5)] \
				    $tile(grid,gap)
			    }
			    manual {
				TileRect $tile(grid,row) \
				    $tile(grid,col) \
				    $tile(grid,gap)
			    }
			}
		    }
		}
	    }
	    blink {TileOne}
	}
    } else {
	set current(frame) {}
	set ds9(next) {}

	# panner
	if {$view(panner)} {
	    panner clear
	}

	# magnifier
	if {$view(magnifier)} {
	    magnifier clear
	}

	# process proper colorbar
	switch -- $ds9(visual) {
	    pseudocolor {
		colorbar show
		set current(colorbar) colorbar
		set colorbar(map) [colorbar get name]
		set colorbar(invert) [colorbar get invert]
	    }
	    truecolor {
		colorbar show
		colorbarrgb hide
		set current(colorbar) colorbar
		set colorbar(map) [colorbar get name]
		set colorbar(invert) [colorbar get invert]
	    }
	}

	# update menus/dialogs
	UpdateDS9
    }
}


# This procedure is called when we have only 1 frames to display

proc TileOne {} {
    global ds9
    global view
    global current
    global canvas
    global colorbar

    set ww $canvas(width)
    set hh $canvas(height)

    set xx 0
    set yy 0

    foreach f $ds9(active) {
	$f configure -x $xx -y $yy -width $ww -height $hh -anchor nw
    }

    # only show the current frame
    $current(frame) show
    FrameToFront $current(frame)
}

proc TileRect {numx numy gap} {
    global view
    global canvas

    set ww $canvas(width)
    set hh $canvas(height)

    set w [expr int(($ww-$gap*($numx-1))/$numx)]
    set h [expr int(($hh-$gap*($numy-1))/$numy)]
    
    for {set jj 0} {$jj<$numy} {incr jj} {
	for {set ii 0} {$ii<$numx} {incr ii} {
	    set c [expr $jj*$numx + $ii]
	    set x($c) [expr ($w+$gap)*$ii]
	    set y($c) [expr ($h+$gap)*$jj]
	}
    }

    TileIt $w $h x y [expr $numx*$numy]
}

proc TileIt {ww hh xx yy nn} {
    upvar $xx x
    upvar $yy y
    global ds9
    global current

    set i 0
    foreach f $ds9(active) {
	if {$i<$nn} {
	    $f configure -x $x($i) -y $y($i) -width $ww -height $hh -anchor nw
	    $f show
	    $ds9(canvas) raise $f
	    if {!$ds9(freeze)} {
		BindEventsFrame $f
	    }
	}
	incr i
    }

    # if manual grid, current frame could be not included
    if {$ds9(active,num) > $nn} {
	set current(frame) [lindex $ds9(active) 0]
    }
    FrameToFront $current(frame)
}

proc DisplayDefaultDialog {} {
    global canvas
    global ed

    set w {.defdpy}

    set ed(ok) 0
    set ed(x) $canvas(width)
    set ed(y) $canvas(height)

    DialogCreate $w [msgcat::mc {Display Size}]

    frame $w.ed -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.ed $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.ed.xTitle -text {X}
    label $w.ed.yTitle -text {Y}
    entry $w.ed.x -textvariable ed(x) -width 10
    entry $w.ed.y -textvariable ed(y) -width 10
    label $w.ed.xunit -text [msgcat::mc {Pixels}]
    label $w.ed.yunit -text [msgcat::mc {Pixels}]
    
    grid $w.ed.xTitle $w.ed.x $w.ed.xunit -padx 4 -sticky w
    grid $w.ed.yTitle $w.ed.y $w.ed.yunit -padx 4 -sticky w

    button $w.buttons.ok -text [msgcat::mc {OK}]  \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    $w.ed.x select range 0 end
    DialogWait $w ed(ok) $w.ed.x
    DialogDismiss $w

    if {$ed(ok)} {
	set canvas(width) $ed(x)
	set canvas(height) $ed(y)
	UpdateView
    }

    set rr $ed(ok)
    unset ed
    return $rr
}

# Process Cmds

proc ProcessHeightCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    # we must have a window to change
    RealizeDS9

    global canvas
    set canvas(height) [lindex $var $i]
    UpdateView
}

proc ProcessSendHeightCmd {proc id param} {
    global canvas
    $proc $id "$canvas(height)\n"
}

proc ProcessWidthCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    # we must have a window to change
    RealizeDS9

    global canvas
    set canvas(width) [lindex $var $i]
    UpdateView
}

proc ProcessSendWidthCmd {proc id param} {
    global canvas
    $proc $id "$canvas(width)\n"
}

proc ProcessViewCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global view
    global rgb

    set item [string tolower [lindex $var $i]]

    switch  -- $item {
	layout {
	    incr i
	    set item [string tolower [lindex $var $i]]
	    switch  -- $item {
		horizontal -
		vertical {LayoutViewReset; set view(layout) $item}
	    }
	}
	horizontal -
	vertical {
	    # backward compatibility
	    LayoutViewReset; set view(layout) $item
	}

	default {
	    set yesno [lindex $var [expr $i+1]]
	    switch -- $yesno {
		1 -
		0 -
		yes -
		no -
		on -
		off -
		true -
		false {incr i}
		default {
		    set yesno 1
		}
	    }

	    switch -- $item {
		info {set view(info) [FromYesNo $yesno]}
		panner {set view(panner) [FromYesNo $yesno]}
		magnifier {set view(magnifier) [FromYesNo $yesno]}
		buttons {set view(buttons) [FromYesNo $yesno]}
		colorbar {set view(colorbar) [FromYesNo $yesno]}
		colorbarnumerics {
		    # backward compatibility
		    set colorbar(numerics) [FromYesNo $yesno]
		}
		graph {
		    incr i
		    set item [string tolower [lindex $var $i]]
		    switch -- $item {
			horizontal {
			    incr i
			    set yesno [string tolower [lindex $var $i]]
			    set view(graph,horz) [FromYesNo $yesno]
			}
			vertical {
			    incr i
			    set yesno [string tolower [lindex $var $i]]
			    set view(graph,vert) [FromYesNo $yesno]
			}
		    }
		}
		horzgraph {
		    # backward compatibility
		    set view(graph,horz) [FromYesNo $yesno]
		}
		vertgraph {
		    # backward compatibility
		    set view(graph,vert) [FromYesNo $yesno]
		}

		filename {set view(info,filename) [FromYesNo $yesno]}
		object {set view(info,object) [FromYesNo $yesno]}
		minmax {set view(info,minmax) [FromYesNo $yesno]}
		lowhigh {set view(info,lowhigh) [FromYesNo $yesno]}
		frame {set view(info,frame) [FromYesNo $yesno]}

		detector {set view(info,detector) [FromYesNo $yesno]}
		amplifier {set view(info,amplifier) [FromYesNo $yesno]}
		physical {set view(info,physical) [FromYesNo $yesno]}
		image {set view(info,image) [FromYesNo $yesno]}
		wcs {set view(info,wcs) [FromYesNo $yesno]}
		wcsa {set view(info,wcsa) [FromYesNo $yesno]}
		wcsb {set view(info,wcsb) [FromYesNo $yesno]}
		wcsc {set view(info,wcsc) [FromYesNo $yesno]}
		wcsd {set view(info,wcsd) [FromYesNo $yesno]}
		wcse {set view(info,wcse) [FromYesNo $yesno]}
		wcsf {set view(info,wcsf) [FromYesNo $yesno]}
		wcsg {set view(info,wcsg) [FromYesNo $yesno]}
		wcsh {set view(info,wcsh) [FromYesNo $yesno]}
		wcsi {set view(info,wcsi) [FromYesNo $yesno]}
		wcsj {set view(info,wcsj) [FromYesNo $yesno]}
		wcsk {set view(info,wcsk) [FromYesNo $yesno]}
		wcsl {set view(info,wcsl) [FromYesNo $yesno]}
		wcsm {set view(info,wcsm) [FromYesNo $yesno]}
		wcsn {set view(info,wcsn) [FromYesNo $yesno]}
		wcso {set view(info,wcso) [FromYesNo $yesno]}
		wcsp {set view(info,wcsp) [FromYesNo $yesno]}
		wcsq {set view(info,wcsq) [FromYesNo $yesno]}
		wcsr {set view(info,wcsr) [FromYesNo $yesno]}
		wcss {set view(info,wcss) [FromYesNo $yesno]}
		wcst {set view(info,wcst) [FromYesNo $yesno]}
		wcsu {set view(info,wcsu) [FromYesNo $yesno]}
		wcsv {set view(info,wcsv) [FromYesNo $yesno]}
		wcsw {set view(info,wcsw) [FromYesNo $yesno]}
		wcsx {set view(info,wcsx) [FromYesNo $yesno]}
		wcsy {set view(info,wcsy) [FromYesNo $yesno]}
		wcsz {set view(info,wcsz) [FromYesNo $yesno]}

		red {set rgb(red) [FromYesNo $yesno]; RGBView}
		green {set rgb(green) [FromYesNo $yesno]; RGBView}
		blue {set rgb(blue) [FromYesNo $yesno]; RGBView}
	    }
	}
    }
    UpdateView
}

proc ProcessSendViewCmd {proc id param} {
    global view

    switch -- [string tolower [lindex $param 0]] {
	layout {$proc $id "$view(layout)\n"}
	info {$proc $id [ToYesNo $view(info)]}
	panner {$proc $id [ToYesNo $view(panner)]}
	magnifier {$proc $id [ToYesNo $view(magnifier)]}
	buttons {$proc $id [ToYesNo $view(buttons)]}
	colorbar {$proc $id [ToYesNo $view(colorbar)]}
	colorbarnumerics {
	    # backward compatibility
	    $proc $id [ToYesNo $colorbar(numerics)]
	}
	graph {
	    switch -- [string tolower [lindex $param 1]] {
		horizontal {$proc $id [ToYesNo $view(graph,horz)]}
		vertical {$proc $id [ToYesNo $view(graph,vert)]}
	    }
	}
	horzgraph {
	    # backward compatibility
	    $proc $id [ToYesNo $view(graph,horz)]
	}
	vertgraph {
	    # backward compatibility
	    $proc $id [ToYesNo $view(graph,vert)]
	}

	filename {$proc $id [ToYesNo $view(info,filename)]}
	object {$proc $id [ToYesNo $view(info,object)]}
	minmax {$proc $id [ToYesNo $view(info,minmax)]}
	lowhigh {$proc $id [ToYesNo $view(info,lowhigh)]}
	frame {$proc $id [ToYesNo $view(info,frame)]}

	detector {$proc $id [ToYesNo $view(info,detector)]}
	amplifier {$proc $id [ToYesNo $view(info,amplifier)]}
	physical {$proc $id [ToYesNo $view(info,physical)]}
	image {$proc $id [ToYesNo $view(info,image)]}
	wcs {$proc $id [ToYesNo $view(info,wcs)]}
	wcsa {$proc $id [ToYesNo $view(info,wcsa)]}
	wcsb {$proc $id [ToYesNo $view(info,wcsb)]}
	wcsc {$proc $id [ToYesNo $view(info,wcsc)]}
	wcsd {$proc $id [ToYesNo $view(info,wcsd)]}
	wcse {$proc $id [ToYesNo $view(info,wcse)]}
	wcsf {$proc $id [ToYesNo $view(info,wcsf)]}
	wcsg {$proc $id [ToYesNo $view(info,wcsg)]}
	wcsh {$proc $id [ToYesNo $view(info,wcsh)]}
	wcsi {$proc $id [ToYesNo $view(info,wcsi)]}
	wcsj {$proc $id [ToYesNo $view(info,wcsj)]}
	wcsk {$proc $id [ToYesNo $view(info,wcsk)]}
	wcsl {$proc $id [ToYesNo $view(info,wcsl)]}
	wcsm {$proc $id [ToYesNo $view(info,wcsm)]}
	wcsn {$proc $id [ToYesNo $view(info,wcsn)]}
	wcso {$proc $id [ToYesNo $view(info,wcso)]}
	wcsp {$proc $id [ToYesNo $view(info,wcsp)]}
	wcsq {$proc $id [ToYesNo $view(info,wcsq)]}
	wcsr {$proc $id [ToYesNo $view(info,wcsr)]}
	wcss {$proc $id [ToYesNo $view(info,wcss)]}
	wcst {$proc $id [ToYesNo $view(info,wcst)]}
	wcsu {$proc $id [ToYesNo $view(info,wcsu)]}
	wcsv {$proc $id [ToYesNo $view(info,wcsv)]}
	wcsw {$proc $id [ToYesNo $view(info,wcsw)]}
	wcsx {$proc $id [ToYesNo $view(info,wcsx)]}
	wcsy {$proc $id [ToYesNo $view(info,wcsy)]}
	wcsz {$proc $id [ToYesNo $view(info,wcsz)]}
	default {
	    # backward compatibility
	    $proc $id "$view(layout)\n"
	}
    }
}

