#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc MarkerDef {} {
    global marker
    global pmarker

    set marker(copy) {}
    set marker(show) 1
    set marker(show,text) 1
    set marker(autocentroid) 0
    set marker(shape) circle
    set marker(color) green
    set marker(dashlist) {8 3}
    set marker(width) 1
    set marker(dash) 0
    set marker(fixed) 0
    set marker(edit) 1
    set marker(move) 1
    set marker(rotate) 1
    set marker(delete) 1
    set marker(include) 1
    set marker(source) 1
    set marker(font) helvetica
    set marker(font,size) 10
    set marker(font,style) normal
    set marker(preserve) 0
    set marker(format) ds9
    set marker(system) physical
    set marker(sky) fk5
    set marker(skyformat) degrees
    set marker(strip) 0
    set marker(maxdialog) 48
    set marker(load) current
    set marker(paste,system) wcs
    set marker(paste,sky) fk5

    array set pmarker [array get marker]
    unset pmarker(copy)

    set pmarker(dialog,system) physical
    set pmarker(dialog,sky) fk5
    set pmarker(dialog,skyformat) degrees
    set pmarker(dialog,dist,system) physical
    set pmarker(dialog,dist,format) degrees

    set pmarker(circle,radius) 20
    set pmarker(annulus,inner) 15
    set pmarker(annulus,outer) 30
    set pmarker(annulus,annuli) 1
    set pmarker(panda,inner) 15
    set pmarker(panda,outer) 30
    set pmarker(panda,annuli) 1
    set pmarker(panda,ang1) 0
    set pmarker(panda,ang2) 360
    set pmarker(panda,angnum) 4
    set pmarker(ellipse,radius1) 40
    set pmarker(ellipse,radius2) 20
    set pmarker(ellipseannulus,radius1) 40
    set pmarker(ellipseannulus,radius2) 20
    set pmarker(ellipseannulus,radius3) 60
    set pmarker(ellipseannulus,annuli) 1
    set pmarker(epanda,radius1) 40
    set pmarker(epanda,radius2) 20
    set pmarker(epanda,radius3) 60
    set pmarker(epanda,annuli) 1
    set pmarker(epanda,ang1) 0
    set pmarker(epanda,ang2) 360
    set pmarker(epanda,angnum) 4
    set pmarker(box,radius1) 80
    set pmarker(box,radius2) 40
    set pmarker(boxannulus,radius1) 80
    set pmarker(boxannulus,radius2) 40
    set pmarker(boxannulus,radius3) 120
    set pmarker(boxannulus,annuli) 1
    set pmarker(bpanda,radius1) 80
    set pmarker(bpanda,radius2) 40
    set pmarker(bpanda,radius3) 120
    set pmarker(bpanda,annuli) 1
    set pmarker(bpanda,ang1) 0
    set pmarker(bpanda,ang2) 360
    set pmarker(bpanda,angnum) 4
    set pmarker(polygon,width) 20
    set pmarker(polygon,height) 20
    set pmarker(projection,thick) 0
    set pmarker(compass,radius) 40
    set pmarker(point,size) 11
}

# procs shared between pointer and catalog mode

proc MarkerControl {which x y} {
    global marker
    global ds9
    
    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    # if 3d frame, just return
    switch -- [$which get type] {
	base {}
	3d {return}
	rgb {}
    }

    # we need this cause MarkerMotion maybe called, 
    # and we don't want it
    set marker(motion) none
    set marker(handle) -1

    set id [$which get marker $ds9(mode) id $x $y]
    if {$id} {
	# are we on a selected annulus?
	if {[$which get marker $ds9(mode) select $x $y] == $id} {
	    switch -- [$which get marker $ds9(mode) $id type] {
		annulus {
		    set marker(handle) \
			[$which marker $ds9(mode) $id create annulus radius $x $y]
		    $which marker $ds9(mode) $id edit begin $marker(handle)
		    set marker(motion) edit
		}
		panda {
		    set marker(handle) \
			[$which marker $ds9(mode) $id create panda radius $x $y]
		    $which marker $ds9(mode) $id edit begin $marker(handle)
		    set marker(motion) edit
		}
		ellipseannulus {
		    set marker(handle) \
			[$which marker $ds9(mode) $id create ellipseannulus radius $x $y]
		    $which marker $ds9(mode) $id edit begin $marker(handle)
		    set marker(motion) edit
		}
		epanda {
		    set marker(handle) \
			[$which marker $ds9(mode) $id create epanda radius $x $y]
		    $which marker $ds9(mode) $id edit begin $marker(handle)
		    set marker(motion) edit
		}
		boxannulus {
		    set marker(handle) \
			[$which marker $ds9(mode) $id create boxannulus radius $x $y]
		    $which marker $ds9(mode) $id edit begin $marker(handle)
		    set marker(motion) edit
		}
		bpanda {
		    set marker(handle) \
			[$which marker $ds9(mode) $id create bpanda radius $x $y]
		    $which marker $ds9(mode) $id edit begin $marker(handle)
		    set marker(motion) edit
		}
	    }
	}
    }
}

proc MarkerControlShift {which x y} {
    global marker
    global ds9
    
    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    # if 3d frame, just return
    switch -- [$which get type] {
	base {}
	3d {return}
	rgb {}
    }

    # we need this cause MarkerMotion maybe called, 
    # and we don't want it
    set marker(motion) none
    set marker(handle) -1

    set id [$which get marker $ds9(mode) id $x $y]
    if {$id} {
	# are we on a selected annulus?
	if {[$which get marker $ds9(mode) select $x $y] == $id} {
	    switch -- [$which get marker $ds9(mode) $id type] {
		panda {
		    set marker(handle) \
			[$which marker $ds9(mode) $id create panda angle $x $y]
		    $which marker $ds9(mode) $id edit begin $marker(handle)
		    set marker(motion) edit
		}
		epanda {
		    set marker(handle) \
			[$which marker $ds9(mode) $id create epanda angle $x $y]
		    $which marker $ds9(mode) $id edit begin $marker(handle)
		    set marker(motion) edit
		}
		bpanda {
		    set marker(handle) \
			[$which marker $ds9(mode) $id create bpanda angle $x $y]
		    $which marker $ds9(mode) $id edit begin $marker(handle)
		    set marker(motion) edit
		}
	    }
	}
    }
}

proc MarkerCursor {which x y handleCursor overCursor} {
    global ds9

    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    # if 3d frame, just return
    switch -- [$which get type] {
	base {}
	3d {return}
	rgb {}
    }

    # are we over any selected marker handles?
    # remember, handles are outside of a marker
    set h [$which get marker $ds9(mode) handle $x $y]
    set id [lindex $h 0]
    set handle [lindex $h 1]
    if {$handle} {
	if {$handle < 5} {
	    # edit/rotate handle
	    SetCursor $handleCursor
	} else { 
	    # polygon/annulus vertex
	    SetCursor dotbox
	}
	return
    }

    # else, see if we are on a segement of a polygon
    set h [$which get marker $ds9(mode) polygon segment $x $y]
    if {[lindex $h 0]} {
	SetCursor draped_box
	return
    }

    # are we over a marker?
    set id [$which get marker $ds9(mode) select $x $y]
    if {$id} {
	# are we on a selected annulus and control key down?
	switch -- [$which get marker $ds9(mode) $id type] {
	    annulus -
	    panda -
	    ellipseannulus -
	    epanda -
	    boxannulus -
	    bpanda {SetCursor $overCursor}
	    default {SetCursor fleur}
	}
	return
    }

    # else, set no cursor
    SetCursor {}
}

proc MarkerArrowKey {which x y} {
    global ds9

    # if 3d frame, just return
    switch -- [$which get type] {
	base {}
	3d {return}
	rgb {}
    }

    $which warp $x $y
    $which marker $ds9(mode) move $x $y
}

# Marker only

proc MarkerButton {which x y} {
    global marker
    global template
    global ds9

    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    # if 3d frame, just return
    switch -- [$which get type] {
	base {}
	3d {return}
	rgb {}
    }

    # see if we are on a handle
    set h [$which get marker handle $x $y]
    set id [lindex $h 0]
    set marker(handle) [lindex $h 1]

    if {$marker(handle)} {
	$which marker $id edit begin $marker(handle)
	set marker(motion) beginEdit
	return
    }

    # else, see if we are on a segment of a polygon
    set h [$which get marker polygon segment $x $y]
    set id [lindex $h 0]
    set segment [lindex $h 1]
    if {$segment} {
	$which marker $id create polygon vertex $segment $x $y
	$which marker $id edit begin $marker(handle)
	set marker(handle) [expr 4+$segment+1]
	set marker(motion) beginEdit
	return
    }

    # else, see if we are on a marker
    if {[$which get marker id $x $y]} {
	$which marker select only $x $y
	$which marker move begin $x $y
	set marker(motion) beginMove
	UpdateRegionMenu
	return
    }

    # see if any markers are selected
    if {[$which get marker select number]>0} {
	$which marker unselect all
	set marker(motion) none
	UpdateRegionMenu
	return
    }

    # else, create a marker
    set marker(handle) 0
    set marker(motion) none

    switch -- $marker(shape) {
	circle -
	annulus -
	panda -
	ellipse -
	ellipseannulus -
	epanda -
	box -
	boxannulus -
	bpanda -
	polygon -
	line -
	vector -
	text -
	ruler -
	compass -
	projection -
	"circle point" -
	"box point" -
	"diamond point" -
	"cross point" -
	"x point" -
	"arrow point" -
	"boxcircle point" {MarkerCreateShape $which $x $y}
	default {
	    set fn "$ds9(root)/template/$template($marker(shape))"
	    set ch [open $fn r]

	    global vardata
	    set vardata [read $ch]
	    close $ch

	    $which marker create template var vardata $x $y
	}
    }
}

proc MarkerShift {which x y} {
    global marker

    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    # if 3d frame, just return
    switch -- [$which get type] {
	base {}
	3d {return}
	rgb {}
    }

    # see if we are on a handle
    set h [$which get marker handle $x $y]
    set id [lindex $h 0]
    set marker(handle) [lindex $h 1]

    if {$marker(handle)} {
	$which marker $id rotate begin
	set marker(motion) beginRotate
	return
    }

    # else, see if we are on a marker
    if {[$which marker select toggle $x $y]} {
	UpdateRegionMenu
	$which marker move begin $x $y
	set marker(motion) beginMove
	return
    }

    # else, start a region select
    $which region select begin $x $y
    set marker(motion) shiftregion
}

proc MarkerMotion {which x y} {
    global marker

    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    # if 3d frame, just return
    switch -- [$which get type] {
	base {}
	3d {return}
	rgb {}
    }

    switch -- $marker(motion) {
	none {}

	beginCreate -
	create {
	    $which marker edit motion $x $y $marker(handle)
	    set marker(motion) create
	}

	beginMove -
	move {
	    $which marker move motion $x $y
	    set marker(motion) move
	}

	beginEdit -
	edit {
	    $which marker edit motion $x $y $marker(handle)
	    set marker(motion) edit
	}

	beginRotate -
	rotate {
	    $which marker rotate motion $x $y $marker(handle)
	    set marker(motion) rotate
	}

	region -
	shiftregion {$which region select motion $x $y}
    }
}

proc MarkerRelease {which x y} {
    global marker

    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    # if 3d frame, just return
    switch -- [$which get type] {
	base {}
	3d {return}
	rgb {}
    }

    switch -- $marker(motion) {
	none {}
	beginCreate {
	    # the user has just clicked, so resize to make visible or delete
	    # assumes marker(id) from create
	    $which marker edit end
	    MarkerDefault $which

	    if {$marker(autocentroid)} {
		$which marker centroid $marker(id) 
	    }
	    unset marker(createx)
	    unset marker(createy)
	    unset marker(id)
	}
	create {
	    $which marker edit end

	    # determine if this is an accident and just create the default
	    set diffx [expr $x-$marker(createx)]
	    set diffy [expr $y-$marker(createy)]
	    if {[expr sqrt($diffx*$diffx + $diffy*$diffy)]<2} {
		MarkerDefault $which
	    }

	    # special callbacks
	    switch [$which get marker $marker(id) type] {
		projection {
		    $which marker $marker(id) callback move ProjectionPlot $which
		    $which marker $marker(id) callback edit ProjectionPlot $which
		    $which marker $marker(id) callback update ProjectionPlot $which
		    $which marker $marker(id) callback delete ProjectionPlotDelete $which
		    ProjectionPlot $which $marker(id)
		}
	    }

	    if {$marker(autocentroid)} {
		$which marker centroid $marker(id) 
	    }

	    unset marker(createx)
	    unset marker(createy)
	    unset marker(id)
	}
	beginMove -
	beginRotate {}
	beginEdit {}
	move {
	    $which marker move end
	    if {$marker(autocentroid)} {
		$which marker centroid
	    }
	}
	edit {
	    $which marker edit end
	    if {$marker(autocentroid)} {
		$which marker centroid
	    }
	}
	rotate {
	    $which marker rotate end
	    if {$marker(autocentroid)} {
		$which marker centroid
	    }
	}
	region {$which region select end}
	shiftregion {$which region select shift end}
    }

    unset marker(motion)
    unset marker(handle)
}

proc MarkerDouble {which x y} {
    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    # if 3d frame, just return
    switch -- [$which get type] {
	base {}
	3d {return}
	rgb {}
    }

    set id [$which get marker id $x $y]

    if {$id} {
	if [$which get marker $id PROPERTY SELECT] {
	    MarkerDialog $which $id
	}
    }
}

proc MarkerCreateShape {which x y} {
    global marker
    global pmarker

    set cmd "$which marker create $marker(shape) $x $y"
    switch -- $marker(shape) {
	circle {append cmd " 0"}
	annulus {append cmd " .001 .002 $pmarker(annulus,annuli)"}
	panda {append cmd " $pmarker(panda,ang1) $pmarker(panda,ang2) $pmarker(panda,angnum) .001 .002 $pmarker(panda,annuli)"}
	ellipse {append cmd " 0 0"}
	ellipseannulus {append cmd " .001 .001 .002 $pmarker(ellipseannulus,annuli)"}
	epanda {append cmd " $pmarker(epanda,ang1) $pmarker(epanda,ang2) $pmarker(epanda,angnum) .001 .001 .002 $pmarker(epanda,annuli)"}
	box {append cmd " 0 0"}
	boxannulus {append cmd " .002 .002 .004 $pmarker(boxannulus,annuli)"}
	bpanda {append cmd " $pmarker(bpanda,ang1) $pmarker(bpanda,ang2) $pmarker(bpanda,angnum) .001 .001 .002 $pmarker(bpanda,annuli)"}
	polygon {append cmd " .001 .001"}
	line {append cmd " $x $y"}
	vector {append cmd " $x $y"}
	text {
	    set txt "Region"
	    set r [EntryDialog "Text Region" "Enter Text:" 40 txt]
	    if {$r == 1 && $txt != {}} {
		append cmd " 0 text = \{\{$txt\}\}"
	    } else {
		return
	    }
	}
	ruler {append cmd " $x $y $pmarker(dialog,system) $pmarker(dialog,sky) $pmarker(dialog,dist,system) $pmarker(dialog,dist,format)"}
	compass {append cmd " 15 $pmarker(dialog,system) $pmarker(dialog,sky) "}
	projection {append cmd " $x $y $pmarker(projection,thick) "}
	"circle point" -
	"box point" -
	"diamond point" -
	"cross point" -
	"x point" -
	"arrow point" -
	"boxcircle point" {append cmd " $pmarker(point,size)"}
    }
    append cmd " color = $marker(color)"
    append cmd " width = $marker(width)"
    append cmd " font = \{\"$marker(font) $marker(font,size) $marker(font,style)\"\}"
    append cmd " dash = $marker(dash)"
    append cmd " fixed = $marker(fixed)"
    append cmd " edit = $marker(edit)"
    append cmd " move = $marker(move)"
    append cmd " rotate = $marker(rotate)"
    append cmd " delete = $marker(delete)"
    append cmd " include = $marker(include)"
    append cmd " source = $marker(source)"

    $which marker unselect all

    set marker(id) [eval $cmd]
    set marker(motion) beginCreate
    set marker(createx) $x
    set marker(createy) $y

    switch -- $marker(shape) {
	circle -
	annulus -
	panda -
	ellipse -
	ellipseannulus -
	epanda -
	box -
	boxannulus -
	bpanda -
	compass -
	polygon {
	    set marker(handle) 1
	    $which marker $marker(id) edit begin $marker(handle)
	}
	line -
	vector -
	ruler -
	projection {
	    set marker(handle) 2
	    $which marker $marker(id) edit begin $marker(handle)
	}
    }
}

proc MarkerDefault {which} {
    global marker
    global pmarker
    global current

    # scale the default size to take into account the current
    set z1 double([lindex $current(zoom) 0])
    set z2 double([lindex $current(zoom) 1])
    if {$z1>$z2} {
	set zz $z1
    } else {
	set zz $z2
    }

    switch -- [$which get marker $marker(id) type] {
	circle {
	    $which marker $marker(id) circle radius \
		[expr ($pmarker(circle,radius)/$zz)] \
		image degrees
	}
	annulus {
	    $which marker $marker(id) annulus radius \
		[expr ($pmarker(annulus,inner)/$zz)] \
		[expr ($pmarker(annulus,outer)/$zz)] \
		$pmarker(annulus,annuli) image degrees
	}
	panda {
	    $which marker $marker(id) panda edit \
		$pmarker(panda,ang1) $pmarker(panda,ang2) \
		$pmarker(panda,angnum) \
		[expr ($pmarker(panda,inner)/$zz)] \
		[expr ($pmarker(panda,outer)/$zz)] \
		$pmarker(panda,annuli) image
	}
	ellipse {
	    $which marker $marker(id) ellipse radius \
		[expr ($pmarker(ellipse,radius1)/$z1)] \
		[expr ($pmarker(ellipse,radius2)/$z2)] \
		image degrees
	}
	ellipseannulus {
	    $which marker $marker(id) ellipseannulus radius \
	      [expr ($pmarker(ellipseannulus,radius1)/$z1)] \
	      [expr ($pmarker(ellipseannulus,radius2)/$z2)] \
	      [expr ($pmarker(ellipseannulus,radius3)/$z1)] \
		$pmarker(ellipseannulus,annuli) image
	}
	epanda {
	    $which marker $marker(id) epanda edit \
		$pmarker(epanda,ang1) $pmarker(epanda,ang2) \
		$pmarker(epanda,angnum) \
		[expr ($pmarker(epanda,radius1)/$z1)] \
		[expr ($pmarker(epanda,radius2)/$z2)] \
		[expr ($pmarker(epanda,radius3)/$z1)] \
		$pmarker(epanda,annuli) image
	}
	box {
	    $which marker $marker(id) box radius \
		[expr ($pmarker(box,radius1)/$z1)] \
		[expr ($pmarker(box,radius2)/$z2)] \
		image degrees
	}
	boxannulus {
	    $which marker $marker(id) boxannulus radius \
		[expr ($pmarker(boxannulus,radius1)/$z1)] \
		[expr ($pmarker(boxannulus,radius2)/$z2)] \
		[expr ($pmarker(boxannulus,radius3)/$z1)] \
		$pmarker(boxannulus,annuli) image
	}
	bpanda {
	    $which marker $marker(id) bpanda edit \
		$pmarker(bpanda,ang1) $pmarker(bpanda,ang2) \
		$pmarker(bpanda,angnum) \
		[expr ($pmarker(bpanda,radius1)/$z1)] \
		[expr ($pmarker(bpanda,radius2)/$z2)] \
		[expr ($pmarker(bpanda,radius3)/$z1)] \
		$pmarker(bpanda,annuli) image
	}
	compass {
	    $which marker $marker(id) compass radius $pmarker(compass,radius) \
		image degrees
	}
	polygon {
	    $which marker $marker(id) polygon reset \
		[expr ($pmarker(polygon,width)/$z1)] \
		[expr ($pmarker(polygon,height)/$z2)] \
		image degrees
	}
	line -
	vector -
	ruler -
	projection {$which marker $marker(id) delete}
    }
}

proc MarkerDeleteKey {which x y} {
    # if nothing is loaded, abort
    if {![$which has fits]} {
	return
    }

    # if 3d frame, just return
    switch -- [$which get type] {
	base {}
	3d {return}
	rgb {}
    }

    # see if we are on a polygon
    set h [$which get marker handle $x $y]
    set id [lindex $h 0]
    set handle [lindex $h 1]

    set t [$which get marker $id type]
    switch -- $t {
	polygon -
	annulus -
	panda -
	ellipseannulus -
	epanda -
	boxannulus -
	bpanda {
	    if {$handle > 4} {
		switch -- $t {
		    polygon {$which marker $id delete polygon vertex $handle}
		    annulus {$which marker $id delete annulus $handle}
		    panda {$which marker $id delete panda $handle}
		    ellipseannulus {$which marker $id delete \
					ellipseannulus $handle}
		    epanda {$which marker $id delete epanda $handle}
		    boxannulus {$which marker $id delete boxannulus $handle}
		    bpanda {$which marker $id delete bpanda $handle}
		}
	    } else {
		# delete polygon
		$which marker delete
		UpdateGroupDialog
	    }
	}
	default {
	    # delete marker
	    $which marker delete
	    UpdateGroupDialog
	}
    }
}

proc MarkerShow {} {
    global current
    global marker

    if {$current(frame) != {}} {
	$current(frame) marker show $marker(show)
    }
}

proc MarkerShowText {} {
    global current
    global marker

    if {$current(frame) != {}} {
	$current(frame) marker show text $marker(show,text)
    }
}

proc MarkerAutoCentroid {} {
    global current
    global marker

    if {$current(frame) != {}} {
	$current(frame) marker centroid auto $marker(autocentroid)
    }
}

proc MarkerPreserve {} {
    global current
    global marker

    if {$current(frame) != {}} {
	$current(frame) marker preserve $marker(preserve)
    }
}

proc MarkerCentroid {} {
    global current
    
    if {$current(frame) != {}} {
	$current(frame) marker centroid
    }
}

proc MarkerFront {} {
    global current
    
    if {$current(frame) != {}} {
	$current(frame) marker move front
    }
}

proc MarkerBack {} {
    global current
    
    if {$current(frame) != {}} {
	$current(frame) marker move back
	$current(frame) marker unselect all
    }
}

proc MarkerSelectAll {} {
    global current
    
    if {$current(frame) != {}} {
	$current(frame) marker select all
    }

    UpdateEditMenu
}

proc MarkerUnselectAll {} {
    global current
    
    if {$current(frame) != {}} {
	$current(frame) marker unselect all
    }

    UpdateEditMenu
}

proc MarkerSelectInvert {} {
    global current
    
    if {$current(frame) != {}} {
	$current(frame) marker select toggle
    }

    UpdateEditMenu
}

proc MarkerDeleteSelect {} {
    global current
    
    if {$current(frame) != {}} {
	$current(frame) marker delete
	UpdateGroupDialog
    }

    UpdateEditMenu
}

proc MarkerDeleteAllMenu {} {
    global current
    global ds9

    if {$ds9(confirm)} {
	if {[tk_messageBox -type okcancel -icon question -message [msgcat::mc {Delete All Regions?}]] != {ok}} {
	    return
	}
    }
    MarkerDeleteAll
}

proc MarkerDeleteAll {} {
    global current
    
    if {$current(frame) != {}} {
	$current(frame) marker delete all
	UpdateGroupDialog
    }

    UpdateEditMenu
}

proc MarkerColor {} {
    global current
    global marker
    
    if {$current(frame) != {}} {
	$current(frame) marker color $marker(color)
    }
}

proc MarkerWidth {} {
    global current
    global marker
    
    if {$current(frame) != {}} {
	$current(frame) marker width $marker(width)
    }
}

proc MarkerProp {prop} {
    global current
    global marker

    if {$current(frame) != {}} {
	$current(frame) marker property $prop $marker($prop)
    }
}

proc MarkerFont {} {
    global current
    global marker

    if {$current(frame) != {}} {
	$current(frame) marker font \
	    \"$marker(font) $marker(font,size) $marker(font,style)\"
    }
}

proc MarkerList {} {
    global current
    global marker

    if {$current(frame) == {}} {
	return
    }
    if {![$current(frame) has fits]} {
	return
    }

    if [MarkerSaveDialog [msgcat::mc {List Regions}]] {
	SimpleTextDialog markertxt [msgcat::mc {Region}] 80 20 insert top \
	    [$current(frame) marker list $marker(format) $marker(system) \
		 $marker(sky) $marker(skyformat) $marker(strip)]
    }
}

proc MarkerLoad {} {
    global ds9
    global current
    global marker

    if {$current(frame) == {}} {
	return
    }

    if {![$current(frame) has fits]} {
	return
    }

    set fns [OpenFileDialog markerfbox]
    if {$fns != {}} {
	if [MarkerLoadDialog] {
	    switch -- $marker(load) {
		current {set frames $current(frame)}
		all {set frames $ds9(frames)}
	    }
	    MarkerLoadFrames $fns $frames
	    FileLastFull markerfbox $fns
	}
    }
}

proc MarkerLoadFrames {str frames} {
    if {$str == {}} {
	return
    }
    
    if {[catch {glob $str} fns]} {
	# could be an unique name, i.e. foo[bar], just try to load
	foreach fr $frames {
	    if {[catch {MarkerLoadFile $str $fr}]} {
		return
	    }
	}
    } else {
	foreach fn $fns {
	    foreach fr $frames {
		if {[catch {MarkerLoadFile $fn $fr}]} {
		    return
		}
	    }
	}
    }
}

proc MarkerLoadFile {filename which} {
    global current
    global marker

    if {$filename == {}} {
	return
    }

    if {![$which has fits]} {
	return
    }

    # if 3d frame, just return
    switch -- [$which get type] {
	base {}
	3d {return}
	rgb {}
    }

    # determine if its a fits file
    # first, strip the filename
    if {![regexp -nocase {(.*)(\[.*\])} $filename foo base ext]} {
	set base $filename
	set ext {}
    }

    if {[catch {open $base} fd]} {
	Error [msgcat::mc {Unable to load region file}]
	return -code error
    }

    set ll [read $fd 9]
    close $fd

    # is it a fits file?
    if {$ll == "SIMPLE  ="} {
	# see if we need to add an extension
	if {$ext == {}} {
	    set filename "$base\[REGION\]"
	}

	# open it
	if [catch {$which marker load fits "\{$filename\}" $marker(color) $marker(dashlist) $marker(width) "\{$marker(font) $marker(font,size) $marker(font,style)\}"}] {
	    if {$ext == {}} {
		# ok now try the first extension
		set filename "$base\[1\]"
		if [catch {$which marker load fits "\{$filename\}" $marker(color) $marker(dashlist) $marker(width) "\{$marker(font) $marker(font,size) $marker(font,style)\}"}] {
		    Error [msgcat::mc {Unable to load region file}]
		    return -code error
		}
	    } else {
		Error [msgcat::mc {Unable to load region file}]
		return -code error
	    }
	}
    } else {
	# no, its ascii
	switch -- $marker(format) {
	    xy {
		if [catch {$which marker load \
			       $marker(format) "\{$filename\}" \
			       $marker(system) $marker(sky)}] {
		    Error [msgcat::mc {Unable to load region file}]
		    return -code error
		}
	    }
	    default {
		if [catch {$which marker load \
			       $marker(format) "\{$filename\}"}] {
		    Error [msgcat::mc {Unable to load region file}]
		    return -code error
		}
	    }
	}
    }

    UpdateGroupDialog
}

proc MarkerSave {} {
    global current
    global marker

    if {$current(frame) == {}} {
	return
    }
    if {![$current(frame) has fits]} {
	return
    }

    set filename [SaveFileDialog markerfbox]
    if {$filename == {}} {
	return
    }

    if [MarkerSaveDialog [msgcat::mc {Save Regions}]] {
	$current(frame) marker save "\{$filename\}" \
	    $marker(format) $marker(system) $marker(sky) \
	    $marker(skyformat) $marker(strip)
    }
}

proc MarkerInfo {} {
    global current
    global marker
    global ds9

    if {$current(frame) != {}} {
	set ll [$current(frame) get marker select]
	if {$ll != {}} {
	    set ii 0
	    foreach dd $ll {
		incr ii
		if {$ii > $marker(maxdialog)} {
		    return
		}
		MarkerDialog $current(frame) $dd
	    }
	} else {
	    if {$ds9(confirm)} {
		tk_messageBox -type ok -icon info -message [msgcat::mc {Please Select a Region}]
	    }
	}
    }
}

proc MarkerDialog {frame id} {

    set type [$frame get marker $id type]

    switch -- $type {
	circle {CircleDialog $frame $id}
	annulus {AnnulusDialog $frame $id}
	panda {PandaDialog $frame $id}
	ellipse {EllipseDialog $frame $id}
	ellipseannulus {EllipseAnnulusDialog $frame $id}
	epanda {EpandaDialog $frame $id}
	box {BoxDialog $frame $id}
	boxannulus {BoxAnnulusDialog $frame $id}
	bpanda {BpandaDialog $frame $id}
	polygon {PolygonDialog $frame $id}
	line {LineDialog $frame $id}
	vector {VectorDialog $frame $id}
	text {TextDialog $frame $id}
	ruler {RulerDialog $frame $id}
	compass {CompassDialog $frame $id}
	projection {ProjectionDialog $frame $id}
	{circle point} -
	{box point} -
	{diamond point} -
	{cross point} -
	{x point} -
	{arrow point} -
	{boxcircle point} {PointDialog $frame $id}
	composite {CompositeDialog $frame $id}
    }
}

proc MarkerCopy {} {
    global current
    global marker

    if {$current(frame) != {}} {
	$current(frame) marker copy
	set marker(copy) $current(frame)
    }
    UpdateEditMenu
}

proc MarkerCut {} {
    global current
    global marker

    if {$current(frame) != {}} {
	$current(frame) marker cut
	set marker(copy) $current(frame)
    }
    UpdateEditMenu
    UpdateGroupDialog
}

proc MarkerUndo {} {
    global current

    if {$current(frame) != {}} {
	$current(frame) marker undo
    }
    UpdateEditMenu
    UpdateGroupDialog
}

proc MarkerPaste {prompt} {
    global current
    global marker

    # if nothing is loaded, abort
    if {$current(frame) == {}} {
	return
    }

    # if 3d frame, just return
    switch -- [$current(frame) get type] {
	base {}
	3d {return}
	rgb {}
    }

    if {![$current(frame) has fits]} {
	return
    }

    if {$marker(copy) == {}} {
	return
    }
    if {![$marker(copy) has fits]} {
	return
    }

    # same frame?
    if {$current(frame) == $marker(copy)} {
	$current(frame) marker paste
    } else {
	if {$prompt} {
	    if {![MarkerPasteDialog]} {
		return
	    }
	}

	global cmd
	set cmd "[$marker(copy) marker paste $marker(paste,system) $marker(paste,sky)]"
	$current(frame) marker command ds9 var cmd
	unset cmd
    }
	
    UpdateEditMenu
    UpdateGroupDialog
}

proc MarkerPasteDialog {} {
    global current
    global marker
    global ds9
    global ed

    set w {.mkld}

    set ed(ok) 0
    set ed(system) $marker(paste,system)
    set ed(sky) $marker(paste,sky)

    DialogCreate $w [msgcat::mc {Paste Regions}]

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons -side top -ipadx 4 -ipady 4 -fill x -expand true

    label $w.param.coordtitle -text [msgcat::mc {Coordinate System}]
    menubutton $w.param.coordbutton -relief raised \
	-menu $w.param.coordbutton.menu -textvariable ed(system) \
	-width 8

    DefaultCoordMenu2 $w.param.coordbutton.menu ed(system) ed(sky) {}

    grid rowconfigure $w.param 0 -pad 0
    grid $w.param.coordtitle $w.param.coordbutton -padx 4 -sticky w

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok)
    DialogDismiss $w

    if {$ed(ok)} {
	set marker(paste,system) $ed(system)
	set marker(paste,sky) $ed(sky)
    }

    set rr $ed(ok)
    unset ed
    return $rr
}

proc CompositeCreate {} {
    global current
    global marker

    if {$current(frame) != {}} {
	set cmd "$current(frame) marker create composite"
	append cmd " color = $marker(color)"
	append cmd " width = $marker(width)"
	append cmd " font = \{\"$marker(font) $marker(font,size) $marker(font,style)\"\}"
	append cmd " dash = $marker(dash)"
	append cmd " edit = $marker(edit)"
	append cmd " move = $marker(move)"
	append cmd " rotate = $marker(rotate)"
	append cmd " delete = $marker(delete)"
	append cmd " fixed = $marker(fixed)"
	append cmd " include = $marker(include)"
	append cmd " source = $marker(source)"

	eval $cmd
    }
}

proc CompositeDelete {} {
    global current

    if {$current(frame) != {}} {
	$current(frame) marker composite delete
    }
}

# Process Cmds

proc ProcessRegionsCmd {varname iname sock fn} {
    upvar $varname var
    upvar $iname i

    global ds9
    global current
    global marker

    switch -- [string tolower [lindex $var $i]] {
	show {
	    incr i
	    set marker(show) [FromYesNo [lindex $var $i]]
	    MarkerShow
	}
	showtext {
	    incr i
	    set marker(show,text) [FromYesNo [lindex $var $i]]
	    MarkerShowText
	}
	autocentroid {
	    incr i
	    set marker(autocentroid) [FromYesNo [lindex $var $i]]
	    MarkerAutoCentroid
	}
	getinfo {MarkerInfo}
	centroid {MarkerCentroid}
	movefront {MarkerFront}
	moveback {MarkerBack}
	move {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		front {MarkerFront}
		back {MarkerBack}
	    }
	}

	selectall {MarkerSelectAll}
	selectnone {MarkerUnselectAll}
	select {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		group {
		    incr i
		    $current(frame) marker "\{[lindex $var $i]\}" select
		}
		all {MarkerSelectAll}
		none {MarkerUnselectAll}
	    }
	}

	deleteall {MarkerDeleteAll}
	delete {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		select {MarkerDeleteSelect}
		all {MarkerDeleteAll}
	    }
	}

	format {
	    incr i
	    set marker(format) [string tolower [lindex $var $i]]
	}
	coord -
	system {
	    # for backward compatibility
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		fk4 -
		b1950 -
		fk5 -
		j2000 -
		icrs -
		galactic -
		ecliptic {
		    incr i
		    set marker(system) wcs
		    set marker(sky) [string tolower [lindex $var $i]]
		}
		
		default {set marker(system) [string tolower [lindex $var $i]]}
	    }
	}
	sky {
	    incr i
	    set marker(sky) [string tolower [lindex $var $i]]
	}
	coordformat -
	skyformat {
	    incr i
	    set marker(skyformat) [string tolower [lindex $var $i]]
	}
	strip {
	    incr i
	    set marker(strip) [FromYesNo [lindex $var $i]]
	}
	delim {
	    incr i
	    if {[lindex $var $i] != "nl"} {
		set marker(strip) 1
	    } else {
		set marker(strip) 0
	    }
	}
	shape {
	    incr i
	    set marker(shape) [string tolower [lindex $var $i]]
	}
	color {
	    incr i
	    set marker(color) [string tolower [lindex $var $i]]
	    MarkerColor
	}
	width {
	    incr i
	    set marker(width) [lindex $var $i]
	    MarkerWidth
	}

	tag -
	tags -
	group -
	groups {
	    incr i
	    set tag "\{[lindex $var $i]\}"
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		color {
		    incr i
		    $current(frame) marker $tag color [string tolower [lindex $var $i]]
		}
		copy {$current(frame) marker $tag copy}
		delete {$current(frame) marker $tag delete}
		cut {$current(frame) marker $tag cut}
		font {
		    incr i
		    $current(frame) marker $tag font "\{[lindex $var $i]\}"
		}
		move {
		    $current(frame) marker $tag move \
			[lindex $var [expr $i+1]] [lindex $var [expr $i+2]]
		    incr i 2
		}
		movefront {$current(frame) marker $tag move front}
		moveback {$current(frame) marker $tag move back}
		property {
		    $current(frame) marker $tag property \
			[lindex $var [expr $i+1]] [lindex $var [expr $i+2]]
		    incr i 2
		}
		select {$current(frame) marker $tag select}
	    }
	}

	copy {MarkerCopy}
	cut {MarkerCut}
	paste {
	    set marker(paste,system) [string tolower [lindex $var [expr $i+1]]] 
	    set marker(paste,sky) [string tolower [lindex $var [expr $i+2]]]
	    MarkerPaste 0
	    incr i 2
	}
	undo {MarkerUndo}

	composite {CompositeCreate}
	desolve -
	dissove {CompositeDelete}

	template {
	    incr i
	    set ff [lindex $var $i]
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		at {
		    incr i
		    set ra [lindex $var $i]
		    incr i
		    set dec [lindex $var $i]
		    incr i
		    set sys [string tolower [lindex $var $i]]
		    incr i
		    set sky [string tolower [lindex $var $i]]
		    switch -- $sys {
			fk4 -
			fk5 -
			icrs -
			galatic -
			ecliptic {
			    set sky $sys
			    set sys wcs
			    incr i -1
			}
		    }
		    LoadTemplateMarkerAt $ff $ra $dec $sys $sky
		    FileLast templatefbox $ff
		}
		default {
		    LoadTemplateMarker $ff
		    FileLast templatefbox $ff
		    incr i -1
		}
	    }
	}
	savetemplate {
	    incr i
	    set ff [lindex $var $i]
	    if {$ff != {}} {
		$current(frame) marker save template "\{$ff\}"
		FileLast templatefbox $ff
	    }
	}

	command {
	    incr i
	    $current(frame) marker command $marker(format) \
		"\{[lindex $var $i]\}"
	    UpdateGroupDialog
	}

	list {
	    if {$current(frame) == {}} {
		return
	    }
	    if {![$current(frame) has fits]} {
		return
	    }
	    SimpleTextDialog markertxt [msgcat::mc {Region}] 80 20 insert top \
		[$current(frame) marker list $marker(format) $marker(system) \
		     $marker(sky) $marker(skyformat) $marker(strip)]
	}
	save {
	    if {$current(frame) == {}} {
		return
	    }
	    if {![$current(frame) has fits]} {
		return
	    }
	    incr i
	    set ff [lindex $var $i]
	    if {$ff == {}} {
		return
	    }
	    $current(frame) marker save "\{$ff\}" \
		$marker(format) $marker(system) $marker(sky) \
		$marker(skyformat) $marker(strip)
	    FileLast markerfbox $ff
	}

	file -
	load {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		all {
		    incr i
		    set frames $ds9(frames)
		}
		default {
		    set frames $current(frame)
		}
	    }
	    MarkerLoadFrames [lindex $var $i] $frames
	}
	default {
	    set format $marker(format)
	    set sys $marker(system)
	    set sky $marker(sky)

	    while {[string range [lindex $var $i] 0 0] == "-"} {
		switch -- [string tolower [lindex $var $i]] {
		    -format {
			incr i
			set format [lindex $var $i]
		    }
		    -sys -
		    -coord -
		    -system {
			incr i
			# for backward compatibility
			switch -- [lindex $var $i] {
			    fk4 -
			    fk5 -
			    icrs -
			    galactic -
			    ecliptic {
				set sys wcs
				set sky [lindex $var $i]
			    }
			    default {
				set sys [lindex $var $i]
			    }
			}
		    }
		    -sky {
			incr i
			set sky [lindex $var $i]
		    }
		    default {
			Error "Illegal option: [lindex $var $i]"
			return
		    }
		}
		incr i
	    }

	    if {$sock != {}} {
		# xpa path
		if {[lindex $var $i] != {}} {
		    MarkerLoadFrames [lindex $var $i] $current(frame)
		} else {
		    # fits regions files not supported  
		    switch -- $format {
			xy {$current(frame) marker load $format $sock $sys $sky}
			default {$current(frame) marker load $format $sock}
		    }
		    UpdateGroupDialog
		}
	    } elseif {$fn != {}} {
		# samp path
		if {[lindex $var $i] != {}} {
		    MarkerLoadFrames [lindex $var $i] $current(frame)
		} else {
		    # fits regions files not supported  
		    switch -- $format {
			xy {$current(frame) marker load $format $fn $sys $sky}
			default {$current(frame) marker load $format $fn}
		    }
		    UpdateGroupDialog
		}
	    } else {
		# this will open a fits regions file
		MarkerLoadFrames [lindex $var $i] $current(frame)
	    }
	}
    }
}

proc ProcessSendRegionsCmd {proc id param sock fn} {
    global current
    global marker

    if {$current(frame) == {}} {
	return
    }

    switch -- [lindex $param 0] {
	show {$proc $id [ToYesNo $marker(show)]}
	showtext {$proc $id [ToYesNo $marker(show,text)]}
	autocentroid {$proc $id [ToYesNo $marker(autocentroid)]}
	format {$proc $id "$marker(format)\n"}
	coord -
	system {$proc $id "$marker(system)\n"}
	sky  {$proc $id "$marker(sky)\n"}
	coordformat -
	skyformat {$proc $id "$marker(skyformat)\n"}
	strip {$proc $id [ToYesNo $marker(strip)]}
	delim {
	    if {$marker(strip)} {
		$proc $id "semicolon\n"
	    } else {
		$proc $id "nl\n"
	    }
	}
	shape {$proc $id "$marker(shape)\n"}
	color {$proc $id "$marker(color)\n"}
	width {$proc $id "$marker(width)\n"}

	tag -
	tags -
	group -
	groups {$proc $id "[lsort [$current(frame) get marker tag all]]\n"}

	default {
	    set format $marker(format)
	    set sys $marker(system)
	    set sky $marker(sky)
	    set skyformat $marker(skyformat)
	    set strip $marker(strip)
	    set select {}
	    set props {}
	    set tags {}

	    set i 0
	    set l [llength $param]
	    while {$i < $l} {
		switch -- [lindex $param $i] {
		    -format {incr i; set format [lindex $param $i]}
		    -sys -
		    -coord -
		    -system {
			incr i
			# for backward compatibility
			switch -- [lindex $param $i] {
			    fk4 -
			    fk5 -
			    icrs -
			    galactic -
			    ecliptic {
				set sys wcs
				set sky [lindex $param $i]
			    }
			    
			    default {set sys [lindex $param $i]}
			}
		    }
		    -sky {incr i; set sky [lindex $param $i]}
		    -coordformat -
		    -skyformat {
			incr i;
			set skyformat [lindex $param $i]
		    }
		    -strip {
			incr i; 
			set strip [FromYesNo [lindex $param $i]]
		    }
		    -delim  {
			incr i;
			if {[lindex $param $i] != "nl"} {
			    set strip 1
			} else {
			    set strip 0
			}
		    }

		    include {append props " include = 1"}
		    exclude {append props " include = 0"}
		    source {append props " source = 1"}
		    background {append props " source = 0"}
		    selected {set select "select"}
		    -prop {
			append props " [lindex $param [expr $i+1]] = [lindex $param [expr $i+2]]"
			incr i 2
		    }
		    -tag -
		    -group {
			incr i
			append tags "tag = \{[lindex $param $i]\}"
		    }
		}
		incr i
	    }

	    switch -- $format {
		xml {set ext {.xml}}
		default {set ext {.rgn}}
	    }
	    ProcessSend $proc $id $sock $fn $ext \
		[$current(frame) marker list $select $format \
		     $sys $sky $skyformat $strip $props $tags]
	}
    }
}
