#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc MarkerLoadDialog {} {
    global marker
    global ed

    set w {.mkd}

    set ed(ok) 0
    set ed(format) $marker(format)
    set ed(system) $marker(system)
    set ed(sky) $marker(sky)
    set ed(skyformat) $marker(skyformat)
    set ed(load) $marker(load)

    DialogCreate $w [msgcat::mc {Load Regions}]

    frame $w.param -relief ridge -bd 1
    frame $w.mode -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.mode $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    # param
    label $w.param.dummy -text {} -width 40

    label $w.param.formattitle -text [msgcat::mc {Format}]
    set m $w.param.formatbutton.menu
    menubutton $w.param.formatbutton -relief raised \
	-textvariable ed(format) -menu $m

    menu $m
    $m add radiobutton -label {DS9/Funtools} -variable ed(format) -value ds9 \
	-command UpdateMarkerLoadDialog
    $m add radiobutton -label {XML} -variable ed(format) -value xml \
	-command UpdateMarkerLoadDialog
    $m add radiobutton -label {CIAO} -variable ed(format) -value ciao \
	-command UpdateMarkerLoadDialog
    $m add radiobutton -label {SAOtng} -variable ed(format) -value saotng \
	-command UpdateMarkerLoadDialog
    $m add radiobutton -label {SAOimage} -variable ed(format) -value saoimage \
	-command UpdateMarkerLoadDialog
    $m add radiobutton -label {IRAF PROS} -variable ed(format) -value pros \
	-command UpdateMarkerLoadDialog
    $m add radiobutton -label {X Y} -variable ed(format) -value xy \
	-command UpdateMarkerLoadDialog

    label $w.param.coordtitle -text [msgcat::mc {Coordinate System}]
    set ed(cb) $w.param.coordbutton
    set ed(mm) $w.param.coordbutton.menu
    menubutton $ed(cb) -relief raised -textvariable ed(system) -menu $ed(mm)

    DefaultCoordMenu $ed(mm) ed(system) ed(sky) ed(skyformat) {}

    grid $w.param.dummy -columnspan 2
    grid $w.param.formattitle $w.param.formatbutton -padx 4 -sticky w
    grid $w.param.coordtitle $w.param.coordbutton -padx 4 -sticky w

    # mode
    radiobutton $w.mode.current -text [msgcat::mc {Load into Current Frame}] \
	-variable ed(load) -value current
    radiobutton $w.mode.all -text [msgcat::mc {Load into All Frames}] \
	-variable ed(load) -value all

    grid $w.mode.current -ipadx 4 -sticky w
    grid $w.mode.all -ipadx 4 -sticky w

    # button
    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    UpdateMarkerLoadDialog

    DialogCenter $w 
    DialogWait $w ed(ok)
    DialogDismiss $w

    if {$ed(ok)} {
	set marker(format) $ed(format)
	set marker(system) $ed(system)
	set marker(sky) $ed(sky)
	set marker(skyformat) $ed(skyformat)
	set marker(load) $ed(load)
    }

    set rr $ed(ok)
    unset ed
    return $rr
}

proc UpdateMarkerLoadDialog {} {
    global ed

    switch -- $ed(format) {
	xy {$ed(cb) configure -state normal}
	default {$ed(cb) configure -state disabled}
    }
}

proc MarkerSaveDialog {title} {
    global marker
    global ed

    set w {.mkd}

    set ed(ok) 0
    set ed(format) $marker(format)
    set ed(system) $marker(system)
    set ed(sky) $marker(sky)
    set ed(skyformat) $marker(skyformat)

    DialogCreate $w $title

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    # param
    label $w.param.dummy -text {} -width 40

    label $w.param.formattitle -text [msgcat::mc {Format}]
    set m $w.param.formatbutton.menu
    menubutton $w.param.formatbutton -relief raised \
	-textvariable ed(format) -menu $m

    menu $m
    $m add radiobutton -label {DS9/Funtools} -variable ed(format) -value ds9 \
	-command UpdateMarkerSaveDialog
    $m add radiobutton -label {XML} -variable ed(format) -value xml \
	-command UpdateMarkerSaveDialog
    $m add radiobutton -label {CIAO} -variable ed(format) -value ciao \
	-command UpdateMarkerSaveDialog
    $m add radiobutton -label {SAOtng} -variable ed(format) -value saotng \
	-command UpdateMarkerSaveDialog
    $m add radiobutton -label {SAOimage} -variable ed(format) -value saoimage \
	-command UpdateMarkerSaveDialog
    $m add radiobutton -label {IRAF PROS} -variable ed(format) -value pros \
	-command UpdateMarkerSaveDialog
    $m add radiobutton -label {X Y} -variable ed(format) -value xy \
	-command UpdateMarkerSaveDialog

    label $w.param.coordtitle -text [msgcat::mc {Coordinate System}]
    set ed(cb) $w.param.coordbutton
    set ed(mm) $w.param.coordbutton.menu
    menubutton $ed(cb) -relief raised -textvariable ed(system) -menu $ed(mm)

    DefaultCoordMenu $ed(mm) ed(system) ed(sky) ed(skyformat) {}

    grid $w.param.dummy -columnspan 2
    grid $w.param.formattitle $w.param.formatbutton -padx 4 -sticky w
    grid $w.param.coordtitle $w.param.coordbutton -padx 4 -sticky w

    # button
    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    UpdateMarkerSaveDialog

    DialogCenter $w 
    DialogWait $w ed(ok)
    DialogDismiss $w

    if {$ed(ok)} {
	set marker(format) $ed(format)
	set marker(system) $ed(system)
	set marker(sky) $ed(sky)
	set marker(skyformat) $ed(skyformat)
    }

    set rr $ed(ok)
    unset ed
    return $rr
}

proc UpdateMarkerSaveDialog {} {
    global ed
    global current
    global ds9

    switch -- $ed(format) {
	ds9 {
	    SetCoordMenu $ed(mm) normal
	    UpdateCoordMenu "$current(frame)" $ed(mm)
	    AdjustCoord "$current(frame)" ed(system)
	}

	xml {
	    SetCoordMenu $ed(mm) normal
	    UpdateCoordMenu "$current(frame)" $ed(mm)
	    AdjustCoord "$current(frame)" ed(system)
	}

	ciao {
	    SetCoordMenu $ed(mm) normal
	    UpdateCoordMenu "$current(frame)" $ed(mm)

	    switch -- $ed(system) {
		detector -
		amplifier -
		physical -
		image {set ed(system) physical}
		wcs -
		default {set ed(system) wcs}
	    }
	    AdjustCoord "$current(frame)" ed(system)
	    set ed(sky) fk5
	    set ed(skyformat) sexagesimal

	    $ed(mm) entryconfig [msgcat::mc {Multiple WCS}] -state disabled
	    $ed(mm) entryconfig [msgcat::mc {Image}] -state disabled
	    $ed(mm) entryconfig [msgcat::mc {Physical}] -state normal
	    $ed(mm) entryconfig [msgcat::mc {Amplifier}] -state disabled
	    $ed(mm) entryconfig [msgcat::mc {Detector}] -state disabled

	    $ed(mm) entryconfig "[msgcat::mc {Equatorial}] B1950" \
		-state disabled
	    $ed(mm) entryconfig "[msgcat::mc {Equatorial}] J2000" \
		-state normal
	    $ed(mm) entryconfig [msgcat::mc {ICRS}] -state disabled
	    $ed(mm) entryconfig [msgcat::mc {Galactic}] -state disabled
	    $ed(mm) entryconfig [msgcat::mc {Ecliptic}] -state disabled

	    $ed(mm) entryconfig [msgcat::mc {Degrees}] -state disabled
	    $ed(mm) entryconfig {Sexagesimal} -state normal
	}

	saotng {
	    SetCoordMenu $ed(mm) normal
	    UpdateCoordMenu "$current(frame)" $ed(mm)

	    switch -- $ed(system) {
		detector -
		amplifier -
		image -
		physical {set ed(system) image}
		wcs -
		default {set ed(system) wcs}
	    }
	    AdjustCoord "$current(frame)" ed(system)

	    $ed(mm) entryconfig [msgcat::mc {Multiple WCS}] -state disabled
	    $ed(mm) entryconfig [msgcat::mc {Physical}] -state disabled
	    $ed(mm) entryconfig [msgcat::mc {Amplifier}] -state disabled
	    $ed(mm) entryconfig [msgcat::mc {Detector}] -state disabled
	}

	saoimage {
	    SetCoordMenu $ed(mm) disabled
	    set ed(system) image
	    $ed(mm) entryconfig [msgcat::mc {Image}] -state normal
	}

	pros {
	    SetCoordMenu $ed(mm) normal
	    UpdateCoordMenu "$current(frame)" $ed(mm)

	    switch -- $ed(system) {
		detector -
		physical -
		amplifier {set ed(system) physical}
		image {}
		wcs -
		default {set ed(system) wcs}
	    }
	    AdjustCoord "$current(frame)" ed(system)
	    if {$ed(sky) == "icrs"} {
		set ed(sky) fk5
	    }

	    $ed(mm) entryconfig [msgcat::mc {Multiple WCS}] -state disabled
	    $ed(mm) entryconfig [msgcat::mc {ICRS}] -state disabled
	    $ed(mm) entryconfig [msgcat::mc {Amplifier}] -state disabled
	    $ed(mm) entryconfig [msgcat::mc {Detector}] -state disabled
	}
	
	xy {
	    SetCoordMenu $ed(mm) normal
	    UpdateCoordMenu "$current(frame)" $ed(mm)
	    AdjustCoord "$current(frame)" ed(system)
	}
    }
}


