#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

# Menus

proc FileMenu {} {
    global ds9

    menu $ds9(mb).file 
    switch $ds9(wm) {
	x11 -
	win32 {
	    $ds9(mb).file add command \
		-label "[msgcat::mc {About SAOImage DS9}]..." \
		-command AboutBox
	    $ds9(mb).file add separator
	}
	aqua {}
    }
    $ds9(mb).file add command -label "[msgcat::mc {Open}]..." \
	-command OpenFits -accelerator "${ds9(ctrl)}O"
    $ds9(mb).file add cascade -label [msgcat::mc {Open Other}] \
	-menu $ds9(mb).file.open
    $ds9(mb).file add separator
    $ds9(mb).file add cascade -label [msgcat::mc {Preserve During Load}] \
	-menu $ds9(mb).file.preserve
    $ds9(mb).file add separator
    $ds9(mb).file add command -label "[msgcat::mc {Save Image}]..." \
	-command SaveImageDialog
    $ds9(mb).file add command -label "[msgcat::mc {Save Frame as Fits}]..." \
	-command SaveFitsDialog
    $ds9(mb).file add command -label "[msgcat::mc {Save Frames as MPEG}]..." \
	-command SaveMPEGDialog
    $ds9(mb).file add separator
    $ds9(mb).file add command -label "[msgcat::mc {Display Fits Header}]..." \
	-command DisplayHeaderMenu 
    $ds9(mb).file add separator
    $ds9(mb).file add cascade -label [msgcat::mc {XPA}] \
	-menu $ds9(mb).file.xpa
    $ds9(mb).file add cascade -label [msgcat::mc {SAMP}] \
	-menu $ds9(mb).file.samp
    $ds9(mb).file add separator
    $ds9(mb).file add command -label "[msgcat::mc {Open TCL Console}]..." \
	-command OpenConsole
    $ds9(mb).file add command -label "[msgcat::mc {Source TCL}]..." \
	-command OpenSource
    $ds9(mb).file add separator
    switch $ds9(wm) {
	x11 {
	    $ds9(mb).file add command -label "[msgcat::mc {Page Setup}]..." \
		-command PageSetup -accelerator "${ds9(shiftctrl)}P"
	    $ds9(mb).file add command -label "[msgcat::mc {Print}]..." \
		-command Print -accelerator "${ds9(ctrl)}P"
	}
	win32 -
	aqua {
	    $ds9(mb).file add command \
		-label "[msgcat::mc {Postscript Page Setup}]..." \
		-command PSPageSetup
	    $ds9(mb).file add command \
		-label "[msgcat::mc {Postscript Print}]..." \
		-command PSPrint
	    $ds9(mb).file add separator
	    $ds9(mb).file add command -label "[msgcat::mc {Page Setup}]..." \
		-command PageSetup -accelerator "${ds9(shiftctrl)}P"
	    $ds9(mb).file add command -label "[msgcat::mc {Print}]..." \
		-command Print -accelerator "${ds9(ctrl)}P"
	}
    }
    switch $ds9(wm) {
	x11 -
	win32 {
	    $ds9(mb).file add separator
	    $ds9(mb).file add command \
		-label [msgcat::mc {Exit}] -command QuitDS9
	}
	aqua {}
    }

    # File Open Menu
    menu $ds9(mb).file.open
    $ds9(mb).file.open add command -label "[msgcat::mc {Open Array}]..." \
	-command OpenArray
    $ds9(mb).file.open add command -label "[msgcat::mc {Open URL}]..." \
	-command OpenURL
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open RGB Fits Image}]..." \
	-command OpenRGBImageFits
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open RGB Fits Cube}]..."\
	-command OpenRGBCubeFits
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open RGB Array}]..." \
	-command OpenRGBArray
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open Multi Ext Data Cube}]..." \
	-command OpenDataCubeFits
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open Multi Ext Multi Frames}]..." \
	-command {OpenMultiFrameFits}
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open Mosaic IRAF}]..." \
	-command OpenMosaicImageIRAFFits
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open Mosaic IRAF Segment}]..." \
	-command OpenMosaicIRAFFits
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open Mosaic WCS}]..." \
	-command OpenMosaicImageWCSFits
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open Mosaic WCS Segment}]..." \
	-command OpenMosaicWCSFits
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open Mosaic WFPC2}]..." \
	-command OpenMosaicImageWFPC2Fits

    # File Preserve Menu
    menu $ds9(mb).file.preserve
    $ds9(mb).file.preserve add checkbutton -label [msgcat::mc {Scale}] \
	-variable scale(preserve) -command PreserveScale
    $ds9(mb).file.preserve add checkbutton -label [msgcat::mc {Pan}] \
	-variable panzoom(preserve) -command PreservePan
    $ds9(mb).file.preserve add checkbutton -label [msgcat::mc {Region}] \
	-variable marker(preserve) -command MarkerPreserve

    SAMPMenu $ds9(mb).file.samp image

    menu $ds9(mb).file.xpa
    $ds9(mb).file.xpa add command -label [msgcat::mc {Information}] \
	-command XPAInfo
    $ds9(mb).file.xpa add separator
    $ds9(mb).file.xpa add command -label [msgcat::mc {Connect}] \
	-command XPAConnect
    $ds9(mb).file.xpa add command -label [msgcat::mc {Disconnect}] \
	-command XPADisconnect

    # Bindings
    bind $ds9(top) <<Open>> OpenFits
    bind $ds9(top) <<PageSetup>> PageSetup
    bind $ds9(top) <<Print>> Print
}

proc PrefsDialogFile {} {
    global prefs

    set w $prefs(tabs)

    frame $w.file
    $prefs(tabs) tab configure file -window $w.file -fill both

    frame $w.file.menu -relief ridge -bd 1
    pack $w.file.menu -side top -fill both -expand true

    # Buttons
    PrefsDialogButtons $w.file

    # Default Menu
    set f $w.file.menu.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Default}]
    menubutton $f.menu -relief raised -text [msgcat::mc {Menu}] \
	-menu $f.menu.menu
    PrefsDialogButtonbarFile $f.buttonbar

    grid $f.title $f.menu $f.buttonbar -padx 4 -sticky w

    set m $f.menu.menu
    menu $m
    $m add cascade -label [msgcat::mc {Preserve During Load}] \
	-menu $m.preserve

    global pscale
    global ppanzoom
    global pmarker
    menu $m.preserve
    $m.preserve add checkbutton -label [msgcat::mc {Scale}] \
	-variable pscale(preserve)
    $m.preserve add checkbutton -label [msgcat::mc {Pan}] \
	-variable ppanzoom(preserve)
    $m.preserve add checkbutton -label [msgcat::mc {Region}] \
	-variable pmarker(preserve)
}

# Buttons

proc ButtonsFileDef {} {
    global pbuttons

    array set pbuttons {
	file,about 1
	file,open 1
	file,saveimage 1
	file,savefits 0
	file,savempeg 0
	file,header 1
	file,samp 1
	file,xpa 0
	file,console 0
	file,tcl 0
	file,pspage 0
	file,psprint 0
	file,page 1
	file,print 1
	file,exit 1
    }
}

proc CreateButtonsFile {} {
    global ds9
    global buttons

    frame $ds9(buttons).file

    ButtonButton $ds9(buttons).file.about \
	[string tolower [msgcat::mc {About}]] AboutBox
    ButtonButton $ds9(buttons).file.open \
	[string tolower [msgcat::mc {Open}]] OpenFits

    ButtonButton $ds9(buttons).file.saveimage \
	[string tolower [msgcat::mc {Save Image}]] SaveImageDialog
    ButtonButton $ds9(buttons).file.savefits \
	[string tolower [msgcat::mc {Save FITS}]] SaveFitsDialog
    ButtonButton $ds9(buttons).file.savempeg \
	[string tolower [msgcat::mc {Save MPEG}]] SaveMPEGDialog

    ButtonButton $ds9(buttons).file.header \
	[string tolower [msgcat::mc {Header}]] DisplayHeaderMenu

    ButtonButton $ds9(buttons).file.samp \
	[string tolower [msgcat::mc {SAMP}]] "SAMPSendImageLoadFits {}"

    ButtonButton $ds9(buttons).file.xpa \
	[string tolower {XPA}] XPAInfo
    ButtonButton $ds9(buttons).file.console \
	[string tolower [msgcat::mc {Console}]] OpenConsole
    ButtonButton $ds9(buttons).file.tcl \
	[string tolower {TCL}] OpenSource

    ButtonButton $ds9(buttons).file.pspage \
	[string tolower [msgcat::mc {PS Page Setup}]] PSPageSetup
    ButtonButton $ds9(buttons).file.psprint \
	[string tolower [msgcat::mc {PS Print}]] PSPrint

    ButtonButton $ds9(buttons).file.page \
	[string tolower [msgcat::mc {Page Setup}]] PageSetup
    ButtonButton $ds9(buttons).file.print \
	[string tolower [msgcat::mc {Print}]] Print

    ButtonButton $ds9(buttons).file.exit \
	[string tolower [msgcat::mc {Exit}]] QuitDS9

    set buttons(file) "
        $ds9(buttons).file.about pbuttons(file,about)
        $ds9(buttons).file.open pbuttons(file,open)
        $ds9(buttons).file.saveimage pbuttons(file,saveimage)
        $ds9(buttons).file.savefits pbuttons(file,savefits)
        $ds9(buttons).file.savempeg pbuttons(file,savempeg)
        $ds9(buttons).file.samp pbuttons(file,samp)
        $ds9(buttons).file.header pbuttons(file,header)
        $ds9(buttons).file.xpa pbuttons(file,xpa)
        $ds9(buttons).file.console pbuttons(file,console)
        $ds9(buttons).file.tcl pbuttons(file,tcl)
        $ds9(buttons).file.pspage pbuttons(file,pspage)
        $ds9(buttons).file.psprint pbuttons(file,psprint)
        $ds9(buttons).file.page pbuttons(file,page)
        $ds9(buttons).file.print pbuttons(file,print)
        $ds9(buttons).file.exit pbuttons(file,exit)
    "
}

proc PrefsDialogButtonbarFile {f} {
    global buttons
    global pbuttons

    menubutton $f -relief raised -text [msgcat::mc {Buttonbar}] -menu $f.menu
    
    set m $f.menu
    menu $m
    $m add checkbutton -label "[msgcat::mc {About SAOImage DS9}]..." \
	-variable pbuttons(file,about) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label "[msgcat::mc {Open}]..." \
	-variable pbuttons(file,open) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label "[msgcat::mc {Save Image}]..." \
	-variable pbuttons(file,saveimage) -command {UpdateButtons buttons(file)}
    $m add checkbutton -label "[msgcat::mc {Save Frame as Fits}]..." \
	-variable pbuttons(file,savefits) -command {UpdateButtons buttons(file)}
    $m add checkbutton -label "[msgcat::mc {Save Frames as MPEG}]..." \
	-variable pbuttons(file,savempeg) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label "[msgcat::mc {Display Fits Header}]..." \
	-variable pbuttons(file,header) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label [msgcat::mc {SAMP}] \
	-variable pbuttons(file,samp) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label "[msgcat::mc {XPA Information}]..." \
	-variable pbuttons(file,xpa) -command {UpdateButtons buttons(file)}
    $m add checkbutton -label "[msgcat::mc {Open TCL Console}]..." \
	-variable pbuttons(file,console) -command {UpdateButtons buttons(file)}
    $m add checkbutton -label "[msgcat::mc {Source TCL}]..." \
	-variable pbuttons(file,tcl) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label "[msgcat::mc {Postscript Page Setup}]..." \
	-variable pbuttons(file,pspage) -command {UpdateButtons buttons(file)}
    $m add checkbutton -label "[msgcat::mc {Postscript Print}]..." \
	-variable pbuttons(file,psprint) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label "[msgcat::mc {Page Setup}]..." \
	-variable pbuttons(file,page) -command {UpdateButtons buttons(file)}
    $m add checkbutton -label "[msgcat::mc {Print}]..." \
	-variable pbuttons(file,print) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label [msgcat::mc {Exit}] \
	-variable pbuttons(file,exit) -command {UpdateButtons buttons(filew)}
}

# Support

proc UpdateFileMenuStatic {} {
    global ds9
 
    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateFileMenuStatic"
    }

    if {$ds9(active,num) > 0} {
	$ds9(mb).file entryconfig "[msgcat::mc {Open}]..." \
	    -state normal
	$ds9(mb).file entryconfig [msgcat::mc {Open Other}] \
	    -state normal
	$ds9(mb).file entryconfig "[msgcat::mc {Save Image}]..." \
	    -state normal
	$ds9(mb).file entryconfig "[msgcat::mc {Save Frame as Fits}]..." \
	    -state normal
	$ds9(mb).file entryconfig "[msgcat::mc {Save Frames as MPEG}]..." \
	    -state normal

	$ds9(buttons).file.open configure -state normal
	$ds9(buttons).file.saveimage configure -state normal
	$ds9(buttons).file.savefits configure -state normal
	$ds9(buttons).file.savempeg configure -state normal
    } else {
	$ds9(mb).file entryconfig "[msgcat::mc {Open}]..." \
	    -state disabled
	$ds9(mb).file entryconfig [msgcat::mc {Open Other}] \
	    -state disabled
	$ds9(mb).file entryconfig "[msgcat::mc {Save Image}]..." \
	    -state disabled
	$ds9(mb).file entryconfig "[msgcat::mc {Save Frame as Fits}]..." \
	    -state disabled
	$ds9(mb).file entryconfig "[msgcat::mc {Save Frames as MPEG}]..." \
	    -state disabled

	$ds9(buttons).file.open configure -state disabled
	$ds9(buttons).file.saveimage configure -state disabled
	$ds9(buttons).file.savefits configure -state disabled
	$ds9(buttons).file.savempeg configure -state disabled
    }
}

proc UpdateFileMenu {} {
    global ds9
    global current
    global xpa
 
    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateFileMenu"
    }

    if {$current(frame) != {}} {
	if {[$current(frame) has fits]} {
	    $ds9(mb).file entryconfig "[msgcat::mc {Display Fits Header}]..." \
		-state normal
	} else {
	    $ds9(mb).file entryconfig "[msgcat::mc {Display Fits Header}]..." \
		-state disabled
	}

	switch -- [$current(frame) get type] {
	    base -
	    3d {
		$ds9(mb).file.open entryconfig \
		    "[msgcat::mc {Open RGB Fits Image}]..." -state disabled
		$ds9(mb).file.open entryconfig \
		    "[msgcat::mc {Open RGB Fits Cube}]..." -state disabled
		$ds9(mb).file.open entryconfig \
		    "[msgcat::mc {Open RGB Array}]..." -state disabled
	    }
	    rgb {
		$ds9(mb).file.open entryconfig \
		    "[msgcat::mc {Open RGB Fits Image}]..." -state normal
		$ds9(mb).file.open entryconfig \
		    "[msgcat::mc {Open RGB Fits Cube}]..." -state normal
		$ds9(mb).file.open entryconfig \
		    "[msgcat::mc {Open RGB Array}]..." -state normal
	    }
	}

	if {[$current(frame) has fits]} {
	    $ds9(buttons).file.header configure -state normal
	} else {
	    $ds9(buttons).file.header configure -state disabled
	}

    } else {
	$ds9(mb).file entryconfig "[msgcat::mc {Display Fits Header}]..." \
	    -state disabled

	$ds9(buttons).file.header configure -state disabled
    }

    if [info exists xpa] {
	$ds9(mb).file.xpa entryconfig [msgcat::mc {Information}] -state normal
	$ds9(mb).file.xpa entryconfig [msgcat::mc {Disconnect}] -state normal
    } else {
	$ds9(mb).file.xpa entryconfig [msgcat::mc {Information}] -state disabled
	$ds9(mb).file.xpa entryconfig [msgcat::mc {Disconnect}] -state disabled
    }

    SAMPUpdateMenu $ds9(mb).file.samp image
    SAMPUpdateButton $ds9(mb).file.samp image
}
